package com.cloudbility.common.util;

import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.*;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Security;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;

public class SensitivePropertyUtil {

    private static String KEY_ALGORITHM = "PBKDF2WithHmacSHA1";
    private static byte[] salt = {(byte) 0xc0, (byte) 0x84, (byte) 0x13, (byte) 0x5f,
            (byte) 0x08, (byte) 0x17, (byte) 0xdd, (byte) 0xff};
    private static final String CIPHER_ALGORITHM = "DES/ECB/PKCS5Padding";

    static {
        Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
    }

    /**
     * 加密
     *
     * @param key
     * @param rawData
     * @return
     * @throws NoSuchPaddingException
     * @throws NoSuchAlgorithmException
     * @throws NoSuchProviderException
     * @throws InvalidKeyException
     * @throws BadPaddingException
     * @throws IllegalBlockSizeException
     * @throws UnsupportedEncodingException
     */
    public static String encrypt(String key, String rawData)
            throws NoSuchPaddingException, NoSuchAlgorithmException, NoSuchProviderException, InvalidKeyException,
            BadPaddingException, IllegalBlockSizeException, UnsupportedEncodingException {

        Cipher cipher = Cipher.getInstance(CIPHER_ALGORITHM, BouncyCastleProvider.PROVIDER_NAME);
        cipher.init(Cipher.ENCRYPT_MODE, getSecretKeySpec(key));
        return new String(Base64.encodeBase64(cipher.doFinal(rawData.getBytes())), "UTF-8");
    }

    /**
     * 解密
     *
     * @param key
     * @param secData
     * @return
     * @throws NoSuchPaddingException
     * @throws NoSuchAlgorithmException
     * @throws NoSuchProviderException
     * @throws InvalidKeyException
     * @throws UnsupportedEncodingException
     * @throws BadPaddingException
     * @throws IllegalBlockSizeException
     */
    public static String decrypt(String key, String secData)
            throws NoSuchPaddingException, NoSuchAlgorithmException, NoSuchProviderException, InvalidKeyException,
            UnsupportedEncodingException, BadPaddingException, IllegalBlockSizeException {

        Cipher cipher = Cipher.getInstance(CIPHER_ALGORITHM, BouncyCastleProvider.PROVIDER_NAME);
        cipher.init(Cipher.DECRYPT_MODE, getSecretKeySpec(key));
        return new String(cipher.doFinal(Base64.decodeBase64(secData.getBytes("UTF-8"))));

    }

    private static SecretKeySpec getSecretKeySpec(String key) {
        if (null == key) {
            throw new RuntimeException("密钥长度非法");
        }
        try {
            SecretKeyFactory factory = SecretKeyFactory.getInstance(KEY_ALGORITHM);
            KeySpec spec = new PBEKeySpec(key.toCharArray(), salt, 2, 64);
            SecretKey tmp = factory.generateSecret(spec);
            return new SecretKeySpec(tmp.getEncoded(), "DES");
        } catch (InvalidKeySpecException | NoSuchAlgorithmException e) {
            throw new RuntimeException("无法初始化DES密钥", e);
        }
    }


    public static void main(String[] args) throws Exception {
        String key = "WbigvYK7j5duOokvOwG2IZIvb";
        String raw = "";
        // encrypt it
        String sec = SensitivePropertyUtil.encrypt(key, raw);
        System.out.println("encrypt: " + sec);
        // decrypt it
        String decryptSec = SensitivePropertyUtil.decrypt(key, sec);
        System.out.println("decrypt: " + decryptSec);

        if (raw.equals(decryptSec)) {
            System.out.println("correct");
        } else {
            System.out.println("wrong");
        }
    }
}
