package com.cloudbility.openapi;

import com.cloudbility.openapi.vo.AkAccessToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AutoRefreshTokenProvider implements TokenProvider {

    private Logger logger = LoggerFactory.getLogger(AutoRefreshTokenProvider.class);

    /**
     * 一个认证的client，用于刷新token
     */
    private OpenapiClient openapiClient;

    private volatile AkAccessToken token;

    /**
     * 如果当前token将在expiringGap毫秒后过期，则刷新token
     */
    private int expiringGap = 60 * 1000;

    private int expireSeconds;

    /**
     *
     */
    public AutoRefreshTokenProvider(int expireSeconds) {
        this.expiringGap = Math.min(expiringGap, 60 * 1000 * expireSeconds / 3);
        this.expireSeconds = expireSeconds;
    }

    public AutoRefreshTokenProvider(AkAccessToken token, int expireSeconds) {
        this(expireSeconds);
        this.token = token;
    }

    @Override
    public String getToken() {
        if (needRefresh()) {
            logger.debug("refreshing token...");
            doRefresh();
        }
        AkAccessToken token = this.token;
        if (token != null) {
            return token.getToken();
        }
        return null;
    }

    protected void doRefresh() {
        if (openapiClient == null) {
            throw new NullPointerException("openapiClient");
        }
        try {
            this.token = openapiClient.otherApi().getOAuth(expireSeconds);
        } catch (OpenapiException e) {
            logger.error("refresh token error", e);
        }
    }

    protected boolean needRefresh() {
        return token == null || token.getExpireTime().getTime() - expiringGap < System.currentTimeMillis();
    }

    public OpenapiClient getOpenapiClient() {
        return openapiClient;
    }

    public void setOpenapiClient(OpenapiClient openapiClient) {
        this.openapiClient = openapiClient;
    }

    public void setToken(AkAccessToken token) {
        this.token = token;
    }
}
