package com.cloudbility.openapi;

import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;

import java.util.List;
import java.util.Map;

public interface CloudApi {

    /**
     * 获取云账户信息
     *
     * @param cloudId
     * @return
     * @throws OpenapiException
     */
    CloudVo getCloudById(Long cloudId) throws OpenapiException;

    /**
     * 获取云账户详情
     *
     * @param cloudId
     * @param scope
     * @return
     * @throws OpenapiException
     */
    CloudVo getCloudDetail(Long cloudId, Scope scope) throws OpenapiException;

    /**
     * 根据云厂商获取区域
     *
     * @param provider
     * @return
     * @throws OpenapiException
     */
    GetRegionResult getRegionByProvider(CloudProvider provider) throws OpenapiException;

    /**
     * 根据云账户获取区域
     *
     * @param cloudId
     * @return
     * @throws OpenapiException
     */
    GetRegionResult getRegionByCloud(Long cloudId) throws OpenapiException;

    GetProjectResult getProjectsByCloud(Long cloudId, String regionId) throws OpenapiException;

    /**
     * 列出区域下可以售卖的实例规格
     *
     * @param cloudId            云账户ID，必填
     * @param regionId           区域ID，必填
     * @param zoneId             可用区ID，可选
     * @param instanceChargeType 实例收费类型，后付费或者预付费，必填
     * @return
     * @throws OpenapiException
     */
    InstanceTypeResult listAvailableInstanceTypes(Long cloudId, String regionId, String zoneId, CloudInstanceChargeType instanceChargeType) throws OpenapiException;

    /**
     * 列出区域下可以售卖的磁盘规格
     *
     * @param cloudId      云账户ID，必填
     * @param regionId     区域ID，必填
     * @param zoneId       可用区ID，可选
     * @param instanceType 实例类型，阿里云必填
     * @return
     * @throws OpenapiException
     */
    DiskTypeResult listAvailableDiskTypes(Long cloudId, String regionId, String zoneId, String instanceType) throws OpenapiException;


    ListCloudsResult getCloudByTeam(long teamId) throws OpenapiException;

    ListCloudsResult getCloudByTeamCode(String teamCode) throws OpenapiException;

    GetProxyScriptResult getProxyScript(long cloudId) throws OpenapiException;

    void revokeAllRolesFromCloud(long cloudId) throws OpenapiException;

    /**
     * NOT tested
     * 创建云账户
     *
     * @param cloudVos
     * @return
     * @throws OpenapiException
     */
    @NotTested
    CreateCloudResult createClouds(List<CreateCloudVo> cloudVos) throws OpenapiException;

    CreateCloudResult createCloud(CreateCloudVo cloudVo) throws OpenapiException;

    @NotTested
    CloudVo createAzure(long teamId, String name, Long creatorId, String roles, String subscriptionId, String tenantId, String clientId, String clientSecret) throws OpenapiException;

    @NotTested
    CloudVo createOpenstack(long teamId, String name, Long creatorId, String roles, String authUrl, String username, String password, String domainName, String projectId) throws OpenapiException;

    CloudVo createPublic(long teamId, String name, Long creatorId, String roles, String accessKeyId, String accessKeySecret, CloudProvider provider) throws OpenapiException;

    @Deprecated
    void updateNameOrRoles(long cloudId, String name, String roles) throws OpenapiException;

    void updateRoles(long cloudId, String roles) throws OpenapiException;

    void updateDepartments(long cloudId, String departments) throws OpenapiException;

    void updateMembers(long cloudId, String members) throws OpenapiException;

    void deleteClouds(List<Long> cloudIds) throws OpenapiException;

    void updateIntranetAccessType(long cloudId, CloudIntranetAccessType cloudIntranetAccessType) throws OpenapiException;

    Long updateLanCloudCidrRule(long cloudId, CloudCidrRule cidrRule, String cidrs) throws OpenapiException;

}
