package com.cloudbility.openapi;

import com.cloudbility.openapi.enums.zstack.ImageStatusType;
import com.cloudbility.openapi.response.CloudImageListResult;
import com.cloudbility.openapi.vo.ImagePermissionType;
import com.cloudbility.openapi.vo.Scope;

/**
 * Created by lidonghao on 2020/8/6.
 */
public interface ImageApi {

    /**
     * 共享镜像
     * @param vendor 供应商
     * @param regionId 共享者的镜像所在的区域
     * @param imageId  共享者的镜像ID
     * @param cloudId  被共享者的云账户ID
     * @param accountId  被共享者的账号ID
     * @param projectId  被共享者的项目ID,华为云需要用到该参数
     * @throws OpenapiException
     */
    void shareImage(String vendor, String regionId, String imageId, Long cloudId, String accountId, String projectId) throws OpenapiException;

    /**
     * 取消共享镜像
     * @param vendor 供应商
     * @param regionId 共享者的镜像所在的区域
     * @param imageId  共享者的镜像ID
     * @param cloudId  被共享者的云账户ID
     * @param accountId  被共享者的账号ID
     * @param projectId  被共享者的项目ID,华为云需要用到该参数
     * @throws OpenapiException
     */
    void cancelShareImage(String vendor, String regionId, String imageId, Long cloudId, String accountId, String projectId) throws OpenapiException;

    CloudImageListResult listZStackImages(Long cloudId, Scope scope, String imageId, ImageStatusType status) throws OpenapiException;

    /**
     * 修改镜像权限,共享或者取消共享
     * @param vendor 供应商
     * @param provider 云厂商
     * @param regionId 共享者的镜像所在的区域
     * @param imageId  共享者的镜像ID
     * @param accountId  被共享者的账号ID
     * @param projectId  被共享者的项目ID,华为云需要用到该参数
     * @param type 类型,SHARE|CANCEL
     * @throws OpenapiException
     */
    void modifyImageSharePermission(String vendor, String provider, String regionId, String imageId, String accountId, String projectId, ImagePermissionType type) throws OpenapiException;

}
