package com.cloudbility.openapi;

import com.cloudbility.openapi.enums.zstack.*;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import org.zstack.sdk.*;

import java.util.List;
import java.util.Set;

/**
 * Created by lidonghao on 2020/8/20.
 */
public interface NetworkApi {

    /**
     * 创建安全组
     *
     * @param cloudId            云账户ID
     * @param regionId           区域ID
     * @param vpcId              VPC ID， 阿里云和华为云需要该参数
     * @param securityGroupName  安全组名称
     * @param description        安全组描述
     * @param securityGroupRules 安全组规则
     * @return
     * @throws OpenapiException
     */
    SecurityGroupVo createSecurityGroup(Long cloudId, String regionId, String vpcId, String securityGroupName, String description, List<SecurityGroupRuleVo> securityGroupRules) throws OpenapiException;

    /**
     * 列出安全组
     *
     * @param cloudId
     * @param regionId
     * @param vpcId
     * @return
     * @throws OpenapiException
     */
    SecurityGroupResult listSecurityGroups(Long cloudId, String regionId, String vpcId) throws OpenapiException;

    /**
     * 列出安全组规则
     *
     * @param cloudId
     * @param regionId
     * @param securityGroupId
     * @return
     * @throws OpenapiException
     */
    SecurityGroupRuleResult listSecurityGroupRules(Long cloudId, String regionId, String securityGroupId) throws OpenapiException;

    /**
     * 把主机加入到安全组中
     *
     * @param cloudId         云账户ID
     * @param hostId          主机ID
     * @param securityGroupId 安全组ID
     * @throws OpenapiException
     */
    void joinHostToSecurityGroup(Long cloudId, Long hostId, String securityGroupId) throws OpenapiException;

    /**
     * 从安全组中移除主机
     *
     * @param cloudId         云账户ID
     * @param hostId          主机ID
     * @param securityGroupId 安全组ID
     * @throws OpenapiException
     */
    void removeHostFromSecurityGroup(Long cloudId, Long hostId, String securityGroupId) throws OpenapiException;

    /**
     * 创建子网
     *
     * @param cloudId      云账户ID
     * @param regionId     区域ID
     * @param zoneId       可用区ID
     * @param instanceType 实例规格
     * @param cidrBlock    CIDR格式，如192.168.6.8/24
     * @param vpcId        VPC ID
     * @param subnetName   子网名称
     * @param description  描述
     * @return
     * @throws OpenapiException
     */
    SubnetVo createSubnet(Long cloudId, String regionId, String zoneId, String instanceType, String cidrBlock, String vpcId, String subnetName, String description) throws OpenapiException;

    /**
     * 查询云账户的所有VPC
     *
     * @param cloudId
     * @param regionId
     * @param zoneId
     * @param projectId
     * @return
     * @throws OpenapiException
     */
    ListVpcsResult getVpcsByCloud(Long cloudId, String regionId, String zoneId, String projectId) throws OpenapiException;

    /**
     * 创建zstackVxlan池,并加入集群
     *
     * @param cloudId
     * @param name
     * @param description
     * @param startVni
     * @param endVni
     * @param clusterId
     * @param cidr
     * @return
     * @throws OpenapiException
     */
    CreateZStackL2VxLanPoolResult createZStackL2VxLanPool(String clientToken, Long cloudId, Scope scope, String name, String description, Integer startVni, Integer endVni, String clusterId, String cidr) throws OpenapiException;

    CreateZStackL2NoVLanNetworkResult createZStackL2NoVLanNetwork(String clientToken,
                                                                  Long cloudId,
                                                                  Scope scope,
                                                                  String name,
                                                                  String physicalInterface,
                                                                  String description,
                                                                  String clusterId) throws OpenapiException;

    CreateZStackL2VLanNetworkResult createZStackL2VLanNetwork(String clientToken,
                                                              Long cloudId,
                                                              Scope scope,
                                                              String name,
                                                              Integer vLanId,
                                                              String physicalInterface,
                                                              String description,
                                                              String clusterId) throws OpenapiException;

    CreateZStackL2VxLanNetworkResult createZStackL2VxLanNetwork(String clientToken,
                                                                Long cloudId,
                                                                Scope scope,
                                                                String name,
                                                                String vxLanPoolId,
                                                                String description,
                                                                Integer vni) throws OpenapiException;

    L2NetworkInventory attachZStackL2NetworkToCluster(String clientToken,
                                                      Long cloudId,
                                                      Scope scope,
                                                      String networkId,
                                                      String clusterId,
                                                      String cidr) throws OpenapiException;

    ListZStackL2VxLanPoolsResult listZStackL2VxLanPools(Long cloudId, Scope scope,Set<String> networkPoolIds) throws OpenapiException;

    ListZStackL2VLanNetworksResult listZStackL2VLanNetworks(Long cloudId, Scope scope,Set<String> networkIds, String physicalInterface) throws OpenapiException;

    ListZStackL2VxLanNetworksResult listZStackL2VxLanNetworks(Long cloudId, Scope scope,Set<String> networkIds) throws OpenapiException;

    ListZStackL2NetworksResult listZStackL2Networks(Long cloudId,
                                                    Scope scope,
                                                    Set<String> networkIds,
                                                    String physicalInterface,
                                                    String startIp) throws OpenapiException;

    void deleteZStackL2Network(String clientToken, Long cloudId, Scope scope,String networkId, DeleteModeType deleteMode) throws OpenapiException;

    ListZStackL3NetworksResult listZStackL3Networks(Long cloudId, Scope scope,Set<String> l3NetworkIds, String name, String l2NetworkId, UsingStateType usingState, L3NetworkType networkType) throws OpenapiException;

    void deleteZStackL3Network(String clientToken, Long cloudId, Scope scope,Set<String> networkIds, DeleteModeType deleteMode) throws OpenapiException;

    void deleteZStackVirtualNetwork(String clientToken, Long cloudId, Scope scope,String l3NetworkId, Boolean safeDeleteL2Network) throws OpenapiException;

    ListZStackNetworkServiceProvidersResult listZStackNetworkServiceProviders(Long cloudId,Scope scope) throws OpenapiException;

    ListZStackIpRangesResult listZStackIpRanges(Long cloudId, Scope scope,Set<String> ipRangeIds, String l2networkId, String l3networkId) throws OpenapiException;

    GetIpAddressCapacityResult getZStackIpAddressCapacity(Long cloudId, Scope scope,Set<String> l3networkIds, Set<String> ipRangeIds) throws OpenapiException;

    L3NetworkInventory addZStackDnsToL3Network(String clientToken, Long cloudId, Scope scope,String dns, String l3networkId) throws OpenapiException;

    L3NetworkInventory removeZStackDnsFromL3Network(String clientToken, Long cloudId,Scope scope, String dns, String l3networkId) throws OpenapiException;

    IpRangeInventory addZStackIpRangeToL3Network(String clientToken, Long cloudId, Scope scope,String l3networkId, String name,
                                                 String startIp, String endIp, String netmask, String gateway, String dhcpIp, String description) throws OpenapiException;

    IpRangeInventory addZStackIpv6RangeToL3Network(String clientToken, Long cloudId, Scope scope,String l3networkId, String name,
                                                   String startIp, String endIp, String gateway, Integer prefixLen, String dhcpIp, String description) throws OpenapiException;

    IpRangeInventory addZStackIpRangeByCidrForL3Network(String clientToken, Long cloudId, Scope scope,String l3networkId, String name,
                                                        String cidr, String dhcpIp, String description) throws OpenapiException;

    IpRangeInventory addZStackIpv6RangeByCidrForL3Network(String clientToken, Long cloudId, Scope scope,String l3networkId, String name,
                                                          String cidr, Ipv6AddressModeType ipv6AddressMode, String dhcpIp, String description) throws OpenapiException;

    void deleteZStackIpRange(String clientToken, Long cloudId, Scope scope,String ipRangeId, DeleteModeType deleteMode) throws OpenapiException;

    L3NetworkInventory createZStackL3PublicNetwork(String clientToken,
                                                   Long cloudId,Scope scope,
                                                   String l2NetworkId,
                                                   IpVersion ipVersion,
                                                   String name,
                                                   String description,
                                                   String dnsDomain,
                                                   String mirrorNetworkId) throws OpenapiException;


    L3NetworkInventory createZStackL3PrivateNetwork(String clientToken,
                                                    Long cloudId,Scope scope,
                                                    String l2NetworkId,
                                                    String name,
                                                    String description,
                                                    String dnsDomain,
                                                    String mirrorNetworkId) throws OpenapiException;

    L3NetworkInventory createZStackL3SystemNetwork(String clientToken,
                                                   Long cloudId,Scope scope,
                                                   String l2NetworkId,
                                                   String name,
                                                   String description,
                                                   String dnsDomain) throws OpenapiException;

    ZStackVirtualNetworkCreateResult createZStackL3Network(String clientToken,
                                                           Long cloudId,Scope scope,
                                                           String l2NetworkName,
                                                           String name,
                                                           String physicalInterface,
                                                           String startIp,
                                                           String endIp,
                                                           String gateway,
                                                           Integer vLanId,
                                                           String netmask,
                                                           Integer prefixLen,
                                                           L3NetworkType l3NetworkType,
                                                           L3NetworkCategoryType l3NetworkCategory,
                                                           IpVersion ipVersion,
                                                           String dhcpIp,
                                                           String dnsDomain) throws OpenapiException;

    VirtualRouterOfferingInventory createZStackVirtualRouterOffering(String clientToken,
                                                                     Long cloudId,Scope scope,
                                                                     String managementL3NetworkId,
                                                                     String publicL3NetworkId,
                                                                     String imageId,
                                                                     String name,
                                                                     Integer cpuNum,
                                                                     Long memorySizeInMb,
                                                                     String description,
                                                                     Boolean isDefault) throws OpenapiException;

    ListZStackVirtualRouterOfferingsResult listZStackVirtualRouterOfferings(Long cloudId,Scope scope,
                                                                            Set<String> virtualRouterTypeIds,
                                                                            String imageId,
                                                                            String managementL3NetworkId,
                                                                            UsingStateType state,
                                                                            Integer cpuNum,
                                                                            Long memorySizeInMb) throws OpenapiException;

    VirtualRouterVmInventory createZStackVpcRouterWithDns(String clientToken,
                                                          Long cloudId,
                                                          Scope scope,
                                                          String virtualRouterOfferingId,
                                                          String name,
                                                          String description,
                                                          String dns,
                                                          String staticVip,
                                                          String haGroupId,
                                                          String clusterId) throws OpenapiException;

    VpcRouterVmInventory operateZStackDnsToVpcRouter(String clientToken, Long cloudId, Scope scope,String vpcRouterId, String dns, VpcRouterDnsActionType action) throws OpenapiException;

    ListZStackVpcRoutersResult listZStackVpcRouters(Long cloudId,
                                                    Scope scope,
                                                    Set<String> vpcRouterIds,
                                                    String name,
                                                    String clusterId,
                                                    String defaultL3NetworkId,
                                                    String defaultRouteL3NetworkId,
                                                    String hostId,
                                                    String imageId,
                                                    String managementNetworkId,
                                                    String publicNetworkId,
                                                    String instanceOfferingId,
                                                    InstanceStateType state) throws OpenapiException;

    void deleteZStackVpcRouter(String clientToken, Long cloudId, Scope scope, String vpcRouterId) throws OpenapiException;

    SetZStackL2VLanIdsResult getZStackExistsL2VLanIdSet(Long cloudId, Scope scope, Set<String> networkIds, String physicalInterface) throws OpenapiException;

    ListZStackL3NetworkIpStatisticResult listZStackL3NetworkIpStatistics(String clientToken, Long cloudId, Scope scope,
                                                                         String l3NetworkId,
                                                                         IpResourceType ipResourceType,
                                                                         String ipAddress) throws OpenapiException;
}
