package com.cloudbility.openapi;

import java.util.Map;

public class OpenapiErrorCode {
    private String category;
    private String item;
    private String description;

    public OpenapiErrorCode() {
    }

    public OpenapiErrorCode(String category, String item, String description) {
        this.category = category;
        this.item = item;
        this.description = description;
    }

    @Override
    public String toString() {
        return getCode() + ":" + description;
    }

    public String getCode() {
        if (item == null || item.length() == 0) {
            return category;
        }
        return category + "." + item;
    }

    public boolean isSameKind(OpenapiErrorCode errorCode) {
        if (category.equalsIgnoreCase(errorCode.getCategory())) {
            return true;
        }
        return false;
    }

    public String getDescription() {
        return description;
    }

    public String getCategory() {
        return category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public String getItem() {
        return item;
    }

    public void setItem(String item) {
        this.item = item;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public static final OpenapiErrorCode InvalidDate = new OpenapiErrorCode("InvalidParameter", "", "客户端时间非法");

    public static final OpenapiErrorCode CreateUserInvalidStartDate = new OpenapiErrorCode("CreateUser", "InvalidStartDate", "主机初始化时间非法");
    public static final OpenapiErrorCode CreateUserInvalidHostId = new OpenapiErrorCode("CreateUser", "InvalidHostId", "主机唯一标识非法");
    public static final OpenapiErrorCode CreateUserInstanceAlreadyExists = new OpenapiErrorCode("CreateUser", "InstanceAlreadyExists", "主机标识已经存在");
    public static final OpenapiErrorCode CreateUserCustomerNotExists = new OpenapiErrorCode("CreateUser", "CustomerNotExists", "客户不存在");
    public static final OpenapiErrorCode CreateUserInvalidPermission = new OpenapiErrorCode("CreateUser", "InvalidPermission", "权限设置非法");
    public static final OpenapiErrorCode CreateUserInvalidQuota = new OpenapiErrorCode("CreateUser", "InvalidQuota", "配额设置非法");
    public static final OpenapiErrorCode CreateUserInvalidUserType = new OpenapiErrorCode("CreateUser", "InvalidUserType", "用户类型非法");

    public static final OpenapiErrorCode InvalidAccessKeyId = new OpenapiErrorCode("InvalidAccessKeyId", null, "AccessKeyId非法");
    public static final OpenapiErrorCode InvalidAccessKeyIdInActive = new OpenapiErrorCode("InvalidAccessKeyId", "InActive", "AccessKeyId被禁用");
    public static final OpenapiErrorCode InvalidAccessKeyIdNotFound = new OpenapiErrorCode("InvalidAccessKeyId", "NotFound", "AccessKeyId不存在");
    public static final OpenapiErrorCode InvalidParameter = new OpenapiErrorCode("InvalidParameter", "", "参数非法");
    public static final OpenapiErrorCode InvalidSignature = new OpenapiErrorCode("InvalidSignature", "", "非法的签名");
    public static final OpenapiErrorCode InternalServerError = new OpenapiErrorCode("InternalServerError", "", "服务端错误");
    public static final OpenapiErrorCode InvalidRequest = new OpenapiErrorCode("InvalidRequest", "", "不支持的请求");
    public static final OpenapiErrorCode InvalidRequestReadRequestBodyError = new OpenapiErrorCode("InvalidRequest", "ReadRequestBodyError", "读取请求数据异常");
    public static final OpenapiErrorCode InvalidQuotaUnauthorized = new OpenapiErrorCode("InvalidQuota", "Unauthorized", "未授权额度");
    public static final OpenapiErrorCode InvalidQuotaInsufficient = new OpenapiErrorCode("InvalidQuota", "Insufficient", "额度不足");
    public static final OpenapiErrorCode InvalidToken = InvalidToken("token非法");
    public static final OpenapiErrorCode UnknownError = new OpenapiErrorCode("UnknownError", "", "未知异常");
    public static final OpenapiErrorCode UnsupportedInLocalOpenapi = UnsupportedOperation("unsupported for local openapi");
    public static final OpenapiErrorCode MissingAK = new OpenapiErrorCode("MissingAK", "", "没有Openapi通讯的Access Key");
    public static final OpenapiErrorCode RequestsExceedLimit = new OpenapiErrorCode("RequestsExceedLimit", "", "API请求次数超限");

    //虚拟机的错误码
    public static final OpenapiErrorCode InvalidInstanceType = new OpenapiErrorCode("InvalidInstanceType", "", "您指定的实例规格不存在，或者该可用区不售卖这种规格的实例");
    public static final OpenapiErrorCode InvalidInternetChargeType = new OpenapiErrorCode("InvalidInternetChargeType", "", "暂不支持指定的网络付费类型的实例");
    public static final OpenapiErrorCode InvalidInstanceChargeType = new OpenapiErrorCode("InvalidInstanceChargeType", "", "指定的实例计费方式不存在");
    public static final OpenapiErrorCode IncorrectInstanceStatus = new OpenapiErrorCode("IncorrectInstanceStatus", "", "该资源目前的状态不支持此操作");
    public static final OpenapiErrorCode InvalidInstanceId = new OpenapiErrorCode("InvalidInstanceId", "", "指定的实例不存在");
    public static final OpenapiErrorCode InvalidPeriod = new OpenapiErrorCode("InvalidPeriod", "", "指定的时段不合法");
    public static final OpenapiErrorCode InvalidPeriodUnit = new OpenapiErrorCode("InvalidPeriodUnit", "", "指定的时段单位不合法");
    public static final OpenapiErrorCode InvalidSecurityGroupId = new OpenapiErrorCode("InvalidSecurityGroupId", "", "指定的安全组ID无效或者不存在");
    public static final OpenapiErrorCode InvalidPassword = new OpenapiErrorCode("InvalidPassword", "", "密码格式不合法");

    //镜像
    public static final OpenapiErrorCode InvalidImageId = new OpenapiErrorCode("InvalidImageId", "", "指定的镜像在该用户账号下不存在");
    public static final OpenapiErrorCode AccountIsYourSelf = new OpenapiErrorCode("AccountIsYourSelf", "", "您无法共享镜像给自己");
    public static final OpenapiErrorCode InvalidShareAccount = new OpenapiErrorCode("InvalidShareAccount", "", "被分享的用户账号不存在");
    public static final OpenapiErrorCode AccountAlreadyExists = new OpenapiErrorCode("AccountAlreadyExists", "", "账号已存在");

    //余额不足
    public static final OpenapiErrorCode InsufficientBalance = new OpenapiErrorCode("InsufficientBalance", "", "账户余额不足，请先充值再操作");

    //账户受限
    public static final OpenapiErrorCode AccountRestriction = new OpenapiErrorCode("AccountRestriction", "", "账户受限，请检查是否余额不足，帐号被冻结等");

    public static final OpenapiErrorCode RamUserPermissionDenied = new OpenapiErrorCode("RamUserPermissionDenied", "", "该RAM用户没有权限访问该区域");

    //账户未通过资格审核
    public static final OpenapiErrorCode AccountQualificationRestrictions = new OpenapiErrorCode("AccountQualificationRestrictions", "", "该请求账户未通过资格审计");

    public static OpenapiErrorCode IntervalTestError(String description) {
        return new OpenapiErrorCode("IntervalTestError", "", description);
    }

    public static OpenapiErrorCode InvalidQuota(String description) {
        return new OpenapiErrorCode("InvalidQuota", "", description);
    }

    public static OpenapiErrorCode InvalidState(String description) {
        return new OpenapiErrorCode("InvalidState", "", description);
    }


    public static OpenapiErrorCode InternalServerError(String description) {
        return new OpenapiErrorCode("InternalServerError", "", description);
    }

    public static OpenapiErrorCode InvalidRequest(String description) {
        return new OpenapiErrorCode("InvalidRequest", "", description);
    }

    public static OpenapiErrorCode SendSmsFail(String description) {
        return new OpenapiErrorCode("SendSmsFail", "", description);
    }

    public static OpenapiErrorCode UnknownError(String description) {
        return new OpenapiErrorCode("UnknownError", "", description);
    }

    public static OpenapiErrorCode InvalidPermission(String item, String description) {
        return new OpenapiErrorCode("InvalidPermission", item, description);
    }

    public static OpenapiErrorCode UnauthorizedPermission(String permission) {
        return new OpenapiErrorCode("UnauthorizedPermission", "", "未授权的许可：" + permission);
    }

    public static OpenapiErrorCode InvalidParameter(String description) {
        return new OpenapiErrorCode("InvalidParameter", "", description);
    }

    public static OpenapiErrorCode InvalidInnerIp(String description) {
        return new OpenapiErrorCode("InvalidInnerIp", "", description);
    }

    public static OpenapiErrorCode InvalidLicense(String description) {
        return new OpenapiErrorCode("InvalidLicense", "", description);
    }

    public static OpenapiErrorCode InvalidToken(String description) {
        return new OpenapiErrorCode("InvalidToken", "", description);
    }

    public static OpenapiErrorCode UnsupportedOperation(String description) {
        return new OpenapiErrorCode("UnsupportedOperation", "", description);
    }

    public static OpenapiErrorCode InvalidAccessKeyId(String description) {
        return new OpenapiErrorCode("InvalidAccessKeyId", "", description);
    }

    public static final OpenapiErrorCode TeamQuotaExceedLimit(String description) {
        return new OpenapiErrorCode("TeamQuotaExceedLimit", "", description);
    }

    //==============> for vbs

    public static final OpenapiErrorCode InvalidAmount(String description) {
        return new OpenapiErrorCode("InvalidAmount", "", description);
    }

    public static final OpenapiErrorCode InvalidStock(String description) {
        return new OpenapiErrorCode("InvalidStock", "", description);
    }

    public static final OpenapiErrorCode InvalidOrder(String description) {
        return new OpenapiErrorCode("InvalidOrder", "", description);
    }

    public static final OpenapiErrorCode ClientNotInWhitelist(String description) {
        return new OpenapiErrorCode("ClientNotInWhitelist", "", description);
    }

    public static final OpenapiErrorCode IdentityAlreadyCertified(String description) {
        return new OpenapiErrorCode("IdentityAlreadyCertified", "", description);
    }

    public static final OpenapiErrorCode InsufficientBalance(String description) {
        return new OpenapiErrorCode("InsufficientBalance", "", description);
    }

    public static final OpenapiErrorCode CloudNotCertified(String description) {
        return new OpenapiErrorCode("CloudNotCertified", "", description);
    }

    public static final OpenapiErrorCode InvalidTenantStatus(String description) {
        return new OpenapiErrorCode("InvalidTenantStatus", "", description);
    }


    public static final OpenapiErrorCode OriginalQuotaChanged(String description) {
        return new OpenapiErrorCode("OriginalQuotaChanged", "", description);
    }


    public static final OpenapiErrorCode InsufficientInvoiceAmount(String description) {
        return new OpenapiErrorCode("InsufficientInvoiceAmount", "", description);
    }


    public static OpenapiErrorCode fromRequestResult(Map<String, Object> result) {
        String errorCode = (String) result.get("errorCode");
        OpenapiErrorCode ec = null;
        if (errorCode != null) {
            ec = new OpenapiErrorCode();
            int idx = errorCode.indexOf('.');
            if (idx > 0) {
                ec.setCategory(errorCode.substring(0, idx));
                ec.setItem(errorCode.substring(idx + 1, errorCode.length()));
            } else {
                ec.setCategory(errorCode);
                ec.setItem("");
            }
            ec.setDescription(String.valueOf(result.get("errorMessage")));
        }
        return ec;
    }
}
