package com.cloudbility.openapi;

import com.cloudbility.openapi.response.OpsPolicyGroupResult;
import com.cloudbility.openapi.response.OpsPolicyItemResult;
import com.cloudbility.openapi.response.OpsPolicyResult;
import com.cloudbility.openapi.vo.*;

import java.util.Map;

/**
 * v2版本的运维策略，直接团队对应策略，无策略组
 */
public interface OpsPolicyApiV2 {


    /**
     * 获取默认运维策略条目
     *
     * @return
     */
    OpsPolicyItemResult getDefaultOpsPolicyItems() throws OpenapiException;


    /**
     * 根据团队id获取子策略列表
     *
     * @param teamId 团队id
     * @return
     * @throws OpenapiException
     */
    OpsPolicyResult getOpsPoliciesByTeamId(Integer teamId) throws OpenapiException;

    /**
     * 获取策略详情
     *
     * @param policyId 策略id
     * @return
     * @throws OpenapiException
     */
    OpsPolicyDetailVo getOpsPolicyDetail(Integer policyId) throws OpenapiException;

    /**
     * 删除策略
     *
     * @param policyId
     * @return
     * @throws OpenapiException
     */
    Boolean deleteOpsPolicy(Integer policyId) throws OpenapiException;

    /**
     * 根据云账户id获取策略列表
     *
     * @param cloudId 云账户id
     * @return
     * @throws OpenapiException
     */
    OpsPolicyResult getOpsPoliciesByCloudId(Integer cloudId) throws OpenapiException;

    /**
     * 获取策略的条目列表
     *
     * @param policyId 子策略ID
     * @return
     * @throws OpenapiException
     */
    OpsPolicyItemResult getOpsPolicyItems(Integer policyId) throws OpenapiException;

    /**
     * 获取策略的指定条目信息
     *
     * @param policyId
     * @param itemIdentity
     * @return
     * @throws OpenapiException
     */
    OpsPolicyItemVo getOpsPolicyItemInfo(Integer policyId, String itemIdentity) throws OpenapiException;


    /**
     * 修改策略的条目，目前不支持批量类型
     *
     * @param policyId
     * @param items
     */
    OpsPolicyItemResult updateOpsPolicyItems(Integer policyId, Map<String, Object> items) throws OpenapiException;

    /**
     * 修改策略关联主机
     *
     * @param policyId 策略ID
     * @param hostIds  主机ID列表，用逗号连接
     * @return
     * @throws OpenapiException
     */
    Boolean updateOpsPolicyHosts(Integer policyId, String hostIds) throws OpenapiException;


    /**
     * 添加主机到策略
     *
     * @param policyId 策略ID
     * @param hostIds  主机ID列表，用逗号连接
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyHosts(Integer policyId, String hostIds) throws OpenapiException;


    /**
     * 删除策略关联的主机
     *
     * @param policyId 策略ID
     * @param hostIds  主机ID列表，用逗号连接
     * @return
     * @throws OpenapiException
     */
    Boolean deleteOpsPolicyHosts(Integer policyId, String hostIds) throws OpenapiException;


    /**
     * 创建策略
     *
     * @param teamId        团队ID
     * @param policyName    子策略名称
     * @param policyDesc    子策略描述
     * @param priority      子策略优先级
     * @param roleIds
     * @param departmentIds
     * @param memberIds
     * @return
     * @throws OpenapiException
     */
    OpsPolicyVo createOpsPolicy(Integer teamId, String policyName, String policyDesc, Integer priority, String roleIds, String departmentIds, String memberIds) throws OpenapiException;


    /**
     * 授权策略给组织单元
     *
     * @param policyId      策略ID
     * @param departmentIds 组织单元ID列表
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyDepartments(Integer policyId, String departmentIds) throws OpenapiException;


    /**
     * 授权策略给团队成员
     *
     * @param policyId  策略ID
     * @param memberIds 团队成员id列表，逗号分割
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyMembers(Integer policyId, String memberIds) throws OpenapiException;

    /**
     * 授权策略给角色
     *
     * @param policyId 策略ID
     * @param roleIds  角色ID列表
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyRoles(Integer policyId, String roleIds) throws OpenapiException;


    /**
     * 修改策略优先级
     *
     * @param teamId          团队ID
     * @param sortedPolicyIds 子策略ID排序列表
     * @return
     * @throws OpenapiException
     */
    Boolean updateOpsPolicyPriorities(Integer teamId, String sortedPolicyIds) throws OpenapiException;


}
