package com.cloudbility.openapi;

import com.cloudbility.openapi.response.OrderListResult;
import com.cloudbility.openapi.response.PagedOrderListResult;
import com.cloudbility.openapi.vo.OrderVo;
import com.cloudbility.openapi.vo.QuotaVo;
import com.cloudbility.openapi.vo.Scope;
import com.cloudbility.openapi.vo.ServeType;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

public interface OrderApi {

    /**
     * 分页获取团队的订单
     * @param teamId
     * @param serveType
     * @param page
     * @param size
     * @return
     * @throws OpenapiException
     */
    PagedOrderListResult getOrdersByTeam(Long teamId, ServeType serveType, int page, int size) throws OpenapiException;

    /**
     * 获取有效的产品订单
     * @param teamId
     * @param moduleCode  模块Code
     * @param page
     * @param size
     * @return
     * @throws OpenapiException
     */
    PagedOrderListResult getValidOrdersByTeam(Long teamId, String moduleCode, int page, int size) throws OpenapiException;

    /**
     * 查询团队的试用订单
     * @param teamId
     * @param moduleCode 模块Code
     * @return
     * @throws OpenapiException
     */
    OrderVo getTrailOrderByTeam(Long teamId, String moduleCode) throws OpenapiException;

    /**
     * 模块模块的订单
     * @param teamId
     * @param moduleCode 模块Code
     * @return
     * @throws OpenapiException
     */
    OrderListResult getOrdersByModule(Long teamId, String moduleCode) throws OpenapiException;

    /**
     * 根据订单号获取订单
     * @param orderNumber
     * @return
     * @throws OpenapiException
     */
    OrderVo getOrderByNumber(String orderNumber) throws OpenapiException;

    /**
     * 创建订单
     * @param userId 创建者ID
     * @param teamId  团队ID
     * @param serveType  服务类型
     * @param productTerm  购买时长
     * @param price  订单总价
     * @param moduleCode 模块Code
     * @param moduleName 模块名称
     * @param editionCode 套餐Code
     * @param editionName 套餐名称
     * @param quotas 配额
     * @param serveExpireTime 服务过期时间
     * @param productId 可选，产品ID，用于标识某个产品
     * @param scope 订单来源，门户或者管理控制台
     * @return
     * @throws OpenapiException
     */
    OrderVo createOrder(Long userId, Long teamId, ServeType serveType, int productTerm, BigDecimal price,
                        String moduleCode, String moduleName, String editionCode, String editionName,
                        List<QuotaVo> quotas, Date serveExpireTime, String productId, Scope scope) throws OpenapiException;


    /**
     * 结束订单(完成或者取消)
     * @param orderNumber
     * @param cancel  是否取消,true表示取消,false表示已完成
     * @param reason  取消订单的原因,如果cancel为true,可以写一下取消原因
     * @return
     * @throws OpenapiException
     */
    OrderVo finishOrder(String orderNumber, boolean cancel, String reason) throws OpenapiException;

    /**
     *
     * @param teamCode 团队code
     * @param moduleCode 模块code
     * @param editionCode 规格code
     * @param productTerm 购买时长
     * @param serveType 类型
     * @param quotaVos 配额
     * @return
     * @throws OpenapiException
     */
    OrderVo createOrderFromMarket(String teamCode, String moduleCode, String editionCode, int productTerm,
                                  ServeType serveType,List<QuotaVo> quotaVos,Scope scope)throws OpenapiException;

    /**
     * 结束订单(完成或者取消)
     * @param orderNumber
     * @param cancel  是否取消,true表示取消,false表示已完成
     * @param reason  取消订单的原因,如果cancel为true,可以写一下取消原因
     * @return
     * @throws OpenapiException
     */
    OrderVo finishMarketOrder(String orderNumber, boolean cancel, String reason) throws OpenapiException;

    /**
     * 创建试用订单
     * @param teamId
     * @param moduleCode
     * @return
     */
    OrderVo createTrialOrderMarket(Integer teamId,String moduleCode) throws OpenapiException;

    /**
     * 更新订单来源
     * @param orderNumber
     * @param scope
     * @return
     */
    boolean updateOrderScope(String orderNumber,Scope scope) throws OpenapiException;
}
