package com.cloudbility.openapi;

import com.cloudbility.openapi.request.OAuthLoginRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface OtherApi {

    AkAccessToken getOAuth(String accessKeyId, String accessKeySecret, int expireSeconds) throws OpenapiException;

    /**
     * 签名的client，获取token
     *
     * @param expireSeconds
     * @return
     * @throws OpenapiException
     */
    AkAccessToken getOAuth(int expireSeconds) throws OpenapiException;

    /**
     * 检测指定的Openapi Token是否合法
     * @param token
     * @return
     */
    CheckTokenResult checkOauth(String token) throws OpenapiException;


    /**
     * 删除token
     *
     * @param accessKeyId
     * @param accessKeySecret
     * @param token
     * @throws OpenapiException
     */
    void deleteOAuth(String accessKeyId, String accessKeySecret, String token) throws OpenapiException;

    /**
     * 删除token，认证的客户端
     *
     * @param token
     * @throws OpenapiException
     */
    void deleteOAuth(String token) throws OpenapiException;

    boolean ping() throws OpenapiException;

    GetQuotaResult getQuota() throws OpenapiException;


    List<PermissionQuota> getPermissionQuota(List<Permission> permissions, Date time) throws OpenapiException;

    SmsResult sendSms(SmsTemplate template, List<String> phones, List<String> parameters) throws OpenapiException;

    SmsResult sendSmsFromOtherProvider(SmsTemplate template, List<String> phones, List<String> parameters) throws OpenapiException;

    Map<String, SmsResult> sendSmsV2(SmsTemplate type, List<String> tos, List<String> parameterList) throws OpenapiException;

    /**
     * 获取一个签名URL地址，可以在浏览器中自动登录，并打开特定团队的首页
     *
     * @return
     */
    OAuthUrlResult getOAuthLogin(OAuthLoginRequest request) throws OpenapiException;

    /**
     * 校验jwt是否合法
     *
     * @param jwt
     * @return
     * @throws OpenapiException
     */
    CheckJwtResult checkJwt(String jwt) throws OpenapiException;

    /**
     * 判断当前是不是SaaS版
     * @return
     * @throws OpenapiException
     */
    boolean isSaaS() throws OpenapiException;

    /**
     * 校验短信验证码
     * @param sessionId
     * @param code
     * @return
     * @throws OpenapiException
     */
    boolean verifyPhoneCode(String sessionId,String code) throws OpenapiException;

}
