package com.cloudbility.openapi;

import com.cloudbility.openapi.client.OpenapiConfig;
import com.cloudbility.openapi.vo.AkAccessToken;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class RecursiveRefreshTokenProvider extends AutoRefreshTokenProvider implements TokenProvider {
    /**
     * 一个初始token，然后调度一个任务每隔一段时间，自动获取新的token
     *
     * @param executor
     * @param initialToken
     * @param endpoint
     * @param expireSeconds
     */
    public RecursiveRefreshTokenProvider(ScheduledExecutorService executor, AkAccessToken initialToken, String endpoint, int expireSeconds) {
        super(initialToken, expireSeconds);
        OpenapiConfig openapiConfig = new OpenapiConfig();
        openapiConfig.setEndpoint(endpoint);
        openapiConfig.setTokenProvider(this);
        setOpenapiClient(new OpenapiClient(openapiConfig));
        long validMs = initialToken.getExpireTime().getTime() - System.currentTimeMillis();
        long initialDelay = System.currentTimeMillis() + validMs / 2;
        if (initialDelay < 1000) {
            throw new RuntimeException("initialToken already (or will soon) expired");
        }
        executor.scheduleAtFixedRate(() -> doRefresh(), initialDelay, (expireSeconds - 60) * 1000, TimeUnit.MILLISECONDS);
    }


    @Override
    protected boolean needRefresh() {
        //父类的刷新机制不需要了
        return false;
    }
}
