package com.cloudbility.openapi;

import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.RoleDetailResult;
import com.cloudbility.openapi.vo.RoleVo;

import java.util.List;

public interface RoleApi {

    GetFunctionResult getFunctions() throws OpenapiException;

    PagedRoleListResult getRoles(long teamId) throws OpenapiException;

    PagedRoleListResult getRolesByTeamCode(String teamCode) throws OpenapiException;

    RoleVo getRolesByCode(long teamId, String roleCode) throws OpenapiException;

    RoleVo getRoleByRoleAndTeamCode(String teamCode, String roleCode) throws OpenapiException;

    PagedRoleListResult getRoleByTeamCodeAndUserAccount(String teamCode, String userAccount) throws OpenapiException;

    PagedRoleListResult getRoleByTeamAndUserId(long teamId, long userId) throws OpenapiException;

    RoleDetailResult getRole(long roleId) throws OpenapiException;

    GetFunctionResult getRoleFunctions(long roleId) throws OpenapiException;

    UserBriefResult getRoleMembers(long roleId) throws OpenapiException;

    GetRoleTemplatesResult getRoleTemplates() throws OpenapiException;

    PagedRoleListResult getRolesByPage(long teamId, Integer page, Integer size) throws OpenapiException;

    PagedRoleListResult getRolesByCloud(long cloudId) throws OpenapiException;

    /**
     * @param hgId 物理分组id
     * @return
     * @throws OpenapiException
     */
    PagedRoleListResult getRolesByHg(Long hgId) throws OpenapiException;

    /**
     * @param vgId 虚拟分组
     * @return
     * @throws OpenapiException
     */
    PagedRoleListResult getRolesByVg(Long vgId) throws OpenapiException;

    GetRoleResourcesResult getRoleResource(long roleId) throws OpenapiException;

    //post
    Long createRole(long teamId, String name, String description) throws OpenapiException;

    void grantRoleToCloud(long roleId, long cloudId) throws OpenapiException;

    void addFunctionsToRole(long roleId, List<String> functionCodes) throws OpenapiException;

    void grantRoleToHg(long roleId, Long hgId) throws OpenapiException;

    void grantRoleToVg(long roleId, Long vgId) throws OpenapiException;

    void grantRoleToMembers(long roleId, List<Long> userIds) throws OpenapiException;

    void grantRoleToMemberByAccount(long roleId, String userAccount) throws OpenapiException;


    void deleteRole(long roleId) throws OpenapiException;

    void revokeRoleFromCloud(long roleId, long cloudId) throws OpenapiException;

    void revokeRoleFromHg(long roleId, Long hgId) throws OpenapiException;

    void revokeRoleFromVg(long roleId, Long vgId) throws OpenapiException;

    void revokeRoleFromMembers(long roleId, List<Long> userIds) throws OpenapiException;

    void revokeRoleFromMembersByAccount(long roleId, String account) throws OpenapiException;

    void deleteRoleFunctions(long roleId, List<String> functionCodes) throws OpenapiException;

    void updateRoleName(long roleId, String name, String description) throws OpenapiException;

    RoleListResult findRoleByName(long teamId, String name) throws OpenapiException;

    RoleListResult findRoleByNameLike(long teamId, String name) throws OpenapiException;


}
