package com.cloudbility.openapi;

import com.cloudbility.openapi.request.AddLdapConfigRequest;
import com.cloudbility.openapi.response.LdapConfigVoList;
import com.cloudbility.openapi.response.LdapSyncConfigVoList;
import com.cloudbility.openapi.vo.LdapSyncConfigVo;

public interface SystemApi {

    LdapConfigVoList getLdapConfigs() throws OpenapiException;

    LdapSyncConfigVoList getLdapSyncConfigs() throws OpenapiException;

    void addLdapConfig(AddLdapConfigRequest request) throws OpenapiException;

    void updateLdapConfig(AddLdapConfigRequest request) throws OpenapiException;

    int updateLdapConfigBaseDn(String preBaseDn, String newBaseDn) throws OpenapiException;

    int deleteLdapConfig(String baseDn) throws OpenapiException;

    void updateLdapSyncConfig(LdapSyncConfigVo request) throws OpenapiException;

    /**
     * 设置ldapAccount时，会将LDAP中的特定账户绑定到指定行云管家用户；未设置ldapAccount时，将根据行云管家用户的账号查询LDAP，并进行绑定。ldapAccountKey可以指定LDAP中的属性名称，未设置时，从LDAP配置中读取
     *
     * @param baseDn
     * @param userId
     * @param ldapAccountKey
     * @param ldapAccount
     */
    void bindLdapUser(String baseDn, long userId, String ldapAccountKey, String ldapAccount) throws OpenapiException;

    /**
     * newAccount未设置时，将生成随机账号，可以设置原来的账号，相当于保留账号，如果设置其他账号，不能与现有用户重复；<br/>" +
     * "newPassword设置时，将按照系统的复杂度规则进行校验,未设置时，不会修改原有用户的本地密码；<br/>" +
     * "generatePassword=true和newPassword不要同时设置；<br/>" +
     * "forceUpdatePwd,emailNewPassword的默认取值：false，当newPassword设置时或者generatePassword=true时，emailNewPassword=true才会生效
     *
     * @param userId
     * @param newAccount
     * @param newPassword
     * @param generatePassword
     * @param forceUpdatePwd
     * @param emailNewPassword
     */
    void unbindLdapUser(long userId, String newAccount, String newPassword, boolean generatePassword, boolean forceUpdatePwd, boolean emailNewPassword) throws OpenapiException;

}
