package com.cloudbility.openapi;

import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.ImportTeamVo;

import java.util.List;

public interface TeamApi {

    GetTeamResult getTeamByCode(String code, boolean includeMembers) throws OpenapiException;

    PagedUserListResult getTeamMembersByCode(String teamCode) throws OpenapiException;

    GetTeamQuotaResult getTeamQuotaByCode(String teamCode) throws OpenapiException;

    TeamBriefResult findTeamIds() throws OpenapiException;

    GetTeamResult getTeam(long teamId, boolean includeMembers) throws OpenapiException;

    PagedUserListResult getTeamMembers(long teamId) throws OpenapiException;

    GetTeamQuotaResult getTeamQuota(long teamId) throws OpenapiException;

    GetTeamListResult getTeams(Integer page, Integer size) throws OpenapiException;

    ImportTeamResult importTeams(List<ImportTeamVo> teams) throws OpenapiException;

    TeamListResult findByName(String name) throws OpenapiException;

    TeamListResult findByNameLike(String name) throws OpenapiException;

    void addMembersByCode(String teamCode, String account) throws OpenapiException;

    void addMembers(long teamId, List<Long> userIds) throws OpenapiException;

    //PUT /team/byCode/{code}
    void updateTeamNameOrCreatorByCode(String code, String name, String creatorAccount) throws OpenapiException;

    void updateTeamNameOrCreator(long teamId, String name, Long creator) throws OpenapiException;

    /**
     * 设置为null时，不更新相应配额
     *
     * @param teamId
     * @param members
     * @param hosts
     * @param cdns
     * @param buckets
     * @param dbs
     * @throws OpenapiException
     */
    void updateTeamQuota(long teamId, Integer members, Integer hosts, Integer cdns, Integer buckets, Integer dbs) throws OpenapiException;

    void updateTeamQuotaByCode(String teamCode, Integer members, Integer hosts, Integer cdns, Integer buckets, Integer dbs) throws OpenapiException;

    void updateTeamCode(Long teamId, String code) throws OpenapiException;

    void deleteTeams(List<Long> teamIds) throws OpenapiException;

    void deleteTeamByCode(String code) throws OpenapiException;

    void deleteTeamMembers(Long teamId, List<Long> memberIds) throws OpenapiException;

    void deleteTeamMemberByCode(String teamCode, String memberAccount) throws OpenapiException;

    // v6 module api ===>

    QuotaItemResult getQuotaItems() throws OpenapiException;

    int countTeamMemberOnline(long teamId) throws OpenapiException;

    /**
     * 创建团队
     * @param teamName 团队名称
     * @param teamCode 团队标识
     * @param teamName 创建者
     * @return
     * @throws OpenapiException
     */
    GetTeamResult createTeam(String teamName, String teamCode, Long creatorId) throws OpenapiException;

}
