package com.cloudbility.openapi;

import com.cloudbility.openapi.vo.VirtualGroupDetailVo;
import com.cloudbility.openapi.vo.VirtualGroupResult;
import com.cloudbility.openapi.vo.VirtualGroupVo;
import org.omg.CORBA.INTERNAL;

import java.util.List;

public interface VgApi {

    VirtualGroupDetailVo getVirtualGroupDetailVo(long id) throws OpenapiException;

    VirtualGroupResult getByTeam(long teamId) throws OpenapiException;

    VirtualGroupResult getByTeamCode(String teamCode) throws OpenapiException;

    /**
     * teamId和cloudId至少指定一个
     *
     * @param teamId
     * @param cloudId
     * @param name
     * @return
     * @throws OpenapiException
     */
    @Deprecated
    VirtualGroupResult getByName(Long teamId, Long cloudId, String name) throws OpenapiException;

    VirtualGroupResult getByName(Long teamId, String name) throws OpenapiException;

    VirtualGroupResult getByTeamId(Long teamId, Long parentVgId) throws OpenapiException;

    VirtualGroupResult getByTeamCode(String teamCode, Long parentVgId) throws OpenapiException;

    @Deprecated
    VirtualGroupVo createVg(long cloudId, String name, String roles) throws OpenapiException;

    /**
     * @param teamId
     * @param parentVgId
     * @param name
     * @param roles
     * @param members
     * @return
     * @throws OpenapiException
     * @since v4.18
     */
    VirtualGroupVo createVgNew(long teamId, Long parentVgId, String name, String roles, String members) throws OpenapiException;

    void addBucketToVg(Long vgId, List<Long> bucketIds) throws OpenapiException;

    void addCdnsToVg(Long vgId, List<Long> cdnIds) throws OpenapiException;

    void addHostsToVg(Long vgId, List<Long> hostIds) throws OpenapiException;

    /**
     * name和roles两个参数不能同时为空。为空时，不会修改s
     *
     * @param vgId
     * @param name
     * @param roles
     * @throws OpenapiException
     */
    void updateVg(Long vgId, String name, String roles) throws OpenapiException;

    void updateVgName(Long vgId, String name) throws OpenapiException;

    void updateVgRoles(Long vgId, String roles) throws OpenapiException;

    void updateVgMembers(Long vgId, String members) throws OpenapiException;

    void updateVgDepartments(Long vgId, String departments) throws OpenapiException;

    /**
     * @param vgId
     * @throws OpenapiException
     */
    void revokeAllVgRoles(Long vgId) throws OpenapiException;

    void deleteVg(List<Long> vgIds) throws OpenapiException;

    void deleteAllResourcesFromVg(Long vgId) throws OpenapiException;

    void deleteAllBucketsFromVg(Long vgId) throws OpenapiException;

    void deleteBucketsFromVg(Long vgId, List<Long> bucketIds) throws OpenapiException;

    void deleteAllCdnsFromVg(Long vgId) throws OpenapiException;

    void deleteCdnsFromVg(Long vgId, List<Long> cdnIds) throws OpenapiException;

    void deleteAllHostsFromVg(Long vgId) throws OpenapiException;

    void deleteHostsFromVg(Long vgId, List<Long> hostIds) throws OpenapiException;

}
