package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.TokenProvider;
import com.cloudbility.openapi.requestor.OpenapiRequestor;
import org.apache.http.client.HttpClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractOpenapiClient {

    protected final Logger log = LoggerFactory.getLogger(getClass());

    protected OpenapiConfig openapiConfig;
    private HttpComponentsOpenapiExecutor executor;

    public AbstractOpenapiClient(OpenapiConfig openapiConfig, HttpClient httpClient) {
        this.openapiConfig = openapiConfig;
        this.executor = new HttpComponentsOpenapiExecutor(openapiConfig, httpClient);
    }

    public AbstractOpenapiClient(OpenapiConfig openapiConfig) {
        this.openapiConfig = openapiConfig;
        this.executor = new HttpComponentsOpenapiExecutor(openapiConfig);
    }


    public <R> R execute(OpenapiRequestor request, Class<R> responseClazz) throws OpenapiException {
        TokenProvider tokenProvider = openapiConfig.getTokenProvider();
        if (tokenProvider != null) {
            ((RequestorBuilder) request).token(tokenProvider.getToken());
        }
        return executor.execute(request, responseClazz);
    }

    public OpenapiConfig config() {
        return openapiConfig;
    }
}
