package com.cloudbility.openapi.client;

import com.cloudbility.openapi.AuthorizationApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.response.ResourceResult;
import com.cloudbility.openapi.vo.AuthorizationInfo;
import com.cloudbility.openapi.vo.ResourceTypeVo;

import java.util.List;

public class DefaultAuthorizationApi extends AbstractApi implements AuthorizationApi {
    public DefaultAuthorizationApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public AuthorizationInfo getAuthorization(long teamId, ResourceTypeVo type, long resourceId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/resource/" + type + "/" + resourceId + "/authorization")
                .paramShouldNotEmpty("teamId", teamId)
                .build(), AuthorizationInfo.class);
    }

    @Override
    public void updateAuthorization(long teamId, ResourceTypeVo type, long resourceId, List<Long> roleIds, List<Long> departmentIds, List<Long> memberIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/resource/" + type + "/" + resourceId + "/authorization")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("roleIds", roleIds != null ? StringUtil.join(roleIds, ",") : null)
                .paramIfNotNull("departmentIds", departmentIds != null ? StringUtil.join(departmentIds, ",") : null)
                .paramIfNotNull("memberIds", memberIds != null ? StringUtil.join(memberIds, ",") : null)
                .build(), Void.class);
    }

    @Override
    public void grantAuthorization(long teamId, ResourceTypeVo type, long resourceId, List<Long> roleIds, List<Long> departmentIds, List<Long> memberIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/resource/" + type + "/" + resourceId + "/authorization")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("roleIds", roleIds != null ? StringUtil.join(roleIds, ",") : null)
                .paramIfNotNull("departmentIds", departmentIds != null ? StringUtil.join(departmentIds, ",") : null)
                .paramIfNotNull("memberIds", memberIds != null ? StringUtil.join(memberIds, ",") : null)
                .build(), Void.class);
    }

    @Override
    public void revokeAuthorization(long teamId, ResourceTypeVo type, long resourceId, List<Long> roleIds, List<Long> departmentIds, List<Long> memberIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/resource/" + type + "/" + resourceId + "/authorization")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("roleIds", roleIds != null ? StringUtil.join(roleIds, ",") : null)
                .paramIfNotNull("departmentIds", departmentIds != null ? StringUtil.join(departmentIds, ",") : null)
                .paramIfNotNull("memberIds", memberIds != null ? StringUtil.join(memberIds, ",") : null)
                .build(), Void.class);
    }

    @Override
    public ResourceResult getResourcesByUserAuthorization(long teamId, ResourceTypeVo type, long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/resources/" + type + "/byUser/" + userId)
                .paramShouldNotEmpty("teamId", teamId)
                .build(), ResourceResult.class);
    }
}
