package com.cloudbility.openapi.client;

import com.cloudbility.openapi.CasServerApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.vo.CasAppVo;
import com.cloudbility.openapi.vo.CasLogoutTypeVo;
import com.cloudbility.openapi.vo.CasServiceTypeVo;

public class DefaultCasServerApi extends AbstractApi implements CasServerApi {

    public DefaultCasServerApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public CasAppVo createCasApp(String serviceId,
                                 String name,
                                 String module,
                                 CasServiceTypeVo serviceType,
                                 Long teamId,
                                 CasLogoutTypeVo logoutType,
                                 String logoutUrl,
                                 Integer evaluationOrder) throws OpenapiException {
        return createCasApp(serviceId, name, module, serviceType, teamId, logoutType, logoutUrl, evaluationOrder, true);
    }

    @Override
    public CasAppVo createCasApp(String serviceId,
                                 String name,
                                 String module,
                                 CasServiceTypeVo serviceType,
                                 Long teamId,
                                 CasLogoutTypeVo logoutType,
                                 String logoutUrl,
                                 Integer evaluationOrder, boolean forceLogout) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/cas/app")
                .paramShouldNotEmpty("serviceId", serviceId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("module", module)
                .paramShouldNotEmpty("serviceType", serviceType)
                .paramIfNotNull("teamId", teamId)
                .paramShouldNotEmpty("logoutType", logoutType)
                .paramIfNotNull("logoutUrl", logoutUrl)
                .paramIfNotNull("evaluationOrder", evaluationOrder)
                .paramIfNotNull("forceLogout", forceLogout)
                .build(), CasAppVo.class);
    }

    @Override
    public CasAppVo deleteCasApp(String serviceId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .delete("/cas/app")
                .paramShouldNotEmpty("serviceId", serviceId)
                .build(), CasAppVo.class);
    }
}
