package com.cloudbility.openapi.client;

import com.cloudbility.openapi.CredentialApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.CredentialHostsResult;
import com.cloudbility.openapi.response.CredentialRolesResult;
import com.cloudbility.openapi.response.GetCredentialListResult;
import com.cloudbility.openapi.vo.CreateCredentialRequest;
import com.cloudbility.openapi.vo.CredentialHostBrief;
import com.cloudbility.openapi.vo.CredentialVo;

public class DefaultCredentialApi extends AbstractApi implements CredentialApi {
    public DefaultCredentialApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public GetCredentialListResult getByHost(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/credential/byHost/" + hostId).build(),
                GetCredentialListResult.class);
    }

    @Override
    public GetCredentialListResult getByTeam(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/credential/byTeam/" + teamId),
                GetCredentialListResult.class);
    }

    @Override
    public GetCredentialListResult getByTeamCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/credential/byTeamCode/" + teamCode),
                GetCredentialListResult.class);
    }

    @Override
    public CredentialHostsResult getCredentailHostsById(long credentialId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/credential/" + credentialId + "/hosts"),
                CredentialHostsResult.class);
    }

    @Override
    public CredentialRolesResult getCredentialRolesById(long credentialId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/credential/" + credentialId + "/roles"),
                CredentialRolesResult.class);
    }


    @Override
    public CredentialVo createCredential(CreateCredentialRequest request) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/credential")
                        .body(request).build().build(),
                CredentialVo.class);
    }

    @Override
    public CredentialHostBrief addCredentailHost(long credentialId, long hostId, Integer port, String password, String key, Long operator) throws OpenapiException {
        return addCredentailHost(credentialId, hostId, port, password, key, operator, null);
    }

    @Override
    public CredentialHostBrief addCredentailHost(long credentialId, long hostId, Integer port, String password, String key, Long operator, String suPassword) throws OpenapiException {
        checkOnlyOneNotNull("set only one of password and key", password, key);
        return execute(RequestorBuilder.create()
                        .post("/credential/" + credentialId + "/" + hostId)
                        .paramIfNotNull("port", port)
                        .paramIfNotNull("password", password)
                        .paramIfNotNull("key", key)
                        .paramIfNotNull("operator", operator)
                        .paramIfNotNull("suPassword", suPassword),
                CredentialHostBrief.class);
    }

    @Override
    public CredentialVo updateCredential(long credentialId, String name, String account, String roles) throws OpenapiException {
        return updateCredential(credentialId, name, account, roles, null, null);
    }

    @Override
    public CredentialVo updateCredential(long credentialId, String name, String account, String roles, String suUser, String suCommand) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .put("/credential/" + credentialId)
                        .paramIfNotNull("name", name)
                        .paramIfNotNull("account", account)
                        .paramIfNotNull("roles", roles)
                        .paramIfNotNull("suUser", suUser)
                        .paramIfNotNull("suCommand", suCommand),
                CredentialVo.class);
    }

    @Override
    public void deleteCredential(long credentialId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/credential/" + credentialId),
                CredentialVo.class);
    }

    @Override
    public void deleteHostFromCredential(long credentialId, long hostId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/credential/" + credentialId + "/" + hostId),
                CredentialVo.class);
    }
}
