package com.cloudbility.openapi.client;

import com.cloudbility.openapi.DbApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.DbType;

/**
 * @author chjgan
 * @date 2020/1/16 15:07
 */
public class DefaultDbApi extends AbstractApi implements DbApi {
    public DefaultDbApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public ListDbGroupResult getDbGroups(Long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/db/group")
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                ListDbGroupResult.class);
    }

    @Override
    public void updateDbGroup(Long dbGroupId, String name) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .method(HttpMethod.PUT)
                        .path("/db/group/" + dbGroupId + "/name")
                        .paramShouldNotEmpty("name", name)
                        .build(),
                Void.class);
    }

    @Override
    public void createDbGroup(Long teamId, String name) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/db/group" )
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .build(),
                Void.class);
    }

    @Override
    public void deleteDbGroup(Long dbGroupId, Long userId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/db/group/" + dbGroupId)
                        .param("userId", userId)
                        .build(),
                Void.class);
    }

    @Override
    public DbVo getDbById(Long dbId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/db/"+dbId)
                        .build(),
                DbVo.class);
    }

    @Override
    public ListDbResult listDbs(Long dbGroupId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/db/byGroup/" + dbGroupId)
                        .build(),
                ListDbResult.class);
    }

    @Override
    public ListDbResult listDbs(Long teamId, Long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/db/byUser")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("userId", userId)
                        .build(),
                ListDbResult.class);
    }

    @Override
    public ListDbResult listDbsByPage(Long teamId, Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/db")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramIfNotNull("page", page)
                        .paramIfNotNull("size", size)
                        .build(),
                ListDbResult.class);
    }

    @Override
    public DbVo createDb(Long teamId, Long dbGroupId, String name, DbType dbType, JumpServerType jumpServerType, Integer jumpServerPort, Long jumpServerId, Long creatorId, Long hostId, String ip, Integer port, DbLocation location, boolean autoStart) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/db")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .paramShouldNotEmpty("dbType", dbType)
                        .paramShouldNotEmpty("jumpServerType", jumpServerType)
                        .paramShouldNotEmpty("jumpServerId", jumpServerId)
                        .paramShouldNotEmpty("creatorId", creatorId)
                        .paramShouldNotEmpty("ip", ip)
                        .paramShouldNotEmpty("port", port)
                        .paramShouldNotEmpty("location", location)
                        .paramIfNotNull("dbGroupId", dbGroupId)
                        .paramIfNotNull("autoStart", autoStart)
                        .paramIfNotNull("hostId", hostId)
                        .paramIfNotNull("jumpServerPort", jumpServerPort)
                        .build(),
                DbVo.class);
    }

    @Override
    public void editDb(Long dbId, Long dbGroupId, String name, DbType dbType, JumpServerType jumpServerType, Long jumpServerId, String ip, String isInner, Integer port) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/db/" + dbId)
                        .paramShouldNotEmpty("name", name)
                        .paramShouldNotEmpty("jumpServerId", jumpServerId)
                        .paramShouldNotEmpty("port", port)
                        .param("dbType", dbType)
                        .param("dbGroupId", dbGroupId)
                        .param("jumpServerType", jumpServerType)
                        .param("ip", ip)
                        .param("isInner", isInner)
                        .build(),
                Void.class);
    }

    @Override
    public void deleteDb(Long dbId, Long userId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/db/" + dbId)
                        .param("userId", userId)
                        .build(),
                Void.class);
    }

    @Override
    public void createDbPlan(Long dbId,Long teamId,Long creatorId, String name, String dbUser, String dbPassword, String dbInstance, String description, String roleIds, String userIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/db/plan")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("dbId", dbId)
                        .paramShouldNotEmpty("name", name)
                        .paramShouldNotEmpty("dbUser", dbUser)
                        .paramShouldNotEmpty("dbPassword", dbPassword)
                        .param("creatorId", creatorId)
                        .param("dbInstance", dbInstance)
                        .param("description", description)
                        .param("roleIds", roleIds)
                        .param("userIds", userIds)
                        .build(),
                Void.class);
    }

    @Override
    public void deleteDbPlan(Long planId, Long userId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/db/plan/" + planId)
                        .param("userId", userId)
                        .build(),
                Void.class);
    }
}
