package com.cloudbility.openapi.client;

import com.cloudbility.openapi.DepartmentApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiErrorCode;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.response.DepartmentTreeVo;
import com.cloudbility.openapi.response.DepartmentVo;
import com.cloudbility.openapi.response.FlatDepartmentInfo;
import com.cloudbility.openapi.vo.BooleanVo;

import java.util.List;

public class DefaultDepartmentApi extends AbstractApi implements DepartmentApi {
    public DefaultDepartmentApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public boolean isDepartmentEnabled(long teamId) throws OpenapiException {
        BooleanVo b = execute(RequestorBuilder.create()
                        .get("/department/init")
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                BooleanVo.class);
        return b.isValue();
    }

    @Override
    public DepartmentTreeVo initDepartment(long teamId, String name, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/department/init")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("description", description)
                        .build(),
                DepartmentTreeVo.class);
    }

    @Override
    public DepartmentTreeVo getDepartments(long teamId, boolean includeMembers) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/department")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("includeMembers", includeMembers)
                        .build(),
                DepartmentTreeVo.class);
    }

    @Override
    public DepartmentVo getDepartment(long departmentId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/department/" + departmentId)
                        .build(),
                DepartmentTreeVo.class);
    }

    @Override
    public FlatDepartmentInfo getFlatDepartments(long teamId, boolean includeMembers) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/departmentFlat")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("includeMembers", includeMembers)
                        .build(),
                FlatDepartmentInfo.class);
    }

    @Override
    public DepartmentVo addDepartment(long teamId, long parentDepartmentId, String name, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/department")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("parentId", parentDepartmentId)
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("description", description)
                        .build(),
                DepartmentVo.class);
    }

    @Override
    public void updateDepartment(long departmentId, String name, String description) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/department/" + departmentId)
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("description", description)
                        .build(),
                Void.class);
    }

    @Override
    public void deleteDepartment(long departmentId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/department/" + departmentId)
                        .build(),
                Void.class);
    }

    @Override
    public void moveToDepartment(long departmentId, List<Long> memberIds) throws OpenapiException {
        if (memberIds == null || memberIds.size() == 0) {
            throw new OpenapiException(OpenapiErrorCode.InvalidParameter("memberIds not set"));
        }
        execute(RequestorBuilder.create()
                        .put("/department/" + departmentId + "/members")
                        .paramShouldNotEmpty("memberIds", StringUtil.join(memberIds, ","))
                        .build(),
                Void.class);
    }
}
