package com.cloudbility.openapi.client;

import com.cloudbility.openapi.HostApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.request.ImportHostRequest;
import com.cloudbility.openapi.request.ImportPublicHostRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.CloudHostVo;
import com.cloudbility.openapi.vo.ImportHostVo;
import com.cloudbility.openapi.vo.JobVo;
import com.cloudbility.openapi.vo.ShortcutVo;

import java.util.List;
import java.util.Map;

public class DefaultHostApi extends AbstractApi implements HostApi {

    public DefaultHostApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public HostGroupVoResult getHostGroupByCloud(long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/hg/byCloud/" + cloudId)
                .build(), HostGroupVoResult.class);
    }

    @Override
    public CloudHostResult findByInnerIpAndPublicIp(long teamId, Long cloudId, String innerIp, String publicIp) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/findByInnerIpAndPublicIp")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("cloudId", cloudId)
                .paramIfNotNull("innerIp", innerIp)
                .paramIfNotNull("publicIp", publicIp)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostResult findByInstanceId(long teamId, Long cloudId, String instanceId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/findByInstanceId")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("cloudId", cloudId)
                .paramShouldNotEmpty("instanceId", instanceId)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostResult getByTeam(long teamId, Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/byTeam/" + teamId)
                .paramIfNotNull("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostResult getByUser(long teamId, long userId) throws OpenapiException {
        return getByUser(teamId, userId, null, null);
    }

    @Override
    public CloudHostResult getByUser(long teamId, long userId, Long cloudId, Long groupId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/byUser/" + userId)
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("cloudId", cloudId)
                .paramIfNotNull("groupId", groupId)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostResult getByTeamCode(String teamCode, Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/byTeamCode/" + teamCode)
                .paramIfNotNull("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostResult getByCloud(long cloudId, Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/byCloud/" + cloudId)
                .paramIfNotNull("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), CloudHostResult.class);
    }

    @Override
    public GetHostBriefResult getHostBriefList(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/findIds")
                .paramIfNotNull("teamId", teamId)
                .build(), GetHostBriefResult.class);
    }

    @Override
    public GetAgentScriptResult getAgentScript(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/" + hostId + "/agentScript")
                .build(), GetAgentScriptResult.class);
    }

    @Override
    public CloudHostResult findByIp(long teamId, Long cloudId, String ip) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/findByIp")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("cloudId", cloudId)
                .paramShouldNotEmpty("ip", ip)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostVo getHost(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/" + hostId)
                .build(), CloudHostVo.class);
    }

/*    @Override
    public ListCredentialsResult getHostCredentials(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/" + hostId + "/credentails")
                .build(), ListCredentialsResult.class);
    }*/

    @Override
    public ImportHostResult importHosts(ImportHostRequest request) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/host")
                .body(request)
                .build(), ImportHostResult.class);
    }

    @Override
    public JobVo importPublic(long cloudId, String regionId, String instanceId, Long creatorId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/host/public")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramShouldNotEmpty("instanceId", instanceId)
                .paramIfNotNull("creatorId", creatorId)
                .build(), JobVo.class);
    }

    @Override
    public JobVo importPublic(ImportPublicHostRequest request) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/hosts/public")
                .body(request)
                .build(), JobVo.class);
    }

    //    @Override
//    public ImportHostResult importHost(long teamId, long cloudId, ImportHostVo hostVo, List<Integer> vgIds) throws OpenapiException {
//        return execute(RequestorBuilder.create()
//                .post("/hostV2")
//                .paramShouldNotEmpty("teamId", teamId)
//                .paramShouldNotEmpty("cloudId", cloudId)
//                .paramShouldNotEmpty("regionId", regionId)
//                .paramShouldNotEmpty("instanceId", instanceId)
//                .paramIfNotNull("creatorId", creatorId)
//                .build(), JobVo.class);
//    }

    @Override
    public void deleteHosts(List<Long> hostIds) throws OpenapiException {
        if (hostIds == null || hostIds.size() == 0) {
            throw new IllegalArgumentException("hostIds is empty");
        }
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/host/" + StringUtil.join(hostIds, ","))
                .build(), Void.class);
    }

    @Override
    public Map<Long, String> getHostStatusMap(long teamId, List<Long> hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/host/findStatus")
                .param("teamId", String.valueOf(teamId))
                .paramIfNotNull("hostIds", hostIds != null && hostIds.size() > 0 ? StringUtil.join(hostIds, ",") : null)
                .build(), GetHostsStatusResult.class)
                .getStatusMap();
    }

    @Override
    public void revokeAllRolesFromHg(Long hgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/hg/" + hgId + "/revokeAllRoles")
                .build(), Void.class);
    }

    @Override
    public void updateHost(long hostId, ImportHostVo hostVo) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/host/" + hostId)
                .body(hostVo)
                .build(), Void.class);
    }

    @Override
    public ListCredentialsResult updateHostCredentials(long hostId, String user, String password) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/host/" + hostId + "/credentials")
                .paramShouldNotEmpty("user", user)
                .paramShouldNotEmpty("password", password)
                .build(), ListCredentialsResult.class);
    }

    @Override
    public GetShortcutsResult getHostShortcuts(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/" + hostId + "/shortcuts")
                .build(), GetShortcutsResult.class);
    }

    @Override
    public int deleteHostShortcuts(long hostId) throws OpenapiException {
        DeleteEntitiesResult result = execute(RequestorBuilder.create()
                .delete("/host/" + hostId + "/shortcuts")
                .build(), DeleteEntitiesResult.class);
        return result.getCount();
    }

    @Override
    public ShortcutVo getShortcut(Long shortcutId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/shortcut/" + shortcutId)
                .build(), ShortcutVo.class);
    }

    @Override
    public void deleteShortcut(Long shortcutId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/shortcut/" + shortcutId)
                .build(), Void.class);
    }

    @Override
    public HostCostResult getHostCostByTeam(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/cost/byTeam/" + teamId)
                .build(), HostCostResult.class);
    }

    @Override
    public HostCostResult getHostCostByCloud(long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/cost/byCloud/" + cloudId)
                .build(), HostCostResult.class);
    }


    @Override
    public OAuthUrlResult getHostCostCsvUrlByTeam(long teamId, String lang, boolean oneoff, long expireSeconds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/costCsvUrl/byTeam/" + teamId)
                .paramShouldNotEmpty("lang", lang)
                .paramShouldNotEmpty("oneoff", oneoff)
                .paramShouldNotEmpty("expireSeconds", expireSeconds)
                .build(), OAuthUrlResult.class);
    }

    @Override
    public OAuthUrlResult getHostCostCsvUrlByCloud(long cloudId, String lang, boolean oneoff, long expireSeconds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/costCsvUrl/byCloud/" + cloudId)
                .paramShouldNotEmpty("lang", lang)
                .paramShouldNotEmpty("oneoff", oneoff)
                .paramShouldNotEmpty("expireSeconds", expireSeconds)
                .build(), OAuthUrlResult.class);
    }

    @Override
    public OptimizedAdviceResult getHostOptimizedAdviceByTeam(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/advice/byTeam/" + teamId)
                .build(), OptimizedAdviceResult.class);
    }

    @Override
    public OptimizedAdviceResult getHostOptimizedAdviceByCloud(long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/advice/byCloud/" + cloudId)
                .build(), OptimizedAdviceResult.class);
    }

    @Override
    public OptimizedAdviceResult getHostOptimizedAdvice(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/host/advice/" + hostId)
                .build(), OptimizedAdviceResult.class);
    }


    @Override
    public void startHost(long hostId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/host/" + hostId + "/start")
                .build(), Void.class);
    }

    @Override
    public void stopHost(long hostId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/host/" + hostId + "/stop")
                .build(), Void.class);
    }

    @Override
    public void restartHost(long hostId, Boolean force) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/host/" + hostId + "/restart")
                .paramIfNotNull("force", force)
                .build(), Void.class);
    }

    @Override
    public AccessAuthResult createHostAccessAuth(Long hostId, Long userId, Long credentialId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/hostAccessAuth/" + hostId + "/" + userId + "/" + credentialId)
                .build(), AccessAuthResult.class);
    }

    @Override
    public HostAccessAuthResult getAccessAuthByHost(Long hostId, Long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/hostAccessAuth/byHost/" + hostId + "/" + userId)
                .build(), HostAccessAuthResult.class);
    }

    @Override
    public AccessAuthResult getAccessAuth(Long accessAuthId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/hostAccessAuth/" + accessAuthId)
                .build(), AccessAuthResult.class);
    }

    @Override
    public void deleteAccessAuthById(Long accessAuthId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/hostAccessAuth/" + accessAuthId)
                .build(), Void.class);
    }

    @Override
    public void deleteAccessAuthByHost(Long hostId, Long userId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/hostAccessAuth/byHost/" + hostId + "/" + userId)
                .build(), Void.class);
    }


    @Override
    public PublicIpHostsResult findPublicHostsByTeam(long teamId, String createTimeFrom, String createTimeTo) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/host/publicHostByTeam/" + teamId)
                .paramIfNotNull("createTimeFrom", createTimeFrom)
                .paramIfNotNull("createTimeTo", createTimeTo)
                .build(), PublicIpHostsResult.class);
    }

    @Override
    public CloudHostResult findHostsWithAgent(Long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/host/findHostsWithAgent")
                .paramShouldNotEmpty("teamId", teamId)
                .build(), CloudHostResult.class);
    }

    @Override
    public CloudHostResult findHostsWithProxy(Long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/host/findHostsWithProxy")
                .paramShouldNotEmpty("teamId", teamId)
                .build(), CloudHostResult.class);
    }

}
