package com.cloudbility.openapi.client;

import com.cloudbility.openapi.ImageApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.enums.zstack.ImageStatusType;
import com.cloudbility.openapi.response.CloudImageListResult;
import com.cloudbility.openapi.vo.ImagePermissionType;
import com.cloudbility.openapi.vo.Scope;

/**
 * @author chjgan
 * @date 2020/1/16 15:07
 */
public class DefaultImageApi extends AbstractApi implements ImageApi {
    public DefaultImageApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public void shareImage(String vendor, String regionId, String imageId, Long cloudId, String accountId, String projectId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/image/share")
                        .paramShouldNotEmpty("vendor", vendor)
                        .paramShouldNotEmpty("regionId", regionId)
                        .paramShouldNotEmpty("imageId", imageId)
                        .paramShouldNotEmpty("cloudId", cloudId)
                        .paramIfNotNull("accountId", accountId)
                        .paramIfNotNull("projectId", projectId)
                        .build(),
                Void.class);
    }

    @Override
    public void cancelShareImage(String vendor, String regionId, String imageId, Long cloudId, String accountId, String projectId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/image/cancelShare")
                        .paramShouldNotEmpty("vendor", vendor)
                        .paramShouldNotEmpty("regionId", regionId)
                        .paramShouldNotEmpty("imageId", imageId)
                        .paramShouldNotEmpty("cloudId", cloudId)
                        .paramIfNotNull("accountId", accountId)
                        .paramIfNotNull("projectId", projectId)
                        .build(),
                Void.class);
    }

    @Override
    public CloudImageListResult listZStackImages(Long cloudId, Scope scope, String imageId, ImageStatusType status) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/image/cloud/" + cloudId + "/zstackList")
                .paramIfNotNull("scope", scope)
                .paramIfNotNull("imageId", imageId)
                .paramIfNotNull("status", status)
                .build(),CloudImageListResult.class);
    }

    @Override
    public void modifyImageSharePermission(String vendor, String provider, String regionId, String imageId, String accountId, String projectId, ImagePermissionType type) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/image/modifyImageSharePermission")
                        .paramShouldNotEmpty("vendor", vendor)
                        .paramShouldNotEmpty("provider", provider)
                        .paramShouldNotEmpty("regionId", regionId)
                        .paramShouldNotEmpty("imageId", imageId)
                        .paramIfNotNull("accountId", accountId)
                        .paramIfNotNull("projectId", projectId)
                        .paramShouldNotEmpty("type", type)
                        .build(),
                Void.class);
    }
}
