package com.cloudbility.openapi.client;

import com.cloudbility.openapi.ImageHostApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiErrorCode;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.enums.zstack.*;
import com.cloudbility.openapi.request.zstack.VMCreateSystemTags;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import com.cloudbility.openapi.vo.monitor.MetricSingletonMap;
import com.cloudbility.openapi.vo.monitor.MonitorMetric;
import org.apache.commons.collections.CollectionUtils;
import org.zstack.sdk.InstanceOfferingInventory;
import org.zstack.sdk.VmInstanceInventory;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static java.util.stream.Collectors.toList;

public class DefaultImageHostApi extends AbstractApi implements ImageHostApi {

    public DefaultImageHostApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public CloudHostVo getHost(Long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/imageHost/" + hostId)
                .build(), CloudHostVo.class);
    }

    @Override
    public Map<Long, String> getHostStatusMap(Long teamId, List<Long> hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/imageHost/findStatus")
                .param("teamId", String.valueOf(teamId))
                .paramIfNotNull("hostIds", hostIds != null && hostIds.size() > 0 ? StringUtil.join(hostIds, ",") : null)
                .build(), GetHostsStatusResult.class)
                .getStatusMap();
    }

    @Override
    public CloudHostResult createHost(Long teamId, Long cloudId, Scope scope, String projectId, String regionId, String zoneId, String imageId, String instanceType, String securityGroupId, String vpcId, String subnetId, String instanceName, String description,
                                      String password, Integer instanceCount, CloudInstanceChargeType instanceChargeType, CloudInternetChargeType internetChargeType, Integer internetMaxBandwidthOut,
                                      Integer period, String periodUnit, SystemDiskVo systemDisk, List<DataDiskVo> dataDisks, String clientToken) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/imageHost/create")
                .paramIfNotNull("teamId", teamId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope == null ? Scope.Portal : scope)
                .paramIfNotNull("projectId", projectId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("zoneId", zoneId)
                .paramShouldNotEmpty("imageId", imageId)
                .paramShouldNotEmpty("instanceType", instanceType)
                .paramIfNotNull("securityGroupId", securityGroupId)
                .paramShouldNotEmpty("vpcId", vpcId)
                .paramShouldNotEmpty("subnetId", subnetId)
                .paramShouldNotEmpty("instanceName", instanceName)
                .paramIfNotNull("description", description)
                .paramIfNotNull("password", password)
                .paramIfNotNull("instanceCount", instanceCount)
                .paramIfNotNull("instanceChargeType", instanceChargeType)
                .paramIfNotNull("internetChargeType", internetChargeType)
                .paramIfNotNull("internetMaxBandwidthOut", internetMaxBandwidthOut)
                .paramIfNotNull("period", period)
                .paramIfNotNull("periodUnit", periodUnit)
                .paramIfNotNull("systemDisk", systemDisk != null ? JsonUtil.toJson(systemDisk) : null)
                .paramIfNotNull("dataDisks", dataDisks != null ? JsonUtil.toJson(dataDisks) : null)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CloudHostResult.class);
    }

    @Override
    public Double inquiryPriceMonthly(String vendor, Long cloudId,  Scope scope, String projectId, String regionId, String zoneId, String imageId, String instanceType, String securityGroupId, String vpcId, String subnetId, String instanceName, String description,
                                      String password, Integer instanceCount, CloudInstanceChargeType instanceChargeType, CloudInternetChargeType internetChargeType, Integer internetMaxBandwidthOut,
                                      Integer period, String periodUnit, SystemDiskVo systemDisk, List<DataDiskVo> dataDisks) throws OpenapiException {
        DoubleResponse response = execute(RequestorBuilder.create()
                .post("/imageHost/inquiryPriceMonthly")
                .paramIfNotNull("vendor", vendor)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope == null ? Scope.Portal : scope)
                .paramIfNotNull("projectId", projectId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("zoneId", zoneId)
                .paramIfNotNull("imageId", imageId)
                .paramShouldNotEmpty("instanceType", instanceType)
                .paramIfNotNull("securityGroupId", securityGroupId)
                .paramIfNotNull("vpcId", vpcId)
                .paramIfNotNull("subnetId", subnetId)
                .paramIfNotNull("instanceName", instanceName)
                .paramIfNotNull("description", description)
                .paramIfNotNull("password", password)
                .paramIfNotNull("instanceCount", instanceCount)
                .paramIfNotNull("instanceChargeType", instanceChargeType)
                .paramIfNotNull("internetChargeType", internetChargeType)
                .paramIfNotNull("internetMaxBandwidthOut", internetMaxBandwidthOut)
                .paramIfNotNull("period", period)
                .paramIfNotNull("periodUnit", periodUnit)
                .paramIfNotNull("systemDisk", systemDisk != null ? JsonUtil.toJson(systemDisk) : null)
                .paramIfNotNull("dataDisks", dataDisks != null ? JsonUtil.toJson(dataDisks) : null)
                .build(), DoubleResponse.class);
        return response.getValue();
    }

    @Override
    public void destroyHost(long hostId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/imageHost/" + hostId + "/destroy")
                .build(), Void.class);
    }

    @Override
    public void startHost(long hostId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/start")
                .build(), Void.class);
    }

    @Override
    public void stopHost(long hostId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/stop")
                .build(), Void.class);
    }

    @Override
    public void restartHost(long hostId, Boolean force) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/restart")
                .paramIfNotNull("force", force)
                .build(), Void.class);
    }

    @Override
    public void renewHost(long hostId, int period, RenewPeriodUnit periodUnit, String clientToken) throws OpenapiException {
        execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/renew")
                .paramShouldNotEmpty("period", period)
                .paramShouldNotEmpty("periodUnit", periodUnit)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public CloudHostVo modifyHostSpec(long hostId, String instanceType, String clientToken) throws OpenapiException {
        CloudHostVo cloudHostVo = execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/modifySpec")
                .paramShouldNotEmpty("instanceType", instanceType)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CloudHostVo.class);
        return cloudHostVo;
    }

    @Override
    public DataDiskVo resizeDisk(long hostId, String diskId, int newSize, String clientToken) throws OpenapiException {
        DataDiskVo dataDiskVo = execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/resizeDisk")
                .paramShouldNotEmpty("diskId", diskId)
                .paramShouldNotEmpty("newSize", newSize)
                .paramIfNotNull("clientToken", clientToken)
                .build(), DataDiskVo.class);
        return dataDiskVo;
    }

    @Override
    public CloudHostVo expandHostAndDisk(long hostId, String instanceType, String diskId, Integer newSize, String clientToken) throws OpenapiException {
        CloudHostVo cloudHostVo = execute(RequestorBuilder.create()
                .post("/imageHost/" + hostId + "/expandHostAndDisk")
                .paramIfNotNull("instanceType", instanceType)
                .paramIfNotNull("diskId", diskId)
                .paramIfNotNull("newSize", newSize)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CloudHostVo.class);
        return cloudHostVo;
    }

    @Override
    public CloudHostResult createZStackHost(String clientToken, Long teamId, Long cloudId, Scope scope, String zoneId, String clusterId, String hostId,
                                            String imageId, String instanceOfferingId, List<String> networkIds, String defaultNetworkId,
                                            String instanceName, String description, String hostName, String rootDiskOfferingId,
                                            Long rootDiskSize, String primaryStorageIdForRootVolume, VolumeProvisioningStrategyType rootVolumeProvisioningStrategy,
                                            List<String> dataDiskOfferingIds, VolumeProvisioningStrategyType dataVolumeProvisioningStrategy,
                                            VMCreateSystemTags systemTags, VMCreateStrategyType createStrategy, Integer instanceCount) throws OpenapiException {
        String networkIdsString = CollectionUtils.isEmpty(networkIds) ? null : String.join(",", networkIds);
        String dataDiskOfferingIdsString = CollectionUtils.isEmpty(dataDiskOfferingIds) ? null : String.join(",", dataDiskOfferingIds);
        return execute(RequestorBuilder.create()
                .post("/imageHost/cloud/" + cloudId + "/zstackCreate")
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .paramIfNotNull("teamId", teamId)
                .paramIfNotNull("zoneId", zoneId)
                .paramIfNotNull("clusterId", clusterId)
                .paramIfNotNull("hostId", hostId)
                .paramShouldNotEmpty("imageId", imageId)
                .paramShouldNotEmpty("instanceOfferingId", instanceOfferingId)
                .paramShouldNotEmpty("networkIds", networkIdsString)
                .paramIfNotNull("defaultNetworkId", defaultNetworkId)
                .paramShouldNotEmpty("instanceName", instanceName)
                .paramIfNotNull("description", description)
                .paramIfNotNull("hostName", hostName)
                .paramIfNotNull("rootDiskOfferingId", rootDiskOfferingId)
                .paramIfNotNull("rootDiskSize", rootDiskSize)
                .paramIfNotNull("primaryStorageIdForRootVolume", primaryStorageIdForRootVolume)
                .paramIfNotNull("rootVolumeProvisioningStrategy", rootVolumeProvisioningStrategy)
                .paramIfNotNull("dataDiskOfferingIds", dataDiskOfferingIdsString)
                .paramIfNotNull("dataVolumeProvisioningStrategy", dataVolumeProvisioningStrategy)
                .paramIfNotNull("systemTags", systemTags != null ? JsonUtil.toJson(systemTags) : null)
                .paramIfNotNull("createStrategy", createStrategy)
                .paramIfNotNull("instanceCount", instanceCount)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CloudHostResult.class);
    }

    @Override
    public ZStackHostSpecVo createZStackHostSpec(String clientToken, Long cloudId, Scope scope, String name, int cpuNum, long memorySizeInMb,
                                                 HostAllocatorStrategyType allocatorStrategy, String description, Integer maxInstancePerHost,
                                                 Long volumeTotalBandwidthInMb, Long volumeReadBandwidthInMb, Long volumeWriteBandwidthInMb,
                                                 Long networkInboundBandwidthInKb, Long networkOutboundBandwidthInKb,
                                                 HostAllocatorStrategyMode minimumCPUUsageHostAllocatorStrategy,
                                                 HostAllocatorStrategyMode minimumMemoryUsageHostAllocatorStrategy) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/imageHost/cloud/" + cloudId + "/zstackSpec")
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("cpuNum", cpuNum)
                .paramShouldNotEmpty("memorySizeInMb", memorySizeInMb)
                .paramShouldNotEmpty("allocatorStrategy", allocatorStrategy)
                .paramIfNotNull("description", description)
                .paramIfNotNull("maxInstancePerHost", maxInstancePerHost)
                .paramIfNotNull("volumeTotalBandwidthInMb", volumeTotalBandwidthInMb)
                .paramIfNotNull("volumeReadBandwidthInMb", volumeReadBandwidthInMb)
                .paramIfNotNull("volumeWriteBandwidthInMb", volumeWriteBandwidthInMb)
                .paramIfNotNull("networkInboundBandwidthInKb", networkInboundBandwidthInKb)
                .paramIfNotNull("networkOutboundBandwidthInKb", networkOutboundBandwidthInKb)
                .paramIfNotNull("minimumCPUUsageHostAllocatorStrategy", minimumCPUUsageHostAllocatorStrategy)
                .paramIfNotNull("minimumMemoryUsageHostAllocatorStrategy", minimumMemoryUsageHostAllocatorStrategy)
                .paramIfNotNull("clientToken", clientToken)
                .build(), ZStackHostSpecVo.class);
    }

    @Override
    public ZStackHostSpecListResult listZStackHostSpecs(Long cloudId, Scope scope, Set<String> instanceOfferingIds,
                                                        String name, Integer cpuNum, Long memorySizeInMb,
                                                        HostAllocatorStrategyType allocatorStrategy,
                                                        UsingStateType state) throws OpenapiException {
        String instanceOfferingIdsString = CollectionUtils.isEmpty(instanceOfferingIds) ? null : String.join(",", instanceOfferingIds);
        return execute(RequestorBuilder.create()
                .get("/imageHost/cloud/" + cloudId + "/zstackSpecs")
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .paramIfNotNull("instanceOfferingIds", instanceOfferingIdsString)
                .paramIfNotNull("name", name)
                .paramIfNotNull("cpuNum", cpuNum)
                .paramIfNotNull("memorySizeInMb", memorySizeInMb)
                .paramIfNotNull("allocatorStrategy", allocatorStrategy)
                .paramIfNotNull("usingState", state)
                .build(), ZStackHostSpecListResult.class);
    }

    @Override
    public ZStackPhysicalHostListResult listZStackPhysicalHosts(Long cloudId, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/imageHost/cloud/" + cloudId + "/zstackPhysicalHosts")
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .build(), ZStackPhysicalHostListResult.class);
    }

    @Override
    public ZStackPhysicalResourceSummary getZStackPhysicalResourceSummary(Long cloudId, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/imageHost/cloud/" + cloudId + "/zstackPhysicalResourceSummary")
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .build(), ZStackPhysicalResourceSummary.class);
    }

    @Override
    public void modifyZStackDefaultNetwork(String clientToken, Long hostId, String defaultNetworkId, Boolean reboot) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/imageHost/" + hostId + "/zstackDefaultNetwork")
                .paramShouldNotEmpty("defaultNetworkId", defaultNetworkId)
                .paramIfNotNull("reboot", reboot)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public VmInstanceInventory attachZStackL3NetworkToHost(String clientToken, Long hostId, String l3NetworkId, String staticIp) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/imageHost/" + hostId + "/zstackAttachL3Network/" + l3NetworkId)
                .paramIfNotNull("staticIp", staticIp)
                .paramIfNotNull("clientToken", clientToken)
                .build(), VmInstanceInventory.class);
    }

    @Override
    public InstanceOfferingInventory changeZStackInstanceOfferingState(String clientToken, Long cloudId, Scope scope, String instanceOfferingId, UsingStateEventType stateEvent) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/imageHost/zstackInstanceOffering/" + instanceOfferingId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .paramShouldNotEmpty("stateEvent", stateEvent)
                .paramIfNotNull("clientToken", clientToken)
                .build(), InstanceOfferingInventory.class);
    }

    @Override
    public void deleteZStackInstanceOffering(String clientToken, Long cloudId, Scope scope, String instanceOfferingId, DeleteModeType deleteMode) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/imageHost/zstackInstanceOffering/" + instanceOfferingId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .paramIfNotNull("deleteMode", deleteMode)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public ZStackVmNicInfoResult getZStackVmNicInfo(Long cloudId, Scope scope, String instanceId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/imageHost/zstackVmNic/" + instanceId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope == null ? Scope.Console : scope)
                .build(), ZStackVmNicInfoResult.class);
    }

    @Override
    public MetricSingletonMap getHostLastMetricData(long hostId, MonitorMetric... monitorMetric) throws OpenapiException {
        if (monitorMetric == null || monitorMetric.length == 0) {
            throw new OpenapiException(OpenapiErrorCode.InvalidParameter("monitorMetric not set"));
        }
        String metrics = StringUtil.join(Arrays.asList(monitorMetric).stream().map(m -> m.name()).collect(toList()), ",");
        return execute(RequestorBuilder.create().get("/imageHost/" + hostId + "/monitor")
                        .paramShouldNotEmpty("metrics", metrics),
                MetricSingletonMap.class);
    }
}
