package com.cloudbility.openapi.client;

import com.cloudbility.openapi.LogApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.request.GetHostLogRequest;
import com.cloudbility.openapi.request.GetSqlLogRequest;
import com.cloudbility.openapi.response.AddLogResult;
import com.cloudbility.openapi.response.DbSqlLogResult;
import com.cloudbility.openapi.response.HostLogResult;
import com.cloudbility.openapi.response.LogResult;
import com.cloudbility.openapi.vo.AddLogRequest;
import com.cloudbility.openapi.vo.Order;

import java.util.Date;

public class DefaultLogApi extends AbstractApi implements LogApi {
    public DefaultLogApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public HostLogResult getLog(long teamId, int page, int size, GetHostLogRequest request) throws OpenapiException {
        return getHostLog(teamId, page, size, Order.DESC, request);
    }

    @Override
    public HostLogResult getHostLog(long teamId, int page, int size, Order order, GetHostLogRequest request) throws OpenapiException {
        if (request == null) {
            request = new GetHostLogRequest();
        }
        return execute(RequestorBuilder.create()
                .get("/log/host")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("actionType", request.getActionType())
                .paramIfNotNull("page", page)
                .paramIfNotNull("size", size)
                .paramIfNotNull("cloudId", request.getCloudId())
                .paramIfNotNull("status", request.getStatus())
                .paramIfNotNull("actorId", request.getActorId())
                .paramIfNotNull("hostId", request.getHostId())
                .paramIfNotNull("startTime", request.getStartTime() != null ? DateUtil.formatDate(request.getStartTime()) : null)
                .paramIfNotNull("endTime", request.getEndTime() != null ? DateUtil.formatDate(request.getEndTime()) : null)
                .paramIfNotNull("order", order)
                .paramIfNotNull("filterRunningSession", request.getFilterRunningSession())
                .build(), HostLogResult.class);
    }

    @Override
    public DbSqlLogResult getSqlLog(long teamId, int page, int size, Order order, GetSqlLogRequest request) throws OpenapiException {
        if (request == null) {
            request = new GetSqlLogRequest();
        }
        return execute(RequestorBuilder.create()
                .get("/log/db/sql")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("page", page)
                .paramIfNotNull("size", size)
                .paramIfNotNull("actorId", request.getActorId())
                .paramIfNotNull("dbId", request.getDbId())
                .paramIfNotNull("sessionId", request.getSessionId())
                .paramIfNotNull("success", request.getSuccess())
                .paramIfNotNull("startTime", request.getStartTime() != null ? DateUtil.formatDate(request.getStartTime()) : null)
                .paramIfNotNull("endTime", request.getEndTime() != null ? DateUtil.formatDate(request.getEndTime()) : null)
                .paramIfNotNull("order", order)
                .build(), DbSqlLogResult.class);
    }

    @Override
    public String addLog(AddLogRequest request) throws OpenapiException {
        AddLogResult result = execute(RequestorBuilder.create()
                .post("/log")
                .body(request)
                .build(), AddLogResult.class);
        return result.getId();
    }

    @Override
    public LogResult getLog(long teamId, String cluster, int page, int size) throws OpenapiException {
        return getLog(teamId, cluster, null, null, null, null, null, null, page, size);
    }

    @Override
    public LogResult getLog(long teamId, String cluster, String action, Long actorId, String status, Date startTime, Date endTime, Order order, int page, int size) throws OpenapiException {
        LogResult result = execute(RequestorBuilder.create()
                .get("/log")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("cluster", cluster)
                .paramIfNotNull("action", action)
                .paramIfNotNull("status", status)
                .paramIfNotNull("startTime", startTime != null ? DateUtil.formatDate(startTime) : null)
                .paramIfNotNull("endTime", endTime != null ? DateUtil.formatDate(endTime) : null)
                .paramShouldNotEmpty("order", order != null ? order : Order.DESC)
                .paramShouldNotEmpty("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), LogResult.class);

        return result;
    }
}
