package com.cloudbility.openapi.client;

import com.cloudbility.openapi.NetworkApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.enums.zstack.*;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import org.apache.commons.collections.CollectionUtils;
import org.zstack.sdk.*;

import java.util.List;
import java.util.Set;

/**
 * Created by lidonghao on 2020/8/20.
 */
public class DefaultNetworkApi extends AbstractApi implements NetworkApi {

    public DefaultNetworkApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public SecurityGroupVo createSecurityGroup(Long cloudId, String regionId, String vpcId, String securityGroupName, String description, List<SecurityGroupRuleVo> securityGroupRules) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/createSecurityGroup")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramShouldNotEmpty("securityGroupName", securityGroupName)
                .paramIfNotNull("vpcId", vpcId)
                .paramIfNotNull("description", description)
                .paramIfNotNull("securityGroupRules", securityGroupRules != null && securityGroupRules.size() > 0 ? JsonUtil.toJson(securityGroupRules) : null)
                .build(), SecurityGroupVo.class);
    }

    @Override
    public SecurityGroupResult listSecurityGroups(Long cloudId, String regionId, String vpcId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/network/securityGroups")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("vpcId", vpcId)
                .build(), SecurityGroupResult.class);
    }

    @Override
    public SecurityGroupRuleResult listSecurityGroupRules(Long cloudId, String regionId, String securityGroupId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/network/securityGroupRules")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramShouldNotEmpty("securityGroupId", securityGroupId)
                .build(), SecurityGroupRuleResult.class);
    }

    @Override
    public void joinHostToSecurityGroup(Long cloudId, Long hostId, String securityGroupId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/joinHostToSecurityGroup")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("hostId", hostId)
                .paramShouldNotEmpty("securityGroupId", securityGroupId)
                .build(), Void.class);
    }

    @Override
    public void removeHostFromSecurityGroup(Long cloudId, Long hostId, String securityGroupId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/removeHostFromSecurityGroup")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("hostId", hostId)
                .paramShouldNotEmpty("securityGroupId", securityGroupId)
                .build(), Void.class);
    }

    @Override
    public SubnetVo createSubnet(Long cloudId, String regionId, String zoneId, String instanceType, String cidrBlock, String vpcId, String subnetName, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/createSubnet")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("zoneId", zoneId)
                .paramIfNotNull("instanceType", instanceType)
                .paramShouldNotEmpty("cidrBlock", cidrBlock)
                .paramShouldNotEmpty("vpcId", vpcId)
                .paramShouldNotEmpty("subnetName", subnetName)
                .paramIfNotNull("description", description)
                .build(), SubnetVo.class);
    }

    @Override
    public ListVpcsResult getVpcsByCloud(Long cloudId, String regionId, String zoneId, String projectId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/network/vpcs")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("regionId", regionId)
                .paramIfNotNull("zoneId", zoneId)
                .paramIfNotNull("projectId", projectId)
                .build(), ListVpcsResult.class);
    }

    @Override
    public CreateZStackL2VxLanPoolResult createZStackL2VxLanPool(String clientToken, Long cloudId, Scope scope, String name, String description, Integer startVni, Integer endVni, String clusterId, String cidr) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL2/vxLanPool")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .paramIfNotNull("startVni", startVni)
                .paramIfNotNull("endVni", endVni)
                .paramIfNotNull("clusterId", clusterId)
                .paramIfNotNull("cidr", cidr)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CreateZStackL2VxLanPoolResult.class);
    }

    @Override
    public CreateZStackL2NoVLanNetworkResult createZStackL2NoVLanNetwork(String clientToken,
                                                                         Long cloudId,
                                                                         Scope scope,
                                                                         String name,
                                                                         String physicalInterface,
                                                                         String description,
                                                                         String clusterId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL2/noVLan")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("physicalInterface", physicalInterface)
                .paramIfNotNull("description", description)
                .paramIfNotNull("clusterId", clusterId)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CreateZStackL2NoVLanNetworkResult.class);
    }

    @Override
    public CreateZStackL2VLanNetworkResult createZStackL2VLanNetwork(String clientToken,
                                                                     Long cloudId,
                                                                     Scope scope,
                                                                     String name,
                                                                     Integer vLanId,
                                                                     String physicalInterface,
                                                                     String description,
                                                                     String clusterId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL2/vLan")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("vLanId", vLanId)
                .paramShouldNotEmpty("physicalInterface", physicalInterface)
                .paramIfNotNull("description", description)
                .paramIfNotNull("clusterId", clusterId)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CreateZStackL2VLanNetworkResult.class);
    }

    @Override
    public CreateZStackL2VxLanNetworkResult createZStackL2VxLanNetwork(String clientToken,
                                                                       Long cloudId,
                                                                       Scope scope,
                                                                       String name,
                                                                       String vxLanPoolId,
                                                                       String description,
                                                                       Integer vni) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL2/vxLan")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("vxLanPoolId", vxLanPoolId)
                .paramIfNotNull("description", description)
                .paramIfNotNull("vni", vni)
                .paramIfNotNull("clientToken", clientToken)
                .build(), CreateZStackL2VxLanNetworkResult.class);
    }

    @Override
    public L2NetworkInventory attachZStackL2NetworkToCluster(String clientToken,
                                                             Long cloudId,
                                                             Scope scope,
                                                             String networkId,
                                                             String clusterId,
                                                             String cidr) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL2/" + networkId + "/attachTo/" + clusterId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("cidr", cidr)
                .paramIfNotNull("clientToken", clientToken)
                .build(), L2NetworkInventory.class);
    }


    @Override
    public ListZStackL2VxLanPoolsResult listZStackL2VxLanPools(Long cloudId, Scope scope, Set<String> networkPoolIds) throws OpenapiException {
        String poolIdsString = CollectionUtils.isEmpty(networkPoolIds) ? null : String.join(",", networkPoolIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackL2/vxLanPools")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("networkPoolIds", poolIdsString)
                .build(), ListZStackL2VxLanPoolsResult.class);
    }

    @Override
    public ListZStackL2VLanNetworksResult listZStackL2VLanNetworks(Long cloudId,
                                                                   Scope scope,
                                                                   Set<String> networkIds,
                                                                   String physicalInterface) throws OpenapiException {
        String networkIdsString = CollectionUtils.isEmpty(networkIds) ? null : String.join(",", networkIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackL2/vLans")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("networkIds", networkIdsString)
                .paramIfNotNull("physicalInterface", physicalInterface)
                .build(), ListZStackL2VLanNetworksResult.class);
    }

    @Override
    public ListZStackL2VxLanNetworksResult listZStackL2VxLanNetworks(Long cloudId, Scope scope, Set<String> networkIds) throws OpenapiException {
        String networkIdsString = CollectionUtils.isEmpty(networkIds) ? null : String.join(",", networkIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackL2/vxLans")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("networkIds", networkIdsString)
                .build(), ListZStackL2VxLanNetworksResult.class);
    }

    @Override
    public ListZStackL2NetworksResult listZStackL2Networks(Long cloudId,
                                                           Scope scope,
                                                           Set<String> networkIds,
                                                           String physicalInterface,
                                                           String startIp) throws OpenapiException {
        String networkIdsString = CollectionUtils.isEmpty(networkIds) ? null : String.join(",", networkIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackL2/networks")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("networkIds", networkIdsString)
                .paramIfNotNull("physicalInterface", physicalInterface)
                .paramIfNotNull("startIp", startIp)
                .build(), ListZStackL2NetworksResult.class);
    }

    @Override
    public void deleteZStackL2Network(String clientToken, Long cloudId, Scope scope, String networkId, DeleteModeType deleteMode) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/network/zstackL2/" + networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("deleteMode", deleteMode)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public ListZStackL3NetworksResult listZStackL3Networks(Long cloudId,
                                                           Scope scope,
                                                           Set<String> l3NetworkIds,
                                                           String l3NetworkName,
                                                           String l2NetworkId,
                                                           UsingStateType usingState,
                                                           L3NetworkType networkType) throws OpenapiException {
        String l3NetworkIdsString = CollectionUtils.isEmpty(l3NetworkIds) ? null : String.join(",", l3NetworkIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackL3/networks")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("l3NetworkIds", l3NetworkIdsString)
                .paramIfNotNull("l3NetworkName", l3NetworkName)
                .paramIfNotNull("l2NetworkId", l2NetworkId)
                .paramIfNotNull("usingState", usingState)
                .paramIfNotNull("networkType", networkType)
                .build(), ListZStackL3NetworksResult.class);
    }

    @Override
    public void deleteZStackL3Network(String clientToken, Long cloudId, Scope scope, Set<String> networkIds, DeleteModeType deleteMode) throws OpenapiException {
        String l3NetworkIdsString = CollectionUtils.isEmpty(networkIds) ? null : String.join(",", networkIds);
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/network/zstackL3")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("networkIds", l3NetworkIdsString)
                .paramIfNotNull("deleteMode", deleteMode)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public void deleteZStackVirtualNetwork(String clientToken, Long cloudId, Scope scope, String l3NetworkId, Boolean safeDeleteL2Network) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/network/zstack/virtualNetwork/" + l3NetworkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("safeDeleteL2Network", safeDeleteL2Network)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public ListZStackNetworkServiceProvidersResult listZStackNetworkServiceProviders(Long cloudId, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/network/zstackServiceProviders")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .build(), ListZStackNetworkServiceProvidersResult.class);
    }

    @Override
    public ListZStackIpRangesResult listZStackIpRanges(Long cloudId, Scope scope, Set<String> ipRangeIds, String l2networkId, String l3networkId) throws OpenapiException {
        String ipRangeIdsString = CollectionUtils.isEmpty(ipRangeIds) ? null : String.join(",", ipRangeIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackIpRanges")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("ipRangeIds", ipRangeIdsString)
                .paramIfNotNull("l2networkId", l2networkId)
                .paramIfNotNull("l3networkId", l3networkId)
                .build(), ListZStackIpRangesResult.class);
    }

    @Override
    public GetIpAddressCapacityResult getZStackIpAddressCapacity(Long cloudId, Scope scope, Set<String> l3networkIds, Set<String> ipRangeIds) throws OpenapiException {
        String l3networkIdsString = CollectionUtils.isEmpty(l3networkIds) ? null : String.join(",", l3networkIds);
        String ipRangeIdsString = CollectionUtils.isEmpty(ipRangeIds) ? null : String.join(",", ipRangeIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackIpAddressCapacity")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("l3networkIds", l3networkIdsString)
                .paramIfNotNull("ipRangeIds", ipRangeIdsString)
                .build(), GetIpAddressCapacityResult.class);
    }

    @Override
    public L3NetworkInventory addZStackDnsToL3Network(String clientToken, Long cloudId, Scope scope, String dns, String l3networkId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackDns/" + dns + "/AddToL3Network/" + l3networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("clientToken", clientToken)
                .build(), L3NetworkInventory.class);
    }

    @Override
    public L3NetworkInventory removeZStackDnsFromL3Network(String clientToken, Long cloudId, Scope scope, String dns, String l3networkId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackDns/" + dns + "/RemoveFromL3Network/" + l3networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("clientToken", clientToken)
                .build(), L3NetworkInventory.class);
    }

    @Override
    public IpRangeInventory addZStackIpRangeToL3Network(String clientToken, Long cloudId, Scope scope, String l3networkId, String name,
                                                        String startIp, String endIp, String netmask, String gateway, String dhcpIp, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackAddIpRangeToL3Network/" + l3networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("startIp", startIp)
                .paramShouldNotEmpty("endIp", endIp)
                .paramShouldNotEmpty("netmask", netmask)
                .paramShouldNotEmpty("gateway", gateway)
                .paramIfNotNull("dhcpIp", dhcpIp)
                .paramIfNotNull("description", description)
                .paramIfNotNull("clientToken", clientToken)
                .build(), IpRangeInventory.class);
    }

    @Override
    public IpRangeInventory addZStackIpv6RangeToL3Network(String clientToken, Long cloudId, Scope scope, String l3networkId, String name,
                                                          String startIp, String endIp, String gateway, Integer prefixLen, String dhcpIp, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackAddIpv6RangeToL3Network/" + l3networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("startIp", startIp)
                .paramShouldNotEmpty("endIp", endIp)
                .paramShouldNotEmpty("gateway", gateway)
                .paramShouldNotEmpty("prefixLen", prefixLen)
                .paramIfNotNull("dhcpIp", dhcpIp)
                .paramIfNotNull("description", description)
                .paramIfNotNull("clientToken", clientToken)
                .build(), IpRangeInventory.class);
    }

    @Override
    public IpRangeInventory addZStackIpRangeByCidrForL3Network(String clientToken, Long cloudId, Scope scope, String l3networkId, String name,
                                                               String cidr, String dhcpIp, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackAddIpRangeByCidrToL3Network/" + l3networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("cidr", cidr)
                .paramIfNotNull("dhcpIp", dhcpIp)
                .paramIfNotNull("description", description)
                .paramIfNotNull("clientToken", clientToken)
                .build(), IpRangeInventory.class);
    }

    @Override
    public IpRangeInventory addZStackIpv6RangeByCidrForL3Network(String clientToken, Long cloudId, Scope scope, String l3networkId, String name,
                                                                 String cidr, Ipv6AddressModeType ipv6AddressMode, String dhcpIp, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackAddIpv6RangeByCidrToL3Network/" + l3networkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("cidr", cidr)
                .paramShouldNotEmpty("ipv6AddressMode", ipv6AddressMode)
                .paramIfNotNull("dhcpIp", dhcpIp)
                .paramIfNotNull("description", description)
                .paramIfNotNull("clientToken", clientToken)
                .build(), IpRangeInventory.class);
    }

    @Override
    public void deleteZStackIpRange(String clientToken, Long cloudId, Scope scope, String ipRangeId, DeleteModeType deleteMode) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/network/zstackIpRange/" + ipRangeId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("deleteMode", deleteMode)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    @Override
    public L3NetworkInventory createZStackL3PublicNetwork(String clientToken,
                                                          Long cloudId,
                                                          Scope scope,
                                                          String l2NetworkId,
                                                          IpVersion ipVersion,
                                                          String name,
                                                          String description,
                                                          String dnsDomain,
                                                          String mirrorNetworkId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL3/publicNetwork")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("l2NetworkId", l2NetworkId)
                .paramShouldNotEmpty("ipVersion", ipVersion)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .paramIfNotNull("dnsDomain", dnsDomain)
                .paramIfNotNull("mirrorNetworkId", mirrorNetworkId)
                .paramIfNotNull("clientToken", clientToken)
                .build(), L3NetworkInventory.class);
    }

    @Override
    public L3NetworkInventory createZStackL3PrivateNetwork(String clientToken,
                                                           Long cloudId,
                                                           Scope scope,
                                                           String l2NetworkId,
                                                           String name,
                                                           String description,
                                                           String dnsDomain,
                                                           String mirrorNetworkId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL3/privateNetwork")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("l2NetworkId", l2NetworkId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .paramIfNotNull("dnsDomain", dnsDomain)
                .paramIfNotNull("mirrorNetworkId", mirrorNetworkId)
                .paramIfNotNull("clientToken", clientToken)
                .build(), L3NetworkInventory.class);
    }

    @Override
    public L3NetworkInventory createZStackL3SystemNetwork(String clientToken,
                                                          Long cloudId,
                                                          Scope scope,
                                                          String l2NetworkId,
                                                          String name,
                                                          String description,
                                                          String dnsDomain) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackL3/privateNetwork")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("l2NetworkId", l2NetworkId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .paramIfNotNull("dnsDomain", dnsDomain)
                .paramIfNotNull("clientToken", clientToken)
                .build(), L3NetworkInventory.class);
    }

    @Override
    public ZStackVirtualNetworkCreateResult createZStackL3Network(String clientToken,
                                                                  Long cloudId,
                                                                  Scope scope,
                                                                  String l2NetworkName,
                                                                  String name,
                                                                  String physicalInterface,
                                                                  String startIp,
                                                                  String endIp,
                                                                  String gateway,
                                                                  Integer vLanId,
                                                                  String netmask,
                                                                  Integer prefixLen,
                                                                  L3NetworkType l3NetworkType,
                                                                  L3NetworkCategoryType l3NetworkCategory,
                                                                  IpVersion ipVersion,
                                                                  String dhcpIp,
                                                                  String dnsDomain) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstack/virtualNetwork")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("l2NetworkName", l2NetworkName)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("physicalInterface", physicalInterface)
                .paramShouldNotEmpty("startIp", startIp)
                .paramShouldNotEmpty("endIp", endIp)
                .paramShouldNotEmpty("gateway", gateway)
                .paramIfNotNull("vLanId", vLanId)
                .paramIfNotNull("netmask", netmask)
                .paramIfNotNull("prefixLen", prefixLen)
                .paramIfNotNull("l3NetworkType", l3NetworkType)
                .paramIfNotNull("l3NetworkCategoryType", l3NetworkCategory)
                .paramIfNotNull("ipVersion", ipVersion)
                .paramIfNotNull("dhcpIp", dhcpIp)
                .paramIfNotNull("dnsDomain", dnsDomain)
                .paramIfNotNull("clientToken", clientToken)
                .build(), ZStackVirtualNetworkCreateResult.class);
    }

    @Override
    public VirtualRouterOfferingInventory createZStackVirtualRouterOffering(String clientToken,
                                                                            Long cloudId,
                                                                            Scope scope,
                                                                            String managementL3NetworkId,
                                                                            String publicL3NetworkId,
                                                                            String imageId,
                                                                            String name,
                                                                            Integer cpuNum,
                                                                            Long memorySizeInMb,
                                                                            String description,
                                                                            Boolean isDefault) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackVirtualRouterOffering")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("managementL3NetworkId", managementL3NetworkId)
                .paramShouldNotEmpty("publicL3NetworkId", publicL3NetworkId)
                .paramShouldNotEmpty("imageId", imageId)
                .paramShouldNotEmpty("name", name)
                .paramShouldNotEmpty("cpuNum", cpuNum)
                .paramShouldNotEmpty("memorySizeInMb", memorySizeInMb)
                .paramIfNotNull("description", description)
                .paramIfNotNull("isDefault", isDefault)
                .paramIfNotNull("clientToken", clientToken)
                .build(), VirtualRouterOfferingInventory.class);
    }

    @Override
    public ListZStackVirtualRouterOfferingsResult listZStackVirtualRouterOfferings(Long cloudId,
                                                                                   Scope scope,
                                                                                   Set<String> virtualRouterTypeIds,
                                                                                   String imageId,
                                                                                   String managementL3NetworkId,
                                                                                   UsingStateType usingState,
                                                                                   Integer cpuNum,
                                                                                   Long memorySizeInMb) throws OpenapiException {
        String virtualRouterTypeIdsString = CollectionUtils.isEmpty(virtualRouterTypeIds) ? null : String.join(",", virtualRouterTypeIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackVirtualRouterOfferings")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("virtualRouterTypeIds", virtualRouterTypeIdsString)
                .paramIfNotNull("imageId", imageId)
                .paramIfNotNull("managementL3NetworkId", managementL3NetworkId)
                .paramIfNotNull("usingState", usingState)
                .paramIfNotNull("cpuNum", cpuNum)
                .paramIfNotNull("memorySizeInMb", memorySizeInMb)
                .build(), ListZStackVirtualRouterOfferingsResult.class);
    }

    @Override
    public VirtualRouterVmInventory createZStackVpcRouterWithDns(String clientToken,
                                                                 Long cloudId,
                                                                 Scope scope,
                                                                 String virtualRouterOfferingId,
                                                                 String name,
                                                                 String description,
                                                                 String dns,
                                                                 String staticVip,
                                                                 String haGroupId,
                                                                 String clusterId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackVpcRouterWithDns")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("virtualRouterOfferingId", virtualRouterOfferingId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .paramIfNotNull("dns", dns)
                .paramIfNotNull("staticVip", staticVip)
                .paramIfNotNull("haGroupId", haGroupId)
                .paramIfNotNull("clusterId", clusterId)
                .paramIfNotNull("clientToken", clientToken)
                .build(), VirtualRouterVmInventory.class);
    }

    @Override
    public VpcRouterVmInventory operateZStackDnsToVpcRouter(String clientToken, Long cloudId, Scope scope, String vpcRouterId, String dns, VpcRouterDnsActionType action) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/network/zstackVpcRouter/" + vpcRouterId + "/Dns/" + dns)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramShouldNotEmpty("action", action)
                .paramIfNotNull("clientToken", clientToken)
                .build(), VpcRouterVmInventory.class);
    }

    @Override
    public ListZStackVpcRoutersResult listZStackVpcRouters(Long cloudId,
                                                           Scope scope,
                                                           Set<String> vpcRouterIds,
                                                           String name,
                                                           String clusterId,
                                                           String defaultL3NetworkId,
                                                           String defaultRouteL3NetworkId,
                                                           String hostId,
                                                           String imageId,
                                                           String managementNetworkId,
                                                           String publicNetworkId,
                                                           String instanceOfferingId,
                                                           InstanceStateType state) throws OpenapiException {
        String vpcRouterIdsString = CollectionUtils.isEmpty(vpcRouterIds) ? null : String.join(",", vpcRouterIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackVpcRouters")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("vpcRouterIds", vpcRouterIdsString)
                .paramIfNotNull("name", name)
                .paramIfNotNull("clusterId", clusterId)
                .paramIfNotNull("defaultL3NetworkId", defaultL3NetworkId)
                .paramIfNotNull("defaultRouteL3NetworkId", defaultRouteL3NetworkId)
                .paramIfNotNull("hostId", hostId)
                .paramIfNotNull("imageId", imageId)
                .paramIfNotNull("managementNetworkId", managementNetworkId)
                .paramIfNotNull("publicNetworkId", publicNetworkId)
                .paramIfNotNull("instanceOfferingId", instanceOfferingId)
                .paramIfNotNull("state", state)
                .build(), ListZStackVpcRoutersResult.class);
    }

    @Override
    public void deleteZStackVpcRouter(String clientToken, Long cloudId, Scope scope, String vpcRouterId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/network/zstackVpcRouter/" + vpcRouterId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("clientToken", clientToken)
                .build(), Void.class);
    }

    /**
     * 获取已占用的L2网络vlanId列表
     *
     * @param cloudId   云账户Id
     * @param scope
     * @param networkIds
     * @param physicalInterface
     * @return SetZStackL2VLanIdsResult
     * @throws OpenapiException
     */
    @Override
    public SetZStackL2VLanIdsResult getZStackExistsL2VLanIdSet(Long cloudId, Scope scope, Set<String> networkIds, String physicalInterface) throws OpenapiException {
        String networkIdsString = CollectionUtils.isEmpty(networkIds) ? null : String.join(",", networkIds);
        return execute(RequestorBuilder.create()
                .get("/network/zstackL2/existsVLanIds")
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("networkIds", networkIdsString)
                .paramIfNotNull("physicalInterface", physicalInterface)
                .build(), SetZStackL2VLanIdsResult.class);
    }


    @Override
    public ListZStackL3NetworkIpStatisticResult listZStackL3NetworkIpStatistics(String clientToken, Long cloudId, Scope scope,
                                                                                String l3NetworkId,
                                                                                IpResourceType ipResourceType,
                                                                                String ipAddress) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/network/zstackIpStatistic/" + l3NetworkId)
                .paramShouldNotEmpty("cloudId", cloudId)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("ipAddress", ipAddress)
                .paramIfNotNull("ipResourceType", ipResourceType)
                .paramIfNotNull("clientToken", clientToken)
                .build(), ListZStackL3NetworkIpStatisticResult.class);
    }

}
