package com.cloudbility.openapi.client;

import com.cloudbility.openapi.NoticeApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.response.NoticeInfo;
import com.cloudbility.openapi.response.NoticeListResult;
import com.cloudbility.openapi.response.NoticeTagResult;
import com.cloudbility.openapi.vo.*;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Date;

public class DefaultNoticeApi extends AbstractApi implements NoticeApi {

    public DefaultNoticeApi(OpenapiClient client) {
        super(client);
    }


    @Override
    public NoticeInfo getNoticeInfo() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/notice/info")
                        .build(),
                NoticeInfo.class);
    }

    @Override
    public void updateNoticeConfig(boolean enabled) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/notice/config")
                .paramIfNotNull("enabled", enabled)
                .build());
    }

    @Override
    public NoticeTagResult getTags() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/notice/tags")
                        .build(),
                NoticeTagResult.class);
    }

    @Override
    public NoticeTagVo createTag(String name, String color) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/notice/tag")
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("color", color)
                        .build(),
                NoticeTagVo.class);
    }

    @Override
    public void deleteTag(String name) throws OpenapiException {
        try {
            execute(RequestorBuilder.create()
                    .delete("/notice/tag/byName/" + URLEncoder.encode(name, "UTF-8"))
                    .build());
        } catch (UnsupportedEncodingException e) {
        }
    }

    @Override
    public void deleteTag(Long tagId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/notice/tag/" + tagId)
                .build());
    }

    @Override
    public NoticeVo createNotice(String title, String content, Long tagId, String sourceUrl) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/notice")
                        .paramShouldNotEmpty("title", title)
                        .paramShouldNotEmpty("content", content)
                        .paramShouldNotEmpty("tagId", tagId)
                        .paramIfNotNull("sourceUrl", sourceUrl)
                        .build(),
                NoticeVo.class);
    }

    @Override
    public void updateNotice(long id, String title, String content, Long tagId, String sourceUrl) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/notice/" + id)
                .paramShouldNotEmpty("title", title)
                .paramShouldNotEmpty("content", content)
                .paramShouldNotEmpty("tagId", tagId)
                .paramIfNotNull("sourceUrl", sourceUrl)
                .build());
    }

    @Override
    public void publishNotice(long id, Date publishTime) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/notice/" + id + "/publish")
                .paramIfNotNull("publishTime", publishTime != null ? DateUtil.formatDate(publishTime) : null)
                .build());
    }

    @Override
    public void unpublishNotice(long id) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/notice/" + id + "/unpublish")
                .build());
    }

    @Override
    public NoticeAttachmentVo updateNoticeCoverImg(long id, File imageFile) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/notice/" + id + "/coverImg")
                .enableMultipart()
                .multipart("image", imageFile)
                .build(), NoticeAttachmentVo.class);
    }

    @Override
    public NoticeAttachmentVo addNoticeAttachment(long id, File attachment) throws OpenapiException {
        return addNoticeAttachment(id, attachment.getName(), attachment);
    }

    @Override
    public NoticeAttachmentVo addNoticeAttachment(long id, String fileName, File attachment) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/notice/" + id + "/attachment")
                .enableMultipart()
                .multipart("file", fileName, attachment)
                .build(), NoticeAttachmentVo.class);
    }

    @Override
    public void deleteAttachment(Long attachmentId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/notice/attachment/" + attachmentId)
                .build());
    }

    @Override
    public NoticeDetailVo getNoticeDetail(long id) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/notice/" + id)
                .build(), NoticeDetailVo.class);
    }

    @Override
    public NoticeListResult getNotices(Long tagId, PublishStatus status, String title, String sortField, SortOrder sortOrder, int page, int size, boolean includeAttachments) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/notice")
                .paramIfNotNull("tagId", tagId)
                .paramIfNotNull("status", status)
                .paramIfNotNull("sortField", sortField)
                .paramIfNotNull("sortOrder", sortOrder)
                .paramIfNotNull("page", page)
                .paramIfNotNull("size", size)
                .paramIfNotNull("includeAttachments", includeAttachments)
                .build(), NoticeListResult.class);
    }
}
