package com.cloudbility.openapi.client;

import com.cloudbility.openapi.Credentials;
import com.cloudbility.openapi.CredentialsProvider;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import org.apache.http.client.HttpClient;

import static com.cloudbility.openapi.OpenapiErrorCode.MissingAK;

public class DefaultOpenapiClientFactory implements OpenapiClientFactory {

    private OpenapiClient openapiClient;
    private CredentialsProvider credentialsProvider;
    private String endpoint;
    private HttpClient client;

    public String getEndpoint() {
        return endpoint;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public HttpClient getClient() {
        return client;
    }

    public void setClient(HttpClient client) {
        this.client = client;
    }

    public CredentialsProvider getCredentialsProvider() {
        return credentialsProvider;
    }

    public void setCredentialsProvider(CredentialsProvider credentialsProvider) {
        this.credentialsProvider = credentialsProvider;
    }

    @Override
    public OpenapiClient getOpenapiClient() throws OpenapiException {
        if (openapiClient == null) {
            synchronized (this) {
                if (openapiClient == null) {
                    openapiClient = createOpenapiClient();
                }
            }
        }
        return openapiClient;
    }

    /**
     * 自有短信网关用到的client
     *
     * @return
     * @throws OpenapiException
     */
    @Override
    public OpenapiClient getOpenapiClient(Credentials credentials, String endpoint) throws OpenapiException {
        OpenapiConfig openapiConfig = new OpenapiConfig();
        if (credentials == null) {
            throw new OpenapiException(MissingAK);
        }
        if (endpoint != null) {
            openapiConfig.setEndpoint(endpoint);
        }
        openapiConfig.setAccessKeyId(credentials.getAccessKeyId());
        openapiConfig.setAccessKeySecret(credentials.getAccessKeySecret());
        HttpClient client = this.client;
        OpenapiClient openapiClient = null;
        if (client != null) {
            openapiClient = new OpenapiClient(openapiConfig, client);
        } else {
            openapiClient = new OpenapiClient(openapiConfig);
        }
        return openapiClient;
    }

    private OpenapiClient createOpenapiClient() throws OpenapiException {
        OpenapiConfig openapiConfig = new OpenapiConfig();
        Credentials credentials = credentialsProvider.getCredentials();
        if (credentials == null) {
            throw new OpenapiException(MissingAK);
        }
        String endpoint = this.endpoint;
        if (endpoint != null) {
            openapiConfig.setEndpoint(endpoint);
        }
        openapiConfig.setAccessKeyId(credentials.getAccessKeyId());
        openapiConfig.setAccessKeySecret(credentials.getAccessKeySecret());
        HttpClient client = this.client;
        OpenapiClient openapiClient = null;
        if (client != null) {
            openapiClient = new OpenapiClient(openapiConfig, client);
        } else {
            openapiClient = new OpenapiClient(openapiConfig);
        }
        return openapiClient;
    }
}
