package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.OpsPolicyApi;
import com.cloudbility.openapi.response.OpsPolicyGroupResult;
import com.cloudbility.openapi.response.OpsPolicyItemResult;
import com.cloudbility.openapi.response.OpsPolicyResult;
import com.cloudbility.openapi.vo.*;
import com.cloudbility.openapi.vo.ops.OpsValueType;

import java.util.Map;

public class DefaultOpsPolicyApi extends AbstractApi implements OpsPolicyApi {

    public DefaultOpsPolicyApi(OpenapiClient client) {
        super(client);
    }


    @Override
    public OpsPolicyResult getOpsPoliciesByGroupId(Long groupId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicies/byGroup/" + groupId)
                .build(), OpsPolicyResult.class);
    }

    @Override
    public OpsPolicyResult getOpsPoliciesByCloudId(Long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicy/byCloud/" + cloudId)
                .build(), OpsPolicyResult.class);
    }

    @Override
    public OpsPolicyItemResult getDefaultOpsPolicyItems() throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicy/defaultItems")
                .build(), OpsPolicyItemResult.class);
    }

    @Override
    public OpsPolicyDetailVo getOpsPolicyDetail(Long policyId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicy/" + policyId)
                .build(), OpsPolicyDetailVo.class);
    }

    @Override
    public OpsPolicyItemResult getOpsPolicyItems(Long policyId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicy/" + policyId + "/items")
                .build(), OpsPolicyItemResult.class);
    }

    @Override
    public OpsPolicyItemVo getOpsPolicyItem(Long policyId, String itemIdentity) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicy/" + policyId + "/item/" + itemIdentity)
                .build(), OpsPolicyItemVo.class);
    }

    @Override
    public OpsPolicyGroupDetailVo getOpsPolicyGroupDetail(Long policyGroupId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicyGroup/" + policyGroupId)
                .build(), OpsPolicyGroupDetailVo.class);
    }

    @Override
    public OpsPolicyGroupResult getOpsPolicyGroupsByTeamId(Long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicyGroups").param("teamId", teamId)
                .build(), OpsPolicyGroupResult.class);
    }


    @Override
    public OpsPolicyItemResult updateOpsPolicyItems(Long policyId, Map<String, Object> items) throws OpenapiException {
        RequestorBuilder builder = RequestorBuilder.create().put("/opsPolicy/" + policyId + "/items");
        for (Map.Entry<String, Object> entry : items.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            if (value == null) {
                value = "";
            }
            if (value instanceof OpsValueType) {
                value = ((OpsValueType) value).getValueAsString();
            }
            if (!(value instanceof String)) {
                value = String.valueOf(value);
            }
            builder.param(String.valueOf(key), String.valueOf(value));
        }
        return execute(builder.build(), OpsPolicyItemResult.class);
    }

    @Override
    public Boolean updateOpsPolicyGroupHosts(Long policyGroupId, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/opsPolicyGroup/" + policyGroupId + "/hosts")
                .param("hosts", hostIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean addOpsPolicyMembers(Long policyId, String memberIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/opsPolicy/" + policyId + "/members")
                .param("members", memberIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean addOpsPolicyDepartments(Long policyId, String departmentIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/opsPolicy/" + policyId + "/departments")
                .param("departmentIds", departmentIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean addOpsPolicyRoles(Long policyId, String roleIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/opsPolicy/" + policyId + "/roles")
                .param("roles", roleIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean updateOpsPolicyGroupPriorities(Long policyGroupId, String sortedPolicyIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/opsPolicyGroup/" + policyGroupId + "/priorities")
                .param("sortedPolicyIds", sortedPolicyIds)
                .build(), Boolean.class);
    }


    @Override
    public OpsPolicyGroupVo createOpsPolicyGroup(Long teamId, String policyGroupName, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/opsPolicyGroup")
                .param("teamId", teamId)
                .param("name", policyGroupName)
                .param("hosts", hostIds)
                .build(), OpsPolicyGroupVo.class);
    }

    @Override
    public OpsPolicyVo createOpsPolicy(Long policyGroupId, String policyName, String policyDesc, Integer priority, String roleIds, String departmentIds, String memberIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/opsPolicy")
                .param("policyGroupId", policyGroupId)
                .param("name", policyName)
                .param("description", policyDesc)
                .param("priority", priority)
                .param("roles", roleIds)
                .param("departments", departmentIds)
                .param("members", memberIds)
                .build(), OpsPolicyVo.class);
    }

    @Override
    public Boolean addOpsPolicyGroupHosts(Long policyGroupId, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/opsPolicyGroup/" + policyGroupId + "/hosts")
                .param("hosts", hostIds)
                .build(), Boolean.class);
    }


    @Override
    public Boolean deleteOpsPolicy(Long policyId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .delete("/opsPolicy/" + policyId)
                .build(), Boolean.class);
    }

    @Override
    public Boolean deleteOpsPolicyGroup(Long policyGroupId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .delete("/opsPolicyGroup/" + policyGroupId)
                .build(), Boolean.class);
    }

    @Override
    public Boolean deleteOpsPolicyGroupHosts(Long policyGroupId, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .delete("/opsPolicyGroup/" + policyGroupId + "/hosts")
                .param("hosts", hostIds)
                .build(), Boolean.class);
    }


}
