package com.cloudbility.openapi.client;


import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.OpsPolicyApiV2;
import com.cloudbility.openapi.response.OpsPolicyItemResult;
import com.cloudbility.openapi.response.OpsPolicyResult;
import com.cloudbility.openapi.vo.OpsPolicyDetailVo;
import com.cloudbility.openapi.vo.OpsPolicyItemVo;
import com.cloudbility.openapi.vo.OpsPolicyVo;
import com.cloudbility.openapi.vo.ops.OpsValueType;

import java.util.Map;


public class DefaultOpsPolicyApiV2 extends AbstractApi implements OpsPolicyApiV2 {


    public DefaultOpsPolicyApiV2(OpenapiClient client) {
        super(client);
    }

    @Override
    public OpsPolicyItemResult getDefaultOpsPolicyItems() throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/v2/opsPolicy/defaultItems")
                .build(), OpsPolicyItemResult.class);
    }

    @Override
    public OpsPolicyResult getOpsPoliciesByTeamId(Integer teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/v2/opsPolicyList").param("teamId", teamId)
                .build(), OpsPolicyResult.class);
    }

    @Override
    public OpsPolicyDetailVo getOpsPolicyDetail(Integer policyId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/v2/opsPolicyDetail/"+policyId)
                .build(), OpsPolicyDetailVo.class);
    }

    @Override
    public Boolean deleteOpsPolicy(Integer policyId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .delete("/v2/opsPolicy" + policyId)
                .build(), Boolean.class);
    }

    @Override
    public OpsPolicyResult getOpsPoliciesByCloudId(Integer cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/v2/opsPolicy/byCloud/" + cloudId)
                .build(), OpsPolicyResult.class);
    }

    @Override
    public OpsPolicyItemResult getOpsPolicyItems(Integer policyId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/opsPolicy/" + policyId + "/items")
                .build(), OpsPolicyItemResult.class);
    }

    @Override
    public OpsPolicyItemVo getOpsPolicyItemInfo(Integer policyId, String itemIdentity) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/v2/opsPolicy/" + policyId + "/item/" + itemIdentity)
                .build(), OpsPolicyItemVo.class);
    }

    @Override
    public OpsPolicyItemResult updateOpsPolicyItems(Integer policyId, Map<String, Object> items) throws OpenapiException {
        RequestorBuilder builder = RequestorBuilder.create().put("/v2/opsPolicy/" + policyId + "/items");
        for (Map.Entry<String, Object> entry : items.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            if (value == null) {
                value = "";
            }
            if (value instanceof OpsValueType) {
                value = ((OpsValueType) value).getValueAsString();
            }
            if (!(value instanceof String)) {
                value = String.valueOf(value);
            }
            builder.param(String.valueOf(key), String.valueOf(value));
        }
        return execute(builder.build(), OpsPolicyItemResult.class);
    }

    @Override
    public Boolean updateOpsPolicyHosts(Integer policyId, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/v2/opsPolicy/" + policyId + "/hosts")
                .param("hosts", hostIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean addOpsPolicyHosts(Integer policyId, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/v2/opsPolicy/" + policyId + "/hosts")
                .param("hosts", hostIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean deleteOpsPolicyHosts(Integer policyId, String hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .delete("/v2/opsPolicy/" + policyId + "/hosts")
                .param("hosts", hostIds)
                .build(), Boolean.class);
    }

    @Override
    public OpsPolicyVo createOpsPolicy(Integer teamId, String policyName, String policyDesc, Integer priority, String roleIds, String departmentIds, String memberIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/opsPolicy")
                .param("teamId", teamId)
                .param("name", policyName)
                .param("description", policyDesc)
                .param("priority", priority)
                .param("roles", roleIds)
                .param("departments", departmentIds)
                .param("members", memberIds)
                .build(), OpsPolicyVo.class);
    }

    @Override
    public Boolean addOpsPolicyDepartments(Integer policyId, String departmentIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/v2/opsPolicy/" + policyId + "/departments")
                .param("departmentIds", departmentIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean addOpsPolicyMembers(Integer policyId, String memberIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/v2/opsPolicy/" + policyId + "/members")
                .param("members", memberIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean addOpsPolicyRoles(Integer policyId, String roleIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/v2/opsPolicy/" + policyId + "/roles")
                .param("roles", roleIds)
                .build(), Boolean.class);
    }

    @Override
    public Boolean updateOpsPolicyPriorities(Integer teamId, String sortedPolicyIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/v2/opsPolicy/" + teamId + "/priorities")
                .param("sortedPolicyIds", sortedPolicyIds)
                .build(), Boolean.class);
    }
}
