package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.QuotaApi;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.QuotaUsedItemVo;

public class DefaultQuotaApi extends AbstractApi implements QuotaApi {
    public DefaultQuotaApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public QuotaItemResult getQuotaItems() throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/quotaItems")
                .build(), QuotaItemResult.class);
    }

    @Override
    public QuotaUsedItemResult getTeamQuotas(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/" + teamId + "/quotas")
                .build(), QuotaUsedItemResult.class);
    }

    @Override
    public QuotaUsedItemVo getTeamQuota(long teamId, String code) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/" + teamId + "/quota/" + code)
                .build(), QuotaUsedItemVo.class);
    }

    @Override
    public QuotaUsedItemVo updateTeamQuotaUsed(long teamId, String code, long used) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .put("/team/" + teamId + "/quota/" + code + "/used")
                .paramShouldNotEmpty("used", used)
                .build(), QuotaUsedItemVo.class);
    }

    @Override
    public QuotaUsedItemVo increaseTeamQuotaUsed(long teamId, String code, long increment) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/team/" + teamId + "/quota/" + code + "/increment")
                .paramShouldNotEmpty("increment", increment)
                .build(), QuotaUsedItemVo.class);
    }

    @Override
    public ModuleVo getTeamModule(long teamId, ModuleDefinition moduleCode) throws OpenapiException {
        GetModuleResult result =  execute(RequestorBuilder.create()
                .get("/team/" + teamId + "/module/" + moduleCode)
                .build(), GetModuleResult.class);
        if(result != null && !result.getModules().isEmpty()){
            return result.getModules().get(0);
        }
        return null;
    }

    @Override
    public ModuleTermListResult getModuleTerms(String moduleCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/moduleTerms/" + moduleCode)
                .build(), ModuleTermListResult.class);
    }
}
