package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.RoleApi;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.RoleDetailResult;
import com.cloudbility.openapi.vo.RoleVo;

import java.util.List;

public class DefaultRoleApi extends AbstractApi implements RoleApi {


    public DefaultRoleApi(OpenapiClient client) {
        super(client);
    }


    @Override
    public GetFunctionResult getFunctions() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/function")
                        .build(),
                GetFunctionResult.class);
    }

    @Override
    public PagedRoleListResult getRoles(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role")
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public PagedRoleListResult getRolesByTeamCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/byTeamCode/" + teamCode)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public RoleVo getRolesByCode(long teamId, String roleCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/byCode/" + roleCode)
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                RoleVo.class);
    }

    @Override
    public RoleVo getRoleByRoleAndTeamCode(String teamCode, String roleCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/byRoleAndTeamCode")
                        .paramShouldNotEmpty("teamCode", teamCode)
                        .paramShouldNotEmpty("roleCode", roleCode)
                        .build(),
                RoleVo.class);
    }

    @Override
    public PagedRoleListResult getRoleByTeamCodeAndUserAccount(String teamCode, String userAccount) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/byTeamCodeAndUserAccount")
                        .paramShouldNotEmpty("teamCode", teamCode)
                        .paramShouldNotEmpty("userAccount", userAccount)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public PagedRoleListResult getRoleByTeamAndUserId(long teamId, long userId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/byUser/" + userId)
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public RoleDetailResult getRole(long roleId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/" + roleId)
                        .build(),
                RoleDetailResult.class);
    }

    @Override
    public GetFunctionResult getRoleFunctions(long roleId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/" + roleId + "/functions")
                        .build(),
                GetFunctionResult.class);
    }

    @Override
    public UserBriefResult getRoleMembers(long roleId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/" + roleId + "/members")
                        .build(),
                UserBriefResult.class);
    }

    @Override
    public GetRoleTemplatesResult getRoleTemplates() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/roleTemplates")
                        .build(),
                GetRoleTemplatesResult.class);
    }

    @Override
    public PagedRoleListResult getRolesByPage(long teamId, Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/roles")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramIfNotNull("page", page)
                        .paramIfNotNull("size", size)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public PagedRoleListResult getRolesByCloud(long cloudId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/roles/byCloud/" + cloudId)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public PagedRoleListResult getRolesByHg(Long hgId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/roles/byHg/" + hgId)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public PagedRoleListResult getRolesByVg(Long vgId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/roles/byVg/" + vgId)
                        .build(),
                PagedRoleListResult.class);
    }

    @Override
    public GetRoleResourcesResult getRoleResource(long roleId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/role/" + roleId + "/resources")
                        .build(),
                GetRoleResourcesResult.class);
    }

    @Override
    public Long createRole(long teamId, String name, String description) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role")
                .paramIfNotNull("teamId", teamId)
                .paramIfNotNull("name", name)
                .paramIfNotNull("description", description)
                .build(), CreateRoleResult.class).getRoleId();
    }

    @Override
    public void grantRoleToCloud(long roleId, long cloudId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role/" + roleId + "/cloud/" + cloudId)
                .build(), Void.class);
    }

    @Override
    public void grantRoleToHg(long roleId, Long hgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role/" + roleId + "/hg/" + hgId)
                .build(), Void.class);
    }

    @Override
    public void grantRoleToVg(long roleId, Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role/" + roleId + "/vg/" + vgId)
                .build(), Void.class);
    }

    @Override
    public void grantRoleToMembers(long roleId, List<Long> userIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role/" + roleId + "/members/" + StringUtil.join(userIds, ","))
                .build(), Void.class);
    }

    @Override
    public void grantRoleToMemberByAccount(long roleId, String userAccount) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role/" + roleId + "/members/byAccount/" + userAccount)
                .build(), Void.class);
    }

    @Override
    public void addFunctionsToRole(long roleId, List<String> functionCodes) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/role/" + roleId + "/functions/" + StringUtil.join(functionCodes, ","))
                .build(), OpenapiResponse.class);
    }

    @Override
    public void deleteRoleFunctions(long roleId, List<String> functionCodes) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/role/" + roleId + "/functions/" + StringUtil.join(functionCodes, ","))
                .build(), OpenapiResponse.class);
    }

    @Override
    public void deleteRole(long roleId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/role/" + roleId)
                .build(), OpenapiResponse.class);

    }

    @Override
    public void revokeRoleFromCloud(long roleId, long cloudId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/role/" + roleId + "/cloud/" + cloudId)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void revokeRoleFromHg(long roleId, Long hgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/role/" + roleId + "/hg/" + hgId)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void revokeRoleFromVg(long roleId, Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/role/" + roleId + "/vg/" + vgId)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void revokeRoleFromMembers(long roleId, List<Long> userIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/role/" + roleId + "/members/" + StringUtil.join(userIds, ","))
                .build(), OpenapiResponse.class);
    }

    @Override
    public void revokeRoleFromMembersByAccount(long roleId, String account) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/role/" + roleId + "/members/byAccount/" + account)
                .build(), OpenapiResponse.class);
    }

    @Override
    public void updateRoleName(long roleId, String name, String description) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/role/" + roleId)
                .paramShouldNotEmpty("name", name)
                .paramIfNotNull("description", description)
                .build(), Void.class);
    }

    @Override
    public RoleListResult findRoleByName(long teamId, String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/roles/byName")
                .paramShouldNotEmpty("name", name)
                .build(), RoleListResult.class);
    }

    @Override
    public RoleListResult findRoleByNameLike(long teamId, String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/roles/byNameLike")
                .paramShouldNotEmpty("name", name)
                .build(), RoleListResult.class);
    }
}
