package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.SessionApi;
import com.cloudbility.openapi.response.BooleanResult;
import com.cloudbility.openapi.response.OAuthUrlResult;
import com.cloudbility.openapi.response.SessionOpenResult;
import com.cloudbility.openapi.vo.AccessType;
import com.cloudbility.openapi.vo.CountResult;
import com.cloudbility.openapi.vo.DesktopType;

public class DefaultSessionApi extends AbstractApi implements SessionApi {

    public DefaultSessionApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public OAuthUrlResult getOAuthSessionUrl(long userId, boolean oneOff, long hostId, DesktopType desktopType,
                                             Integer port, AccessType accessType, int expireSeconds, String account,
                                             String password, String privateKey, String privateKeyPass) throws OpenapiException {
        return getOAuthSessionUrl(userId, oneOff, hostId, desktopType, port, accessType, expireSeconds, account, password, privateKey, privateKeyPass, null);
    }

    @Override
    public OAuthUrlResult getOAuthSessionUrl(long userId, boolean oneOff, long hostId, DesktopType desktopType, Integer port, AccessType accessType, int expireSeconds, String account, String password, String privateKey, String privateKeyPass, String screenResolution) throws OpenapiException {
        checkOnlyOneNotNull("only set one of password and privateKey", password, privateKey);
        return execute(RequestorBuilder.create()
                        .get("/oauthSession")
                        .paramShouldNotEmpty("userId", userId)
                        .paramShouldNotEmpty("oneOff", oneOff)
                        .paramShouldNotEmpty("hostId", hostId)
                        .paramShouldNotEmpty("desktopType", desktopType)
                        .paramIfNotNull("port", port)
                        .paramShouldNotEmpty("accessType", accessType)
                        .paramShouldNotEmpty("expireSeconds", expireSeconds)
                        .paramIfNotNull("account", account)
                        .paramIfNotNull("password", password)
                        .paramIfNotNull("privateKey", privateKey)
                        .paramIfNotNull("privateKeyPass", privateKeyPass)
                        .paramIfNotNull("screenResolution", screenResolution)
                ,
                OAuthUrlResult.class);
    }

    @Override
    public SessionOpenResult openSessionGetWSUrl(long userId, long hostId, DesktopType desktopType, Integer port, AccessType accessType, String account, String password, String privateKey, String privateKeyPass) throws OpenapiException {
        checkOnlyOneNotNull("only set one of password and privateKey", password, privateKey);
        return execute(RequestorBuilder.create()
                        .get("/openSessionGetWS")
                        .paramShouldNotEmpty("userId", userId)
                        .paramShouldNotEmpty("hostId", hostId)
                        .paramShouldNotEmpty("desktopType", desktopType)
                        .paramIfNotNull("port", port)
                        .paramShouldNotEmpty("accessType", accessType)
                        .paramIfNotNull("account", account)
                        .paramIfNotNull("password", password)
                        .paramIfNotNull("privateKey", privateKey)
                        .paramIfNotNull("privateKeyPass", privateKeyPass)
                ,
                SessionOpenResult.class);
    }

    @Override
    public boolean closeSession(Long sessionId, String reason) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .delete("/closeSession")
                        .paramShouldNotEmpty("sessionId", sessionId)
                        .paramIfNotNull("reason", reason)
                ,
                BooleanResult.class).isSuccess();
    }

    @Override
    public int countSessionReady() throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                .get("/sessions/countReady")
                .build(), CountResult.class);
        return result.getCount();
    }

    @Override
    public int countSessionReadyByTeam(long teamId) throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                .get("/sessions/countReadyByTeam")
                .paramShouldNotEmpty("teamId", teamId)
                .build(), CountResult.class);
        return result.getCount();
    }
}