package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.SystemApi;
import com.cloudbility.openapi.request.AddLdapConfigRequest;
import com.cloudbility.openapi.response.LdapConfigVoList;
import com.cloudbility.openapi.response.LdapSyncConfigVoList;
import com.cloudbility.openapi.vo.CountResult;
import com.cloudbility.openapi.vo.LdapSyncConfigVo;

public class DefaultSystemApi extends AbstractApi implements SystemApi {
    public DefaultSystemApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public LdapConfigVoList getLdapConfigs() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/ldap/configs")
                        .build(),
                LdapConfigVoList.class);
    }

    @Override
    public LdapSyncConfigVoList getLdapSyncConfigs() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/ldap/syncConfigs")
                        .build(),
                LdapSyncConfigVoList.class);
    }

    @Override
    public void addLdapConfig(AddLdapConfigRequest request) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/ldap/config")
                        .body(request)
                        .build(),
                Void.class);
    }

    @Override
    public void updateLdapConfig(AddLdapConfigRequest request) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/ldap/config")
                        .body(request)
                        .build(),
                Void.class);
    }

    @Override
    public int updateLdapConfigBaseDn(String prevBaseDn, String newBaseDn) throws OpenapiException {
        CountResult cr = execute(RequestorBuilder.create()
                        .put("/ldap/config/baseDn")
                        .paramShouldNotEmpty("prevBaseDn", prevBaseDn)
                        .paramShouldNotEmpty("newBaseDn", newBaseDn)
                        .build(),
                CountResult.class);
        return cr.getCount();
    }

    @Override
    public int deleteLdapConfig(String baseDn) throws OpenapiException {
        CountResult cr = execute(RequestorBuilder.create()
                        .delete("/ldap/config")
                        .paramShouldNotEmpty("baseDn", baseDn)
                        .build(),
                CountResult.class);
        return cr.getCount();
    }

    @Override
    public void updateLdapSyncConfig(LdapSyncConfigVo request) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/ldap/syncConfig")
                        .body(request)
                        .build(),
                Void.class);
    }

    @Override
    public void bindLdapUser(String baseDn, long userId, String ldapAccountKey, String ldapAccount) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/ldap/bind")
                        .paramShouldNotEmpty("baseDn", baseDn)
                        .paramShouldNotEmpty("userId", userId)
                        .paramIfNotNull("ldapAccountKey", ldapAccountKey)
                        .paramIfNotNull("ldapAccount", ldapAccount)
                        .build(),
                Void.class);
    }

    @Override
    public void unbindLdapUser(long userId, String newAccount, String newPassword, boolean generatePassword, boolean forceUpdatePwd, boolean emailNewPassword) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/ldap/unbind")
                        .paramShouldNotEmpty("userId", userId)
                        .paramIfNotNull("newAccount", newAccount)
                        .paramIfNotNull("newPassword", newPassword)
                        .paramIfNotNull("generatePassword", generatePassword)
                        .paramIfNotNull("forceUpdatePwd", forceUpdatePwd)
                        .paramIfNotNull("emailNewPassword", emailNewPassword)
                        .build(),
                Void.class);
    }
}
