package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.TagApi;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.response.*;

import java.util.List;

public class DefaultTagApi extends AbstractApi implements TagApi {
    public DefaultTagApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public ListTagResult getTags(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/tag/list")
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                ListTagResult.class);
    }

    @Override
    public ListTagResult getByHost(long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/tag/listByHost")
                        .paramShouldNotEmpty("hostId", hostId)
                        .build(),
                ListTagResult.class);
    }

    @Override
    public TagDetailResult getTag(int tagId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/tag/" + tagId)
                        .build(),
                TagDetailResult.class);
    }

    @Override
    public HostDetailListResult getHostByTag(int tagId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/tag/" + tagId + "/hostDetailList")
                        .build(),
                HostDetailListResult.class);
    }

    @Override
    public GetHostBriefResult getHostIdsByTag(int tagId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/tag/" + tagId + "/hostIdList")
                        .build(),
                GetHostBriefResult.class);
    }

    @Override
    public TagDetailResult createTag(long teamId, String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/tag")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .build(),
                TagDetailResult.class);
    }

    @Override
    public TagDetailResult attachToHostByName(long teamId, String name, List<Long> hostIds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/tag/attachToHostByName/" + StringUtil.join(hostIds, ","))
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .build(),
                TagDetailResult.class);
    }

    @Override
    public void updateTagName(int tagId, String newName) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/tag/" + tagId + "/tagName")
                        .paramShouldNotEmpty("newName", newName)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateTagNameAndValue(int tagValueId, String newName, String newValue) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/tag/" + tagValueId)
                        .paramShouldNotEmpty("newName", newName)
                        .paramIfNotNull("newValue", newValue)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateTagValue(int tagValueId, String newValue) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/tag/" + tagValueId + "/tagValue")
                        .paramShouldNotEmpty("newValue", newValue)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void detachFromHostByName(long teamId, String name, List<Long> hostIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/tag/detachFromHostByName/" + StringUtil.join(hostIds, ","))
                        .paramShouldNotEmpty("name", name)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void deleteTag(int tagId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/tag/" + tagId)
                        .build(),
                OpenapiResponse.class);
    }
}
