package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.VgApi;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.response.OpenapiResponse;
import com.cloudbility.openapi.vo.VirtualGroupDetailVo;
import com.cloudbility.openapi.vo.VirtualGroupResult;
import com.cloudbility.openapi.vo.VirtualGroupVo;

import java.util.List;

public class DefaultVgApi extends AbstractApi implements VgApi {

    public DefaultVgApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public VirtualGroupDetailVo getVirtualGroupDetailVo(long id) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/vg/" + id)
                        .build(),
                VirtualGroupDetailVo.class);
    }

    @Override
    public VirtualGroupResult getByTeam(long teamId) throws OpenapiException {
        return getByTeamId(teamId, null);
    }

    @Override
    public VirtualGroupResult getByTeamCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/vg/byTeamCode/" + teamCode)
                        .build(),
                VirtualGroupResult.class);
    }

    @Override
    public VirtualGroupResult getByName(Long teamId, Long cloudId, String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/vg/byName")
                        .paramIfNotNull("teamId", teamId)
                        .paramIfNotNull("cloudId", cloudId)
                        .build(),
                VirtualGroupResult.class);
    }

    @Override
    public VirtualGroupResult getByName(Long teamId, String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/vg/byName")
                        .paramIfNotNull("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .build(),
                VirtualGroupResult.class);
    }

    @Override
    public VirtualGroupResult getByTeamId(Long teamId, Long parentVgId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/vg/byTeam/" + teamId)
                        .paramIfNotNull("parentVgId", parentVgId)
                        .build(),
                VirtualGroupResult.class);
    }

    @Override
    public VirtualGroupResult getByTeamCode(String teamCode, Long parentVgId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/vg/byTeamCode/" + teamCode)
                        .paramIfNotNull("parentVgId", parentVgId)
                        .build(),
                VirtualGroupResult.class);
    }

    @Override
    public VirtualGroupVo createVg(long cloudId, String name, String roles) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/vg")
                        .paramShouldNotEmpty("cloudId", cloudId)
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("roles", roles)
                        .build(),
                VirtualGroupVo.class);
    }

    @Override
    public VirtualGroupVo createVgNew(long teamId, Long parentVgId, String name, String roles, String members) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/vg")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramIfNotNull("parentVgId", parentVgId)
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("roles", roles)
                        .paramIfNotNull("members", members)
                        .build(),
                VirtualGroupVo.class);
    }

    @Override
    public void addBucketToVg(Long vgId, List<Long> bucketIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/vg/" + vgId + "/buckets/" + StringUtil.join(bucketIds, ","))
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void addCdnsToVg(Long vgId, List<Long> cdnIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/vg/" + vgId + "/cdns/" + StringUtil.join(cdnIds, ","))
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void addHostsToVg(Long vgId, List<Long> hostIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/vg/" + vgId + "/hosts/" + StringUtil.join(hostIds, ","))
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateVg(Long vgId, String name, String roles) throws OpenapiException {
        checkNotAllNull("name and roles all null", name, roles);
        execute(RequestorBuilder.create()
                        .put("/vg/" + vgId)
                        .paramIfNotNull("name", name)
                        .paramIfNotNull("roles", roles)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateVgName(Long vgId, String name) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/vg/" + vgId + "/name")
                        .paramShouldNotEmpty("name", name)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateVgRoles(Long vgId, String roles) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/vg/" + vgId + "/roles")
                        .paramShouldNotEmpty("roles", roles)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateVgMembers(Long vgId, String members) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/vg/" + vgId + "/members")
                        .paramShouldNotEmpty("members", members)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void updateVgDepartments(Long vgId, String departments) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/vg/" + vgId + "/departments")
                        .paramIfNotNull("departments", departments)
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void revokeAllVgRoles(Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/vg/" + vgId + "/revokeAllVgRoles")
                        .build(),
                OpenapiResponse.class);
    }

    @Override
    public void deleteVg(List<Long> vgIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + StringUtil.join(vgIds, ",")),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteAllResourcesFromVg(Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/allResources"),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteAllBucketsFromVg(Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/buckets"),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteBucketsFromVg(Long vgId, List<Long> bucketIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/buckets/" + StringUtil.join(bucketIds, ",")),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteAllCdnsFromVg(Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/cdns"),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteCdnsFromVg(Long vgId, List<Long> cdnIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/cdns/" + StringUtil.join(cdnIds, ",")),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteAllHostsFromVg(Long vgId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/hosts"),
                OpenapiResponse.class
        );
    }

    @Override
    public void deleteHostsFromVg(Long vgId, List<Long> hostIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/vg/" + vgId + "/hosts/" + StringUtil.join(hostIds, ",")),
                OpenapiResponse.class
        );
    }
}
