package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.WorkflowApi;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.response.WorkTaskVoResult;
import com.cloudbility.openapi.response.WorkflowInstanceVoResult;
import com.cloudbility.openapi.vo.WorkItemStageVo;
import com.cloudbility.openapi.vo.WorkTypeVo;

import java.util.Date;

public class DefaultWorkflowApi extends AbstractApi implements WorkflowApi {

    public DefaultWorkflowApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public WorkflowInstanceVoResult getMyWorkflowInstances(long teamId, long userId, WorkItemStageVo stage, WorkTypeVo workType, int page, int size, String sortField, String sortOrder) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/workflow/myInstances")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("userId", userId)
                        .paramShouldNotEmpty("stage", stage)
                        .paramIfNotNull("workType", workType)
                        .paramShouldNotEmpty("page", page)
                        .paramShouldNotEmpty("size", size)
                        .paramIfNotNull("sortField", sortField)
                        .paramIfNotNull("sortOrder", sortOrder)
                        .build(),
                WorkflowInstanceVoResult.class);
    }

    @Override
    public WorkTaskVoResult getWorkflowTasksByCursor(long teamId, int cursor, WorkTypeVo workType, WorkItemStageVo stage, int size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/workTasks/byCursor")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("cursorId", cursor)
                        .paramIfNotNull("stage", stage)
                        .paramIfNotNull("workType", workType)
                        .paramShouldNotEmpty("size", size)
                        .build(),
                WorkTaskVoResult.class);
    }

    @Override
    public WorkTaskVoResult getWorkflowTasksByCreateTime(long teamId, WorkTypeVo workType, WorkItemStageVo stage, Date fromTime, Date toTime) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/workTasks/byCreateTime")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramIfNotNull("stage", stage)
                        .paramIfNotNull("workType", workType)
                        .paramShouldNotEmpty("fromTime", DateUtil.formatDate(fromTime))
                        .paramShouldNotEmpty("toTime", DateUtil.formatDate(toTime))
                        .build(),
                WorkTaskVoResult.class);
    }


    public WorkTaskVoResult getWorkflowTasksByUpdateTime(long teamId, WorkTypeVo workType, WorkItemStageVo stage, Date fromTime, Date toTime) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/workTasks/byUpdateTime")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramIfNotNull("stage", stage)
                        .paramIfNotNull("workType", workType)
                        .paramShouldNotEmpty("fromTime", DateUtil.formatDate(fromTime))
                        .paramShouldNotEmpty("toTime", DateUtil.formatDate(toTime))
                        .build(),
                WorkTaskVoResult.class);
    }


}
