package com.cloudbility.openapi.client;


import com.cloudbility.openapi.TokenProvider;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class OpenapiConfig {
    private String version = OpenapiVersions.v1;
    public static final String defaultUserAgent = "cloudbility-openapi-sdk-v2";
    private String endpoint = "https://openapi.cloudbility.com";//默认的endpoint
    /**
     * 设置accessKeyId或accessKeySecret则用签名方式请求；
     * 设置tokenProvider则用token方式请求
     */
    private String accessKeyId;
    private String accessKeySecret;
    private TokenProvider tokenProvider;

    private int connectTimeout = 20 * 1000;//连接超时，毫秒
    private int readTimeout = 60 * 1000;//读取超时，毫秒
    private int maxConnections = 2;
    private String userAgent = defaultUserAgent;
    private Map<String, String> sharedHeaders = new HashMap<>();

    /**
     * 这些路径，不会签名，也不会设置Token
     */
    private List<String> noAuthPaths = new ArrayList<>();

    public OpenapiConfig() {
    }

    public OpenapiConfig(String accessKeyId, String accessKeySecret) {
        this.accessKeyId = accessKeyId;
        this.accessKeySecret = accessKeySecret;
    }

    public String getVersion() {
        return version;
    }

    public String getEndpoint() {
        return endpoint;
    }

    public String getAccessKeyId() {
        return accessKeyId;
    }

    public String getAccessKeySecret() {
        return accessKeySecret;
    }

    public void setAccessKeyId(String accessKeyId) {
        this.accessKeyId = accessKeyId;
    }

    public void setAccessKeySecret(String accessKeySecret) {
        this.accessKeySecret = accessKeySecret;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public int getConnectTimeout() {
        return connectTimeout;
    }

    public void setConnectTimeout(int connectTimeout) {
        this.connectTimeout = connectTimeout;
    }

    public int getReadTimeout() {
        return readTimeout;
    }

    public void setReadTimeout(int readTimeout) {
        this.readTimeout = readTimeout;
    }

    public Map<String, String> getSharedHeaders() {
        return sharedHeaders;
    }

    public void setSharedHeaders(Map<String, String> sharedHeaders) {
        this.sharedHeaders = sharedHeaders;
    }

    public int getMaxConnections() {
        return maxConnections;
    }

    public void setMaxConnections(int maxConnections) {
        this.maxConnections = maxConnections;
    }

    public String getUserAgent() {
        return userAgent;
    }

    public void setUserAgent(String userAgent) {
        this.userAgent = userAgent;
    }

    public TokenProvider getTokenProvider() {
        return tokenProvider;
    }

    public void setTokenProvider(TokenProvider tokenProvider) {
        this.tokenProvider = tokenProvider;
    }

    public OpenapiConfig copy() {
        OpenapiConfig config = new OpenapiConfig();
        config.version = version;
        config.endpoint = endpoint;
        config.accessKeyId = accessKeyId;
        config.connectTimeout = connectTimeout;
        config.maxConnections = maxConnections;
        config.userAgent = userAgent;
        config.readTimeout = readTimeout;
        config.sharedHeaders = new HashMap<>(sharedHeaders);
        config.accessKeySecret = accessKeySecret;
        return config;
    }

    public List<String> getNoAuthPaths() {
        return noAuthPaths;
    }

    public void setNoAuthPaths(List<String> noAuthPaths) {
        this.noAuthPaths = noAuthPaths;
    }

}
