package com.cloudbility.openapi.common;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.OpenapiErrorCode;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;


public class DateUtil {

    public static final String DEFAULT_DATE_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";

    public static final String SIMPLE_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    public static String formatDate(Date date) {
        if (date != null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat(DEFAULT_DATE_FORMAT, Locale.US);
            dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
            return dateFormat.format(date);
        }
        return null;
    }



    public static String formatDateDefaultTimezone(Date date) {
        if (date != null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
            return dateFormat.format(date);
        }
        return null;
    }

    public static String formatSimpleDate(Date date) {
        if (date != null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
            return dateFormat.format(date);
        }
        return null;
    }

    public static Date toDate(String string) throws OpenapiException {
        if (StringUtil.notEmpty(string)) {
            String pattern = "yyyy-MM-dd'T'HH:mm:ss'Z'";//20
            if (string.length() == 24) {
                pattern = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'";
            }
            if (string.length() == 23) {
                pattern = "yyyy-MM-dd'T'HH:mm:ss.SS'Z'";
            }
            if (string.length() == 22) {
                pattern = "yyyy-MM-dd'T'HH:mm:ss.S'Z'";
            }
            if (string.length() == 19 || string.length() == 13) {//纳秒和毫秒
                try {
                    Long timestamp = Long.parseLong(string);
                    if (string.length() == 19) {
                        timestamp = TimeUnit.NANOSECONDS.toMillis(timestamp);
                    }
                    return new Date(timestamp);
                } catch (NumberFormatException e) {
                }
            }
            try {
                SimpleDateFormat dateFormat = new SimpleDateFormat(pattern, Locale.US);
                dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
                return dateFormat.parse(string);
            } catch (ParseException e) {
                throw new OpenapiException(OpenapiErrorCode.InvalidParameter("invalid date string " + string));
            }
        }
        return null;
    }
}
