/*$Id: $
 --------------------------------------
  Skybility
 ---------------------------------------
  Copyright By Skybility ,All right Reserved
 * author   date   comment
 * tongjun  2012-7-24  Created
*/
package com.cloudbility.openapi.common;

import com.google.gson.*;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonWriter;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.Array;
import java.lang.reflect.Type;
import java.util.*;

public final class JsonUtil {

    public static final Type MAP_TYPE = new TypeToken<Map<String, Object>>() {
    }.getType();

    public static final Type STRING_MAP_TYPE = new TypeToken<Map<String, String>>() {
    }.getType();

    public static final Type LIST_TYPE = new TypeToken<List<Object>>() {
    }.getType();

    private JsonUtil() {

    }

    public static final Gson GSON = new GsonBuilder()
            .registerTypeAdapter(Date.class, new GsonUTCDateAdapter())
            .registerTypeAdapter(Double.class, new JsonSerializer<Double>() {
                @Override
                public JsonElement serialize(Double src, Type typeOfSrc, JsonSerializationContext context) {
                    if (src.isNaN()) {
                        return JsonNull.INSTANCE;
                    } else if (src == src.longValue())
                        return new JsonPrimitive(src.longValue());
                    return new JsonPrimitive(src);
                }
            }).create();

    //Json字符串转HashMap
    public static Map<String, String> jsonDecode(String json) {
        Map<String, String> returnMap = new HashMap<String, String>();
        try {
            returnMap = GSON.fromJson(json, STRING_MAP_TYPE);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return returnMap;
    }

    //Json字符串转HashMap
    public static Map<String, Object> fromJsonAsMap(String json) {
        if (StringUtil.isEmpty(json)) {
            return new HashMap<String, Object>(0);
        }
        return GSON.fromJson(json, MAP_TYPE);
    }

    public static Map<String, Object> fromJsonAsMap(Reader json) {
        if (json == null) {
            return new HashMap<String, Object>(0);
        }
        return GSON.fromJson(json, MAP_TYPE);
    }


    //HashMap转Json字符串
    public static String jsonEncode(Object mapObj) {
        return GSON.toJson(mapObj);
    }

    public static Map<String, Object> convertAsMap(Object object) {
        if (object == null) {
            return Collections.emptyMap();
        }
        JsonElement jsonElement = GSON.toJsonTree(object, object.getClass());
        return GSON.fromJson(jsonElement, MAP_TYPE);
    }

    public static <T> T fromJsonMap(Map<String, Object> jsonMap, Class<T> clazz) {
        if (jsonMap == null) {
            return null;
        }
        JsonElement jsonElement = GSON.toJsonTree(jsonMap, MAP_TYPE);
        return GSON.fromJson(jsonElement, clazz);
    }

    //Json字符串转List
    public static List fromJsonAsList(String json) {
        if (StringUtil.isEmpty(json)) {
            return new ArrayList();
        }
        return GSON.fromJson(json, LIST_TYPE);
    }

    public static <T> List<T> fromJsonAsList(String json, Class<T> clazz) {
        if (StringUtil.isEmpty(json)) {
            return new ArrayList();
        }
        return fromJsonList(fromJsonAsList(json), clazz);
    }

    public static <T> List<T> fromJsonList(List list, Class<T> clazz) {
        if (list == null || list.size() == 0) {
            return Collections.emptyList();
        }
        JsonElement jsonElement = GSON.toJsonTree(list, new TypeToken<List<Map<String, Object>>>() {
        }.getType());
        T[] array = GSON.fromJson(jsonElement, (Type) Array.newInstance(clazz, 0).getClass());
        return Arrays.asList(array);
    }

    /**
     * 由于json输出成js字符串然后使用jquey.parseJson解析时，对转义过的'\'需要再次转义成双'\'
     * 参照考 http://api.jquery.com/jquery.parsejson/
     *
     * @param json
     * @return
     */
    public static String escapseForJsliteral(String json) {
        if (!StringUtil.isEmpty(json)) {
            return json.replaceAll("\\\\", "\\\\\\\\");
        }
        return json;
    }

    public static <T> T fromJson(String json, Class<T> clazz) {
        if (StringUtil.isEmpty(json)) {
            return null;
        }
        return GSON.fromJson(json, clazz);
    }

    public static <T> T fromJson(Reader json, Class<T> clazz) {
        if (json == null) {
            return null;
        }
        return GSON.fromJson(json, clazz);
    }

    public static <T> T fromJson(String json, Type typeOfT) {
        if (StringUtil.isEmpty(json)) {
            return null;
        }
        return GSON.fromJson(json, typeOfT);
    }

    public static <T> T[] parseArray(String jsonStr, Class<T> componentType) {
        return GSON.fromJson(jsonStr, (Type) Array.newInstance(componentType, 0).getClass());
    }

    public static <T> T parseObject(String jsonStr, Class<T> componentType) {
        return GSON.fromJson(jsonStr, componentType);
    }


    public static String toJson(Object src) {
        return GSON.toJson(src);
    }

    public static void toJsonPretty(Object src, Writer writer) throws IOException {
        JsonWriter jsonWriter = new JsonWriter(writer);
        jsonWriter.setIndent(" ");
        GSON.toJson(src, src.getClass(), jsonWriter);
        writer.flush();
    }

    public static void toJson(Object src, Writer writer) throws IOException {
//        GSON.toJson(src, writer);//OK
        GSON.toJson(src, src.getClass(), new JsonWriter(writer));
        writer.flush();
    }

    public static String toJson(Object src, Type typeOfSrc) {
        return GSON.toJson(src, typeOfSrc);
    }
}
