package com.cloudbility.openapi.common;


import javax.activation.MimetypesFileTypeMap;
import java.io.*;
import java.util.HashMap;
import java.util.Map;

public class MimetypesUtil {
    private static Map<String, String> typeMap = new HashMap<>();

    private static MimetypesFileTypeMap mimetypesFileTypeMap = new MimetypesFileTypeMap();

    private static String defaultType = "application/octet-stream";

    static {
        load();
    }

    private static void load() {
        InputStream inputStream = MimetypesUtil.class.getResourceAsStream("/mime.types");
        String content = readStringFromFile(inputStream);
        String[] lines = content.split("\n");
        for (String line : lines) {
            String[] parts = line.split("\\s+");
            if (parts.length == 2) {
                if (parts[0].trim().startsWith("#")) {
                    continue;
                }
                typeMap.put(parts[0], parts[1]);
                typeMap.put(parts[0].toLowerCase(), parts[1]);
            }
        }
    }


    public static String readStringFromFile(InputStream inputStream) {
        BufferedReader input = null;
        try {
            input = new BufferedReader(new InputStreamReader(inputStream));
            StringWriter writer = new StringWriter();
            IOUtil.copyWriter(input, writer);
            String contents = writer.toString();
            return contents;
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            IOUtil.closeQuietly(input);
        }
    }


    public static String getContentType(String path) {
        int dot_pos = path.lastIndexOf(".");
        if (dot_pos < 0) {
            return defaultType;
        } else {
            String file_ext = path.substring(dot_pos + 1);
            String tye = typeMap.get(file_ext);
            if (tye != null) {
                return tye;
            }
            tye = typeMap.get(file_ext.toLowerCase());
            if (tye != null) {
                return tye;
            }
        }
        return mimetypesFileTypeMap.getContentType(path);
    }

}
