package com.cloudbility.openapi.demo;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.ScriptListener;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.impl.ScriptApiWrapper;
import com.cloudbility.openapi.vo.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

public class AsyncScriptDemo extends AbstractDemo {

    private static Logger logger = LoggerFactory.getLogger(AsyncScriptDemo.class);

    public static void main(String[] args) throws OpenapiException {
        OpenapiClient client = createOpenapiClient();
//        prepareScript(client);
        ScriptApiWrapper scriptApiWrapper = new ScriptApiWrapper(client, Executors.newSingleThreadScheduledExecutor());
        //scriptApiWrapper对象可以复用，可以并发使用。当并发量大的时候，可以增大polling线程池
        ScriptExecutingHostsInfo info = new ScriptExecutingHostsInfo();
        ScriptExecutingHost executingHost = new ScriptExecutingHost();
        executingHost.setHostId(1L);
        executingHost.setCredentialId(1L);
        info.getExecutingHosts().add(executingHost);
        info.setOperatorId(1L);
        ScriptListener listener = new ScriptListener() {
            @Override
            public void onDone(JobVo jobVo, ScriptRunResult result) {
                System.out.println("executing result:" + JsonUtil.toJson(result));
                try {
                    InputStreamResult sr = client.scriptApi().getScriptOutputOnHost(jobVo.getId(), 1);
                    String output = StringUtil.readStringFromFile(sr.getInputStream());
                    logger.info("job {} on {} output:\n{}", jobVo.getId(), 1, output);
                } catch (OpenapiException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onError(JobVo jobVo, Throwable throwable) {
                System.out.println("executing fail:" + throwable.getMessage());
                throwable.printStackTrace();
            }
        };
        //test 1
        info.setParameter("p1 p2 p3");
        scriptApiWrapper.executeScript(1116, info, listener);

        //test 2
        info.setParameter("x1 x2");
        scriptApiWrapper.executeScript(1116, info, listener);

        // test 3
        info.setParameter("L1 L2 L3");
        Future<ScriptRunResult> future = scriptApiWrapper.executeScriptAsync(1116, info);
        try {
            ScriptRunResult result = future.get();
            InputStreamResult sr = client.scriptApi().getScriptOutputOnHost(result.getId(), 1);
            String output = StringUtil.readStringFromFile(sr.getInputStream());
            logger.info("job {} on {} output:\n{}", result.getId(), 1, output);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
    }

    public static void prepareScript(OpenapiClient client) throws OpenapiException {
        //scriptApiWrapper 可以复用，可以多线程使用，如果并发量大，可以把polling线程池扩大
        ScriptLibraryListResult result = client.scriptApi().getScriptLibraryList(1);
        System.out.println(JsonUtil.toJson(result));
        //{"scriptLibraries":[{"id":108,"name":"默认脚本库","description":"系统为团队预置的默认脚本库","creator":1,"createTime":"2019-08-07T11:10:24Z"}],"requestId":"9waP3un2QzGJ_G53jvSgOA"}
        ScriptVo scriptVo = client.scriptApi().createScript(108, "test_async_sh", "echo Hello\n sleep 10\n echo World\necho $*", null, null, ScriptLanguage.shell, 30, 1L);
        System.out.println(JsonUtil.toJson(scriptVo));
        //{"id":1116,"name":"test_async_sh","libraryId":108,"creator":1,"createTime":"2019-08-12T10:48:54Z","modifyTime":"2019-08-12T10:48:54Z","timeout":300,"language":"shell","requestId":"oFVHrRycSDumm5ttM01Qbg"}
    }
}
