package com.cloudbility.openapi.demo;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.response.WorkTaskVoResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;

import static java.util.stream.Collectors.toList;

public class BatchPullWorkTaskDemo extends AbstractDemo {

    static final Logger logger = LoggerFactory.getLogger("Demo");

    public static void main(String[] args) {
        OpenapiClient client = createSAASClient();
        int delaySeconds = 5;
        int interval = 10;
        //扫描时间窗口的结束时间,即当前时间往前推delaySeconds秒
        Long end = System.currentTimeMillis() - delaySeconds * 1000;//当前时间往前推delaySeconds秒
        //扫描时间窗口的开始时间
        Long start = end - interval * 1000;
        while (true) {
            try {
                Date fromTime = new Date(start);
                Date toTime = new Date(end);
                WorkTaskVoResult result = client.workflowApi().getWorkflowTasksByUpdateTime(2, null, null, fromTime, toTime);
                logger.info("Got workTasks {} between {} - {} :{}", result.getTasks().size(),
                        DateUtil.formatSimpleDate(fromTime),
                        DateUtil.formatSimpleDate(toTime),
                        result.getTasks().stream().map(t -> t.getId()).collect(toList()));
            } catch (OpenapiException e) {
                logger.error("error call getWorkflowTasksByUpdateTime", e);
            }
            start = end;
            end = start + interval * 1000;
            long sleepUntil = end + delaySeconds * 1000;
            try {
                long sleepMs = sleepUntil - System.currentTimeMillis();
                Thread.sleep(sleepMs);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
