package com.cloudbility.openapi.demo;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.vo.monitor.FuncName;
import com.cloudbility.openapi.vo.monitor.FuncUtil;
import com.cloudbility.openapi.vo.monitor.MonitorMetric;
import com.cloudbility.openapi.vo.monitor.SingletonResultList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.Date;

public class FetchMonitorDemo extends AbstractDemo {

    private static Logger logger = LoggerFactory.getLogger(AsyncScriptDemo.class);

    public static void main(String[] args) throws OpenapiException {
        OpenapiClient client = createOpenapiClient("duXVUW0psGc8Vy8x", "1qIR86z7jyciSsGe2sZpQcml9692g4", "http://192.168.5.51:8080/api/openapi");
        SingletonResultList list = client.monitorApi().getHostAggregation(1303,
                MonitorMetric.CpuUsage,//获取CPU的统计数据
                //获取特定时间范围内：最后一个点（包括时间），最早一个点（包括时间），最大值，最小值，95分位数
                Arrays.asList(FuncUtil.create(FuncName.last), FuncUtil.create(FuncName.first), FuncUtil.create(FuncName.max), FuncUtil.create(FuncName.min), FuncUtil.create(FuncName.percentile, 95)),
                //特定时间：5小时以前到3小时以前
                new Date((System.currentTimeMillis() - 5L * 60 * 60 * 10000)),
                new Date(System.currentTimeMillis() - 3L * 60 * 60 * 1000));
        System.out.println(JsonUtil.toJson(list));
        /*
        {
	"dataPoints": [{
			"tags": {
				"teamId": "6",
				"hostId": "1303"
			},
			"measurement": "private.agent.cpu_total",
			"func": {
				"name": "last"
			},
			"time": "2019-08-29T06:20:33Z",
			"value": 17.35260786656643
		}, {
			"tags": {
				"teamId": "6",
				"hostId": "1303"
			},
			"measurement": "private.agent.cpu_total",
			"func": {
				"name": "first"
			},
			"time": "2019-08-27T07:20:44Z",
			"value": 7.7000790009795335
		}, {
			"tags": {
				"teamId": "6",
				"hostId": "1303"
			},
			"measurement": "private.agent.cpu_total",
			"func": {
				"name": "max"
			},
			"time": "2019-08-29T01:08:19Z",
			"value": 88.75416430568154
		}, {
			"tags": {
				"teamId": "6",
				"hostId": "1303"
			},
			"measurement": "private.agent.cpu_total",
			"func": {
				"name": "min"
			},
			"time": "2019-08-29T03:43:51Z",
			"value": 0.5741808639418745
		}, {
			"tags": {
				"teamId": "6",
				"hostId": "1303"
			},
			"measurement": "private.agent.cpu_total",
			"func": {
				"name": "percentile"
			},
			"time": "2019-08-28T07:06:02Z",
			"value": 32.38198094049532
		}
	]
}

         */

        list = client.monitorApi().getHostAggregation(1303,
                MonitorMetric.DiskUsed,//获取磁盘使用率统计数据
                //获取特定时间范围内：标准差，平均值，中值
                Arrays.asList(FuncUtil.create(FuncName.stddev), FuncUtil.create(FuncName.mean), FuncUtil.create(FuncName.median)),
                //特定时间：5小时以前到3小时以前
                new Date((System.currentTimeMillis() - 5L * 60 * 60 * 10000)),
                new Date(System.currentTimeMillis() - 3L * 60 * 60 * 1000));
        System.out.println(JsonUtil.toJson(list));
        /*
        {
	"dataPoints": [{
			"tags": {
				"diskname": "C:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "stddev"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 0.44491296349594817
		}, {
			"tags": {
				"diskname": "C:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "mean"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 55.78267873291601
		}, {
			"tags": {
				"diskname": "C:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "median"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 56.00000000000001
		}, {
			"tags": {
				"diskname": "D:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "stddev"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 0.4992416430491491
		}, {
			"tags": {
				"diskname": "D:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "mean"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 26.48943917483025
		}, {
			"tags": {
				"diskname": "D:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "median"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 26
		}, {
			"tags": {
				"diskname": "E:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "stddev"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 0
		}, {
			"tags": {
				"diskname": "E:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "mean"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 31
		}, {
			"tags": {
				"diskname": "E:",
				"hostId": "1303",
				"teamId": "6"
			},
			"measurement": "private.agent.diskusage_utilization",
			"func": {
				"name": "median"
			},
			"time": "2019-08-27T07:22:48Z",
			"value": 31
		}
	]
}
         */
    }
}
