package com.cloudbility.openapi.request.zstack;

import com.cloudbility.openapi.enums.zstack.ConsoleModeType;
import com.cloudbility.openapi.enums.zstack.HAType;
import com.cloudbility.openapi.enums.zstack.VMMachineType;

import java.io.Serializable;
import java.util.List;

/**
 * vm create systemTag常用参数
 */
public class VMCreateSystemTags implements Serializable {

    /**
     * 主机名
     * eg: hostname::host.zstack.org
     */
    private String hostname;

    /**
     * 设置vnc控制台密码
     * consolePassword::XXXXXX
     */
    private String consolePassword;

    /**
     * sshkey
     * sshkey::ssh-rsa XXXX     -> sshkey
     */
    private String sshKey;

    /**
     * 地缘组uuid
     * affinityGroupUuid::XXXX  -> 地缘组uuid
     */
    private String affinityGroupId;

    /**
     * userdata -> 传入时不需要Base64
     * userdata::Base64         -> 用户自定义脚本
     */
    private String userdata;

    /**
     * 控制台模式
     * vmConsoleMode::spice|vnc
     */
    private ConsoleModeType vmConsoleMode;

    /**
     * 高可用级别
     * ha::NeverStop|None
     */
    private HAType haType;

    /**
     * 指定静态ip
     * staticIp::三层网络UUID::指定的IP, eg: staticIp::81a21a81cde84c1084c191354053a3b5::172.20.196.0
     */
    private String staticIp;

    /**
     * 指定三层网络UUID
     */
    private String staticNetworkId;

    /**
     * 指定mac地址
     * customMac::三层网络UUID::指定的MAC地址, eg: customMac::81a21a81cde84c1084c191354053a3b5::fa:00:37:ee:9f:00
     */
    private String customMac;


    /////////////////////////////////
    /**
     * cpu比率
     * vmCpuPinning::1:3;1-3:4-7,^6 仅admin支持更新、创建、删除该SystemTag。克隆云主机时，同时会克隆此SystemTag
     */
    private String vmCpuPinning;

    /**
     * 云主机内安装的GuestTools版本
     * GuestTools::1.0.0, TOOLS_VERSION初值为1.0.0
     */
    private String guestTools;

    /**
     * 设置跨集群高可用策略的集群Id
     * resourceBindings::Cluster:clusterUuid, eg: resourceBindings::Cluster:2sdasf231jvznsdak
     */
    private String resourceBindingsClusterId;

    /**
     * mdv设备Uuid
     * mdevDevice::MDEV_DEV_UUID, eg: mdevDevice::e2af8f869eff49d2a3d6f86cadc27090
     */
    private String mdevDeviceId;

    /**
     * MDEV设备规格
     * mdevDeviceSpec::MDEV_SPEC_UUID::MDEV_DEV_NUM, eg: mdevDeviceSpec::e2af8f869eff49d2a3d6f86cadc27090::4
     */
    private String mdevDeviceSpecId;
    private Integer mdevDeviceSpecNum;

    /**
     * 是否创建cdrom
     * reateWithoutCdRom::(true|false)
     *
     * 创建一个空的cdrom:    cdroms::empty::none::none
     * 创建三个空的cdrom:    cdroms::empty::empty::empty
     * 创建3个挂载了iso的cdrom：cdroms::iso_1_uuid::iso_2_uuid::iso_3_uuid
     */
    private Boolean createWithoutCdRom;

    /**
     * 挂载的cdrom列表: [none | empty | isoUuid]
     */
    private List<String> cdRoms;

    /**
     * 根云盘所在的Ceph存储池
     * ceph::rootPoolName::pri-v-r-26e11ca6814d4e1ba504d845f7848db3
     * eg: ceph::rootPoolName::pri-v-r-26e11ca6814d4e1ba504d845f7848db3
     */
    private String cephRootPoolName;

    /**
     * 是否关机自动卸载从设备规格分配的物理PCI设备
     * autoReleaseSpecReleatedPhysicalPciDevice::true
     */
    private Boolean autoReleaseSpecReleatedPhysicalPciDevice;

    /**
     * 是否关机自动卸载从设备规格分配的虚拟PCI设备
     * autoReleaseSpecReleatedVirtualPciDevice::true
     */
    private Boolean autoReleaseSpecReleatedVirtualPciDevice;

    /**
     * pci设备, eg: pciDevice::XXXX
     */
    private String pciDevice;

    /**
     * 指定PCI设备规格和数量
     * pciDeviceSpec::PCI_SPEC_UUID::PCI_DEV_NUM, eg: pciDeviceSpec::e2af8f869eff49d2a3d6f86cadc27090::4
     */
    private String pciDeviceSpecId;
    private Integer pciDeviceSpecNum;

    /**
     * 主板类型, eg: vmMachineType::pc|q35
     */
    private VMMachineType vmMachineType;

    /**
     * 支持启用SR-IOV功能的网络Id
     * enableSRIOV::{L3_NETWORK_UUID}, eg: enableSRIOV::9e19dafe81c64fed8e34f72e27582339
     */
    private String enableSRIOVNetworkId;

    /**
     * usb重定向
     * usbRedirect::true|false
     */
    private Boolean usbRedirect;

    /**
     * 防欺诈模式
     */
    private Boolean cleanTraffic;


    public String getHostname() {
        return hostname;
    }

    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    public String getConsolePassword() {
        return consolePassword;
    }

    public void setConsolePassword(String consolePassword) {
        this.consolePassword = consolePassword;
    }

    public String getSshKey() {
        return sshKey;
    }

    public void setSshKey(String sshKey) {
        this.sshKey = sshKey;
    }

    public String getAffinityGroupId() {
        return affinityGroupId;
    }

    public void setAffinityGroupId(String affinityGroupId) {
        this.affinityGroupId = affinityGroupId;
    }

    public String getUserdata() {
        return userdata;
    }

    public void setUserdata(String userdata) {
        this.userdata = userdata;
    }

    public ConsoleModeType getVmConsoleMode() {
        return vmConsoleMode;
    }

    public void setVmConsoleMode(ConsoleModeType vmConsoleMode) {
        this.vmConsoleMode = vmConsoleMode;
    }

    public HAType getHaType() {
        return haType;
    }

    public void setHaType(HAType haType) {
        this.haType = haType;
    }

    public String getStaticIp() {
        return staticIp;
    }

    public void setStaticIp(String staticIp) {
        this.staticIp = staticIp;
    }

    public String getStaticNetworkId() {
        return staticNetworkId;
    }

    public void setStaticNetworkId(String staticNetworkId) {
        this.staticNetworkId = staticNetworkId;
    }

    public String getCustomMac() {
        return customMac;
    }

    public void setCustomMac(String customMac) {
        this.customMac = customMac;
    }

    public String getVmCpuPinning() {
        return vmCpuPinning;
    }

    public void setVmCpuPinning(String vmCpuPinning) {
        this.vmCpuPinning = vmCpuPinning;
    }

    public String getGuestTools() {
        return guestTools;
    }

    public void setGuestTools(String guestTools) {
        this.guestTools = guestTools;
    }

    public String getResourceBindingsClusterId() {
        return resourceBindingsClusterId;
    }

    public void setResourceBindingsClusterId(String resourceBindingsClusterId) {
        this.resourceBindingsClusterId = resourceBindingsClusterId;
    }

    public String getMdevDeviceId() {
        return mdevDeviceId;
    }

    public void setMdevDeviceId(String mdevDeviceId) {
        this.mdevDeviceId = mdevDeviceId;
    }

    public String getMdevDeviceSpecId() {
        return mdevDeviceSpecId;
    }

    public void setMdevDeviceSpecId(String mdevDeviceSpecId) {
        this.mdevDeviceSpecId = mdevDeviceSpecId;
    }

    public Integer getMdevDeviceSpecNum() {
        return mdevDeviceSpecNum;
    }

    public void setMdevDeviceSpecNum(Integer mdevDeviceSpecNum) {
        this.mdevDeviceSpecNum = mdevDeviceSpecNum;
    }

    public Boolean getCreateWithoutCdRom() {
        return createWithoutCdRom;
    }

    public void setCreateWithoutCdRom(Boolean createWithoutCdRom) {
        this.createWithoutCdRom = createWithoutCdRom;
    }

    public List<String> getCdRoms() {
        return cdRoms;
    }

    public void setCdRoms(List<String> cdRoms) {
        this.cdRoms = cdRoms;
    }

    public String getCephRootPoolName() {
        return cephRootPoolName;
    }

    public void setCephRootPoolName(String cephRootPoolName) {
        this.cephRootPoolName = cephRootPoolName;
    }

    public String getPciDevice() {
        return pciDevice;
    }

    public void setPciDevice(String pciDevice) {
        this.pciDevice = pciDevice;
    }

    public Boolean getAutoReleaseSpecReleatedPhysicalPciDevice() {
        return autoReleaseSpecReleatedPhysicalPciDevice;
    }

    public void setAutoReleaseSpecReleatedPhysicalPciDevice(Boolean autoReleaseSpecReleatedPhysicalPciDevice) {
        this.autoReleaseSpecReleatedPhysicalPciDevice = autoReleaseSpecReleatedPhysicalPciDevice;
    }

    public Boolean getAutoReleaseSpecReleatedVirtualPciDevice() {
        return autoReleaseSpecReleatedVirtualPciDevice;
    }

    public void setAutoReleaseSpecReleatedVirtualPciDevice(Boolean autoReleaseSpecReleatedVirtualPciDevice) {
        this.autoReleaseSpecReleatedVirtualPciDevice = autoReleaseSpecReleatedVirtualPciDevice;
    }

    public String getPciDeviceSpecId() {
        return pciDeviceSpecId;
    }

    public void setPciDeviceSpecId(String pciDeviceSpecId) {
        this.pciDeviceSpecId = pciDeviceSpecId;
    }

    public Integer getPciDeviceSpecNum() {
        return pciDeviceSpecNum;
    }

    public void setPciDeviceSpecNum(Integer pciDeviceSpecNum) {
        this.pciDeviceSpecNum = pciDeviceSpecNum;
    }

    public VMMachineType getVmMachineType() {
        return vmMachineType;
    }

    public void setVmMachineType(VMMachineType vmMachineType) {
        this.vmMachineType = vmMachineType;
    }

    public String getEnableSRIOVNetworkId() {
        return enableSRIOVNetworkId;
    }

    public void setEnableSRIOVNetworkId(String enableSRIOVNetworkId) {
        this.enableSRIOVNetworkId = enableSRIOVNetworkId;
    }

    public Boolean getUsbRedirect() {
        return usbRedirect;
    }

    public void setUsbRedirect(Boolean usbRedirect) {
        this.usbRedirect = usbRedirect;
    }

    public Boolean getCleanTraffic() {
        return cleanTraffic;
    }

    public void setCleanTraffic(Boolean cleanTraffic) {
        this.cleanTraffic = cleanTraffic;
    }

}
