package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.Permissions;
import com.cloudbility.openapi.vo.CasAppVo;
import com.cloudbility.openapi.vo.CasLogoutTypeVo;
import com.cloudbility.openapi.vo.CasServiceTypeVo;
import com.cloudbility.openapi.vo.Permission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "23 cas server api", description = "SSO服务端API")
public interface CasServerApi {
    @ApiOperation(value = "创建一个CAS应用", notes = "指定teamId的app，在serviceValidate阶段，会在attributes里面添加一个字段teamId：{teamId}标识这个团队ID\n" +
            "evaluationOrder:当系统创建了多个serviceId是正则表达式的时候，寻找到匹配的app的校验顺序。这个值约小，优先级越高。")
    @RequestMapping(value = "/cas/app", method = RequestMethod.POST)
    @Permissions(Permission.CreateCasServer)
    CasAppVo createCasApp(
            @ApiParam(value = "服务ID，全局唯一", required = true) @RequestParam("serviceId") String serviceId,
            @ApiParam(value = "应用名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "应用所属模块", required = false) @RequestParam(value = "module", required = false) String module,
            @ApiParam(value = "服务ID类型，完全匹配或者正则匹配", required = true) @RequestParam(value = "serviceType", defaultValue = "Identical") CasServiceTypeVo serviceType,
            @ApiParam("指定所属团队") @RequestParam(value = "teamId", required = false) Long teamId,
            @ApiParam(value = "SLO单点登出方式，默认是back channel", required = true) @RequestParam(value = "logoutType", defaultValue = "back") CasLogoutTypeVo logoutType,
            @ApiParam("指定app全局的登录URL，如果未指定，将使用SSO过程中的service参数发起注销") @RequestParam(value = "logoutUrl", required = false) String logoutUrl,
            @ApiParam("匹配服务ID的顺序") @RequestParam(value = "evaluationOrder", required = false, defaultValue = "0") Integer evaluationOrder,
            @ApiParam("注销CasServer强制注销SSO会话") @RequestParam(value = "forceLogout", required = false, defaultValue = "true") boolean forceLogout
    ) throws OpenapiException;

    @ApiOperation(value = "删除一个CAS应用")
    @RequestMapping(value = "/cas/app", method = RequestMethod.DELETE)
    @Permissions(Permission.CreateCasServer)
    void deleteCasApp(@ApiParam(value = "服务ID，全局唯一", required = true) @RequestParam("serviceId") String serviceId) throws OpenapiException;
}
