package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.NoneSAAS;
import com.cloudbility.openapi.request.CreateCloudRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@Api(tags = "Cloud api", description = "云账户操作API")
public interface CloudApi extends Openapi {

    @ApiOperation(value = "根据ID查询云账户")
    @RequestMapping(value = "/cloud/{id}", method = RequestMethod.GET)
    CloudVo getCloudById(@ApiParam(value = "云账户ID", required = true) @PathVariable("id") Long cloudId) throws OpenapiException;


    @ApiOperation(value = "根据ID查询云账户详情")
    @RequestMapping(value = "/cloud/detail", method = RequestMethod.GET)
    CloudVo getCloudDetail(@ApiParam(value = "云账户ID", required = true) @RequestParam(value = "cloudId", required = true) Long cloudId,
                           @ApiParam(value = "云账户来源", required = false) @RequestParam(value = "scope", required = false, defaultValue = "Portal") Scope scope) throws OpenapiException;

    @ApiOperation(value = "批量创建局域网云账户", response = CreateCloudResult.class)
    @RequestMapping(value = "/cloud", method = RequestMethod.POST)
    CreateCloudResult createClouds(@ApiParam(value = "云账户列表", required = true) @RequestBody CreateCloudRequest request) throws OpenapiException;


    @ApiOperation(value = "创建一个局域网云账户", response = CreateCloudResult.class, notes = "1，参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。" +
            "例如：roles='12,TEAM_ADMINER'表示id=12的角色和团队管理员角色。roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色.。" +
            "<br/> 2,网段划分方式:4.18新增，默认是auto(自动按C网段划分)。manual:按设定的CIDR划分；cidr串：手工（cidrRule=manual）指定cidr网段，4.18新增，用逗号连接。" +
            "<br/>3,referencedPolicyId指定团队内的一个运维子策略ID，新建的云账户将引用这个策略中的条目设置默认值，4.18新增。")
    @RequestMapping(value = "/cloudV2", method = RequestMethod.POST)
    CreateCloudResult createCloud(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "创建者用户ID，未设置时使用团队拥有者ID") @RequestParam(value = "creatorId", required = false) Long creatorId,
            @ApiParam(value = "新建云账户默认授予的角色，如果不设置时，只有团队拥有者能访问云账户") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "内网连接方式，默认proxy", required = true) @RequestParam(value = "intranetAccessType", defaultValue = "proxy") CloudIntranetAccessType intranetAccessType,
            @ApiParam(value = "网段划分方式", required = true) @RequestParam(value = "cidrRule", defaultValue = "auto") CloudCidrRule cidrRule,
            @ApiParam(value = "cidr串") @RequestParam(value = "cidrs", required = false) String cidrs,
            @ApiParam(value = "参考运维子策略ID", required = false) @RequestParam(value = "referencedPolicyId", required = false) Long referencedPolicyId
    ) throws OpenapiException;

    @ApiOperation(value = "创建一个公有云云账户，不包括Azure", response = CloudVo.class,
            notes = "参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示id=12的角色和团队管理员角色。roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/cloud/public", method = RequestMethod.POST)
    CloudVo createPublicCloud(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "创建者用户ID，未设置时使用团队拥有者ID") @RequestParam(value = "creatorId", required = false) Long creatorId,
            @ApiParam(value = "新建云账户默认授予的角色，如果不设置时，只有团队拥有者能访问云账户") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "Access Key Id", required = true) @RequestParam("akId") String accessKeyId,
            @ApiParam(value = "Access Key Secret", required = true) @RequestParam("akSecret") String accessKeySecret,
            @ApiParam(value = "云厂商", required = true) @RequestParam("provider") CloudProvider provider,
            @ApiParam(value = "内网连接方式，默认proxy", required = true) @RequestParam(value = "intranetAccessType", defaultValue = "proxy") CloudIntranetAccessType intranetAccessType
    ) throws OpenapiException;

    @ApiOperation(value = "创建一个Openstack云账号", response = CloudVo.class,
            notes = "参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示id=12的角色和团队管理员角色。roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/cloud/openstack", method = RequestMethod.POST)
    CloudVo createOpenstackCloud(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "创建者用户ID，未设置时使用团队拥有者ID") @RequestParam(value = "creatorId", required = false) Long creatorId,
            @ApiParam(value = "新建云账户默认授予的角色，如果不设置时，只有团队拥有者能访问云账户") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "authUrl", required = true) @RequestParam("authUrl") String authUrl,
            @ApiParam(value = "username", required = true) @RequestParam("username") String username,
            @ApiParam(value = "password", required = true) @RequestParam("password") String password,
            @ApiParam(value = "domainName", required = true) @RequestParam("domainName") String domainName,
            @ApiParam(value = "projectId", required = true) @RequestParam("projectId") String projectId,
            @ApiParam(value = "内网连接方式，默认proxy", required = true) @RequestParam(value = "intranetAccessType", defaultValue = "proxy") CloudIntranetAccessType intranetAccessType
    ) throws OpenapiException;


    @ApiOperation(value = "创建一个Azure云账号", response = CloudVo.class,
            notes = "参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示id=12的角色和团队管理员角色。roles不能设置为TEAM_CREATOR。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/cloud/azure", method = RequestMethod.POST)
    CloudVo createAzureCloud(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "创建者用户ID，未设置时使用团队拥有者ID") @RequestParam(value = "creatorId", required = false) Long creatorId,
            @ApiParam(value = "新建云账户默认授予的角色，如果不设置时，只有团队拥有者能访问云账户") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "subscriptionId", required = true) @RequestParam("subscriptionId") String subscriptionId,
            @ApiParam(value = "tenantId", required = true) @RequestParam("tenantId") String tenantId,
            @ApiParam(value = "clientId", required = true) @RequestParam("clientId") String clientId,
            @ApiParam(value = "clientSecret", required = true) @RequestParam("clientSecret") String clientSecret,
            @ApiParam(value = "内网连接方式，默认proxy", required = true) @RequestParam(value = "intranetAccessType", defaultValue = "proxy") CloudIntranetAccessType intranetAccessType
    ) throws OpenapiException;

    @ApiOperation(value = "删除云账户")
    @RequestMapping(value = "/cloud/{ids}", method = RequestMethod.DELETE)
    void deleteClouds(@ApiParam(value = "云账户id列表，用','连接", required = true) @PathVariable("ids") String idStr) throws OpenapiException;


    @ApiOperation(value = "修改云账户", notes = "包括两部分：修改云账户名称、重新指定授权角色。name和roles两个参数不能同时为空。为空时，不会修改。私有部署版包括通过role_templates.json配置的系统角色")
    @RequestMapping(value = "/cloud/{id}", method = RequestMethod.PUT)
    void updateCloud(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id,
                     @ApiParam(value = "云账户名称") @RequestParam(value = "name", required = false) String name,
                     @Deprecated
                     @ApiParam(value = "新的角色授权") @RequestParam(value = "roles", required = false) String roles) throws OpenapiException;

    @ApiOperation(value = "授权云账户给用户")
    @RequestMapping(value = "/cloud/{id}/members", method = RequestMethod.PUT)
    void updateCloudMembers(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id,
                            @ApiParam(value = "用户ID列表，用逗号连接") @RequestParam(value = "members", required = false) String members) throws OpenapiException;

    @ApiOperation(value = "授权云账户给角色")
    @RequestMapping(value = "/cloud/{id}/roles", method = RequestMethod.PUT)
    void updateCloudRoles(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id,
                          @ApiParam(value = "角色ID列表，用逗号连接") @RequestParam(value = "roles", required = false) String roles) throws OpenapiException;

    @ApiOperation(value = "授权云账户给组织单元")
    @RequestMapping(value = "/cloud/{id}/departments", method = RequestMethod.PUT)
    void updateCloudDepartments(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id,
                                @ApiParam(value = "组织单元ID列表，用逗号连接") @RequestParam(value = "departments", required = false) String departments) throws OpenapiException;

    @ApiOperation(value = "修改云账户内网连接方式", notes = "仅私有部署版支持")
    @RequestMapping(value = "/cloud/{id}/intranetAccessType", method = RequestMethod.PUT)
    @NoneSAAS
    void updateIntranetAccessType(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id,
                                  @ApiParam(value = "内网连接方式", required = true) @RequestParam("intranetAccessType") CloudIntranetAccessType innerAccessType) throws OpenapiException;

    @ApiOperation(value = "修改（局域网）云账户网段划分规则", notes = "注意：1，仅局域网云账户支持修改；2，修改之后，主机会按照新的规则重新组织分组；3，耗时操作，后台系统异步完成请求；")
    @RequestMapping(value = "/cloud/{id}/cidrRule", method = RequestMethod.PUT)
    @NoneSAAS
    JobVo updateLanCloudCidrRule(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id,
                                 @ApiParam(value = "网段划分方式", required = true) @RequestParam(value = "cidrRule") CloudCidrRule cidrRule,
                                 @ApiParam(value = "cidr串。在cidrRule=manual时必须指定cidr网段", required = true) @RequestParam(value = "cidrs", required = false) String cidrs) throws OpenapiException;

    @ApiOperation(value = "撤销云账户的所有角色授权", notes = "清空角色授权以后，只有团队拥有者有权限访问云账户")
    @RequestMapping(value = "/cloud/{id}/revokeAllRoles", method = RequestMethod.PUT)
    void cleanCloudRoles(@ApiParam(value = "云账户id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "查询团队所有云账户列表")
    @RequestMapping(value = "/cloud/byTeam/{teamId}", method = RequestMethod.GET)
    ListCloudsResult findCloudByTeam(@ApiParam(value = "团队ID", required = true) @PathVariable("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "查询团队所有云账户列表（根据teamCode）")
    @RequestMapping(value = "/cloud/byTeamCode/{teamCode:.+}", method = RequestMethod.GET)
    ListCloudsResult findCloudByTeamCode(@ApiParam(value = "团队Code", required = true) @PathVariable("teamCode") String teamCode) throws OpenapiException;

    @ApiOperation(value = "获取Proxy安装脚本", response = GetProxyScriptResult.class)
    @RequestMapping(value = "/cloud/{cloudId}/proxyScript", method = RequestMethod.GET)
    GetProxyScriptResult getProxyScript(@ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") Long cloudId, @ApiParam(value = "base64", defaultValue = "false") boolean base64) throws OpenapiException;

    @ApiOperation(value = "获取云厂商的所有区域", response = GetRegionResult.class)
    @RequestMapping(value = "/region/byProvider/{provider}", method = RequestMethod.GET)
    GetRegionResult getRegions(@ApiParam(value = "云厂商", required = true) @PathVariable("provider") String provider) throws OpenapiException;

    @ApiOperation(value = "获取云厂商的所有区域", response = GetRegionResult.class)
    @RequestMapping(value = "/region/byCloud/{cloudId}", method = RequestMethod.GET)
    GetRegionResult getRegions(@ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") Long cloudId) throws OpenapiException;

    @ApiOperation(value = "获取云账户的项目列表", response = GetRegionResult.class)
    @RequestMapping(value = "/cloud/projects", method = RequestMethod.GET)
    GetProjectResult getProjects(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                 @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId) throws OpenapiException;

    @ApiOperation(value = "获取区域下可售卖的实例规格", response = InstanceTypeResult.class)
    @RequestMapping(value = "/cloud/availableInstanceTypes", method = RequestMethod.GET)
    InstanceTypeResult listAvailableInstanceTypes(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                  @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
                                                  @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
                                                  @ApiParam(value = "实例收费类型", required = true) @RequestParam(value = "instanceChargeType", defaultValue = "PostPaid") CloudInstanceChargeType instanceChargeType) throws OpenapiException;

    @ApiOperation(value = "获取区域下可售卖的磁盘规格", response = InstanceTypeResult.class)
    @RequestMapping(value = "/cloud/availableDiskTypes", method = RequestMethod.GET)
    DiskTypeResult listAvailableDiskTypes(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                          @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
                                          @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
                                          @ApiParam(value = "实例规格", required = false) @RequestParam(value = "instanceType", required = false) String instanceType) throws OpenapiException;

}
