package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.DbType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "Database api", description = "数据库API")
public interface DbAPi extends Openapi {

    @ApiOperation(value = "查询用户有权限的DB实例", response = ListDbResult.class)
    @RequestMapping(value = "/db/byUser", method = RequestMethod.GET)
    ListDbResult listDbs(@ApiParam("团队ID") @RequestParam("teamId") Long teamId,
                         @ApiParam("用户ID") @RequestParam("userId") Long userId) throws OpenapiException;

    @ApiOperation(value = "分页查询团队DB实例", response = ListDbResult.class)
    @RequestMapping(value = "/db", method = RequestMethod.GET)
    ListDbResult listDbsByPage(@ApiParam("团队ID") @RequestParam("teamId") Long teamId,
                               @ApiParam("页码") @RequestParam(value = "page", defaultValue = "1") Integer page,
                               @ApiParam("数量") @RequestParam(value = "size", defaultValue = "10") Integer size) throws OpenapiException;

    @ApiOperation(value = "创建一个DB", response = DbVo.class)
    @RequestMapping(value = "/db", method = RequestMethod.POST)
    DbVo createDb(
            @ApiParam("团队ID") @RequestParam("teamId") Long teamId,
            @ApiParam("DB名称") @RequestParam("name") String name,
            @ApiParam("DB类型") @RequestParam("dbType") DbType dbType,
            @ApiParam("DB连接IP") @RequestParam(value = "ip", required = false) String ip,
            @ApiParam("DB连接端口") @RequestParam(value = "port", required = false) Integer port,
            @ApiParam("跳板机ID") @RequestParam("jumpServerId") Long jumpServerId,
            @ApiParam("跳板机类型") @RequestParam("jumpServerType") JumpServerType jumpServerType,
            @ApiParam("创建者ID（未设置时，默认团队创建者）") @RequestParam(value = "creatorId", required = false) Long creatorId,
            @ApiParam("DB位置：主机、RDS") @RequestParam("location") DbLocation location,
            @ApiParam("跳板机端口，不设置或设置为0时，自动寻找可用端口") @RequestParam(value = "jumpServerPort", required = false) Integer jumpServerPort,
            @ApiParam("DB宿主机ID，RDS时为null") @RequestParam(value = "hostId", required = false) Long hostId,
            @ApiParam("DB分组ID") @RequestParam(value = "dbGroupId", required = false) Long dbGroupId,
            @ApiParam("DB集群连接串") @RequestParam(value = "clusterUri", required = false) String clusterUri,
            @ApiParam("DB是否集群模式") @RequestParam(value = "isCluster", required = false) Boolean isCluster,
            @ApiParam("创建后启动代理") @RequestParam(value = "autoStart", required = false) boolean autoStart
    ) throws OpenapiException;


    @ApiOperation(value = "修改DB")
    @RequestMapping(value = "/db/{dbId}", method = RequestMethod.PUT)
    void editDb(
            @ApiParam("DBID") @PathVariable("dbId") Long dbId,
            @ApiParam("执行删除操作的用户Id") @PathVariable("userId") Long userId,
            @ApiParam("DB名称") @RequestParam("name") String name,
            @ApiParam("DB连接端口") @RequestParam(value = "port", required = false) Integer port,
            @ApiParam("跳板机ID") @RequestParam("jumpServerId") Long jumpServerId,
            @ApiParam("DB分组ID") @RequestParam(value = "dbGroupId", required = false) Long dbGroupId,
            @ApiParam("DB类型") @RequestParam(value = "dbType", required = false) DbType dbType,
            @ApiParam("跳板机类型") @RequestParam(value = "jumpServerType", required = false) JumpServerType jumpServerType,
            @ApiParam("DB连接IP") @RequestParam(value = "ip", required = false) String ip,
            @ApiParam("DB集群连接串") @RequestParam(value = "clusterUri", required = false) String clusterUri,
            @ApiParam("DB是否集群模式") @RequestParam(value = "isCluster", required = false) Boolean isCluster,
            @ApiParam("DB连接IP是否是内网IP") @RequestParam(value = "isInner", defaultValue = "false") String isInner
    ) throws OpenapiException;

    @ApiOperation(value = "删除DB")
    @RequestMapping(value = "/db/{dbId}", method = RequestMethod.DELETE)
    void deleteDb(@ApiParam("dbId") @PathVariable("dbId") Long dbId,
                             @ApiParam("执行删除操作的用户Id") @RequestParam("userId") Long userId) throws OpenapiException;

    @ApiOperation(value = "创建一个DBPlan")
    @RequestMapping(value = "/db/plan", method = RequestMethod.POST)
    void createDbPlan(
            @ApiParam("DBID") @RequestParam("dbId") Long dbId,
            @ApiParam("DB名称") @RequestParam("name") String name,
            @ApiParam("DB账号") @RequestParam("dbUser") String dbUser,
            @ApiParam("DB密码") @RequestParam("dbPassword") String dbPassword,
            @ApiParam("团队ID") @RequestParam("teamId") Long teamId,
            @ApiParam("创建者ID（未设置时，默认团队创建者）") @RequestParam(value = "creatorId", required = false) Long creatorId,
            @ApiParam("DB实例") @RequestParam(value = "dbInstance", required = false) String dbInstance,
            @ApiParam("DB描述") @RequestParam(value = "description", required = false) String description,
            @ApiParam("DBPlan关联角色") @RequestParam(value = "roleIds", required = false) String roleIds,
            @ApiParam("DBPlan关联用户") @RequestParam(value = "userIds", required = false) String userIds
    ) throws OpenapiException;


    @ApiOperation(value = "删除DBPlan")
    @RequestMapping(value = "/db/plan/{planId}", method = RequestMethod.DELETE)
    void deleteDbPlan(@ApiParam("planId") @PathVariable("planId") Long planId,
                                 @ApiParam("执行删除操作的用户Id") @RequestParam("userId") Long userId) throws OpenapiException;

}
