package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.Permissions;
import com.cloudbility.openapi.request.CloudHostTemplateVo;
import com.cloudbility.openapi.request.ImportHostRequest;
import com.cloudbility.openapi.request.ImportPublicHostRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.response.hostTemplate.CloudHostTemplateInfoResult;
import com.cloudbility.openapi.response.hostTemplate.HostTemplateResult;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;

@Api(tags = "Host api", description = "主机操作API")
public interface HostApi extends Openapi {

    @ApiOperation(value = "批量导入局域网主机", response = ImportHostResult.class, notes = "可以根据vgIds参数选择是否默认把新建主机加入到虚拟分组中,vgIds代表的虚拟分组必须属于当前团队。4.8版本以后，导入主机时创建的物理分组将不再默认授权给所有成员。资源授权请使用虚拟分组API。")
    @RequestMapping(value = "/host", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    ImportHostResult importHosts(@ApiParam(value = "主机列表", required = true) @RequestBody ImportHostRequest request) throws OpenapiException;

    @ApiOperation(value = "导入一台局域网主机")
    @RequestMapping(value = "/hostV2", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    public ImportHostResult importHost(
            @ApiParam(value = "局域网云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "内网IP", required = true) @RequestParam("innerIp") String innerIp,
            @ApiParam(value = "公网IP", required = false) @RequestParam(value = "publicIp", required = false) String publicIp,
            @ApiParam(value = "厂商提供的ID，在云账户中唯一，不设置时系统自动生成", required = false) @RequestParam(value = "instanceId", required = false) String instanceId,
            @ApiParam(value = "主机名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "描述信息") @RequestParam(value = "description", required = false) String description,
            @ApiParam(value = "操作系统：", required = true) @RequestParam("operatingSystem") String operatingSystem,
            @ApiParam(value = "CPU核数", required = false) @RequestParam(value = "processor", required = false, defaultValue = "0") int processor,
            @ApiParam(value = "内存，单位MB", required = false) @RequestParam(value = "memory", required = false, defaultValue = "0") int memory,
            @ApiParam(value = "主机状态", required = false) @RequestParam(value = "status", defaultValue = "Unknown") HostStatus status,
            @ApiParam(value = "默认加入的虚拟分组", required = false) @RequestParam(value = "vgIds" , required = false) String vgIds
    ) throws OpenapiException;

    @ApiOperation(value = "导入一台公有云主机，不支持Azure", response = JobVo.class, notes = "导入过程创建的物理分组不会设置任何角色，即只有团队拥有者可见。如果物理分组已经存在则不会改变已有授权结构")
    @RequestMapping(value = "/host/public", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    JobVo importPublicHost(
            @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") long cloudId,
            @ApiParam(value = "云厂商区域ID", required = true) @RequestParam("regionId") String regionId,
            @ApiParam(value = "云主机instanceId", required = true) @RequestParam("instanceId") String instanceId,
            @ApiParam(value = "导入主机的用户ID，不设置时，是团队所有者") @RequestParam(value = "creatorId", required = false) Long creatorId
    ) throws OpenapiException;

    @ApiOperation(value = "批量导入公有云主机，不支持Azure", response = JobVo.class, notes = "导入过程创建的物理分组不会设置任何角色，即只有团队拥有者可见。如果物理分组已经存在则不会改变已有授权结构")
    @RequestMapping(value = "/hosts/public", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    JobVo importPublicHost(@ApiParam(value = "实例ID和RegionId信息", required = true) @RequestBody ImportPublicHostRequest request) throws OpenapiException;

    @ApiOperation(value = "删除主机", notes = "删除的主机，可以来自API导入和行云管家页面导入")
    @RequestMapping(value = "/host/{ids}", method = RequestMethod.DELETE)
    @Permissions(Permission.HostAccess)
    void deleteHosts(@ApiParam(value = "主机id列表，用','连接", required = true) @PathVariable("ids") String idStr) throws OpenapiException;

    @ApiOperation(value = "更新主机信息", notes = "不需要更新的属性，请保持默认值")
    @RequestMapping(value = "/host/{id}", method = RequestMethod.PUT)
    @Permissions(Permission.HostAccess)
    void updateHost(@ApiParam(value = "主机id", required = true) @PathVariable("id") long hostId,
                    @ApiParam(value = "主机信息", required = true) @RequestBody ImportHostVo hostVo) throws OpenapiException;

    @ApiOperation(value = "获取主机详情", response = CloudHostVo.class)
    @RequestMapping(value = "/host/{id}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    public CloudHostVo getHost(@ApiParam(value = "主机id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @ApiOperation(value = "获取主机ID列表", response = GetHostBriefResult.class, notes = "包括API导入和行云管家页面导入的主机ID")
    @RequestMapping(value = "/host/findIds", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    GetHostBriefResult findUserIds(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取主机状态", response = GetHostsStatusResult.class, notes = "包括API导入和行云管家页面导入的主机ID")
    @RequestMapping(value = "/host/findStatus", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    GetHostsStatusResult getHostStatus(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                       @ApiParam(value = "主机ID列表", required = false) @RequestParam(value = "hostIds", required = false) String hostIds) throws OpenapiException;

    @ApiOperation(value = "根据IP地址获取主机信息", notes = "包括API导入和行云管家页面导入的主机。匹配规则：内网IP或公网IP匹配给定IP")
    @RequestMapping(value = "/host/findByIp", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findByIp(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
            @ApiParam(value = "云账户id") @RequestParam(value = "cloudId", required = false) Long cloudId,
            @ApiParam(value = "IP地址", required = true) @RequestParam(value = "ip") String ip) throws OpenapiException;

    @ApiOperation(value = "根据内网和公网IP获取主机信息", notes = "包括API导入和行云管家页面导入的主机。匹配规则：内网IP=innerIp（如果设置） 且 公网IP=publicIp（如果设置）。innerIp和publicIp两个参数必须至少指定一个")
    @RequestMapping(value = "/host/findByInnerIpAndPublicIp", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findByInnerIpAndPublicIp(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
            @ApiParam(value = "云账户id") @RequestParam(value = "cloudId", required = false) Long cloudId,
            @ApiParam(value = "内网IP") @RequestParam(value = "innerIp", required = false) String innerIp,
            @ApiParam(value = "公网IP") @RequestParam(value = "publicIp", required = false) String publicIp
    ) throws OpenapiException;

    @ApiOperation(value = "根据instanceId获取主机信息", notes = "包括云主机和局域网主机。instanceId可以是云主机的实例ID或者Openapi导入主机时用户指定的instanceId，或者由系统自动生成的instanceId")
    @RequestMapping(value = "/host/findByInstanceId", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findByInstanceId(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
            @ApiParam(value = "云账户id") @RequestParam(value = "cloudId", required = false) Long cloudId,
            @ApiParam(value = "instanceId", required = true) @RequestParam(value = "instanceId") String instanceId) throws OpenapiException;

    @ApiOperation(value = "获取云账户中所有主机")
    @RequestMapping(value = "/host/byCloud/{cloudId}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findByCloud(
            @ApiParam(value = "云账户id", required = true) @PathVariable(value = "cloudId") Long cloudId,
            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;


    @ApiOperation(value = "获取团队中所有主机")
    @RequestMapping(value = "/host/byTeam/{teamId}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findByTeam(
            @ApiParam(value = "团队ID", required = true) @PathVariable(value = "teamId") Long teamId,
            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "获取团队中所有主机（根据团队code）")
    @RequestMapping(value = "/host/byTeamCode/{teamCode}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findByTeam(
            @ApiParam(value = "团队code", required = true) @PathVariable(value = "teamCode") String teamCode,
            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;


    @ApiOperation(value = "更新主机所有登录凭证密码", response = ListCredentialsResult.class, notes = "同时满足下面条件的登录凭证都会被修改：1，认证类型是密码而不是密钥文件；2，登录凭证关联到当前主机。风险提示：不会校验被修改的登录凭证原来密码是否正确，都会被修改成指定的密码，可能使原本错误的登录凭证可以登录成功。返回被修改的登录凭证列表。")
    @RequestMapping(value = "/host/{id}/credentials", method = RequestMethod.PUT)
    @Permissions(Permission.HostAccess)
    ListCredentialsResult updateCredentials(@ApiParam(value = "主机id", required = true) @PathVariable("id") long hostId,
                                            @ApiParam(value = "用户名", required = true) @RequestParam(value = "user") String user,
                                            @ApiParam(value = "登录密码，长度>=4，不校验复杂性", required = true) @RequestParam(value = "password") String password,
                                            @ApiParam(value = "密码是否是加密字串，可选。默认为false") @RequestParam(value = "encryptSensitive", required = false) boolean encryptSensitive) throws OpenapiException;

    @ApiOperation(value = "获取主机Agent安装脚本", response = GetAgentScriptResult.class)
    @RequestMapping(value = "/host/{id}/agentScript", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    GetAgentScriptResult getHostAgentScript(@ApiParam(value = "主机id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @ApiOperation(value = "获取云账户的物理网络列表", notes = "物理网络，也叫主机物理分组。在公有云账户中，按区域进行划分；在局域网账户中，按子网进行划分（比如主机192.168.1.123属于物理分组：192.168.1.*。物理网络也可以进行角色授权")
    @RequestMapping(value = "/hg/byCloud/{cloudId}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    HostGroupVoResult getHostGroups(@ApiParam(value = "云账户ID", required = true) @PathVariable("cloudId") long cloudId) throws OpenapiException;

//    @ApiOperation(value = "获取主机登录凭证列表", response = CredentialHostsResult.class)
//    @RequestMapping(value = "/host/{id}/credentials", method = RequestMethod.GET)
//     GetCredentialListResult getHostCredentials(@ApiParam(value = "主机ID", required = true) @PathVariable("id") Long hostId) throws OpenapiException {
//        return ResponseEntity.ok(isvOpenapiService.getCredentialsByHost(OpenapiContext.get().getUser(), hostId));
//    }


    @ApiOperation(value = "撤销物理分组的所有角色授权", notes = "清空角色授权以后，只有团队拥有者有权限")
    @RequestMapping(value = "/hg/{id}/revokeAllRoles", method = RequestMethod.PUT)
    @Permissions(Permission.HostAccess)
    void revokeAllHgRoles(@ApiParam(value = "物理分组id", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "删除主机所有快捷方式", notes = "v6.3后废弃")
    @RequestMapping(value = "/host/{id}/shortcuts", method = RequestMethod.DELETE)
    DeleteEntitiesResult deleteHostShortcuts(@ApiParam(value = "主机id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "获取主机所有快捷方式", notes = "v6.3后废弃")
    @RequestMapping(value = "/host/{id}/shortcuts", method = RequestMethod.GET)
    GetShortcutsResult getHostShortcuts(@ApiParam(value = "主机id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "获取快捷方式", notes = "v6.3后废弃")
    @RequestMapping(value = "/shortcut/{id}", method = RequestMethod.GET)
    ShortcutVo getHostShortcut(@ApiParam(value = "快捷方式id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @Deprecated
    @ApiOperation(value = "删除快捷方式", notes = "v6.3后废弃")
    @RequestMapping(value = "/shortcut/{id}", method = RequestMethod.DELETE)
    void deleteHostShortcut(@ApiParam(value = "快捷方式id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @ApiOperation(value = "获取用户在团队中的可见主机列表", notes = "如果传递cloudId或groupId，则会按照云账户或物理分组再过滤一次。（groupId优先级比cloudId高）")
    @RequestMapping(value = "/host/byUser/{userId}", method = RequestMethod.GET)
    public CloudHostResult getHostByUser(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                         @ApiParam(value = "用户ID", required = true) @PathVariable("userId") Long userId,
                                         @ApiParam(value = "云账户ID", required = false) @RequestParam(value = "cloudId", required = false) Long cloudId,
                                         @ApiParam(value = "物理分组ID", required = false) @RequestParam(value = "groupId", required = false) Long groupId
    ) throws OpenapiException;

    @ApiOperation(value = "查询团队主机成品评估明细")
    @RequestMapping(value = "/host/cost/byTeam/{teamId}", method = RequestMethod.GET)
    public HostCostResult getHostCostByTeam(@ApiParam(value = "所属团队ID", required = true) @PathVariable("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "查询云账户主机成品评估明细")
    @RequestMapping(value = "/host/cost/byCloud/{cloudId}", method = RequestMethod.GET)
    public HostCostResult getHostCostByCloud(@ApiParam(value = "所属云账户ID", required = true) @PathVariable("cloudId") Long cloudId) throws OpenapiException;

    @ApiOperation(value = "查询团队主机成品评估明细（csv文件下载）", response = Void.class, notes = "直接返回流式数据。")
    @RequestMapping(value = "/host/costCsv/byTeam/{teamId}", method = RequestMethod.GET)
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "流式数据")
    })
    public void getHostCostCsvByTeam(
            @ApiParam(value = "所属团队ID", required = true) @PathVariable("teamId") Long teamId,
            @ApiParam(value = "表格语言，目前支持：zh,en") @RequestParam(value = "lang", defaultValue = "zh") String lang) throws OpenapiException, IOException;

    @ApiOperation(value = "查询云账户主机成品评估明细（csv文件下载）", response = Void.class, notes = "直接返回流式数据。")
    @RequestMapping(value = "/host/costCsv/byCloud/{cloudId}", method = RequestMethod.GET)
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "流式数据")
    })
    public void getHostCostCsvByCloud(
            @ApiParam(value = "所属云账户ID", required = true) @PathVariable("cloudId") Long cloudId,
            @ApiParam(value = "表格语言，目前支持：zh,en") @RequestParam(value = "lang", defaultValue = "zh") String lang) throws OpenapiException, IOException;

    @ApiOperation(value = "查询团队主机成品评估明细（csv文件下载URL）")
    @RequestMapping(value = "/host/costCsvUrl/byTeam/{teamId}", method = RequestMethod.GET)
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "流式数据")
    })
    public OAuthUrlResult getHostCostCsvUrlByTeam(
            @ApiParam(value = "所属团队ID", required = true) @PathVariable("teamId") Long teamId,
            @ApiParam(value = "表格语言，目前支持：zh,en") @RequestParam(value = "lang", defaultValue = "zh") String lang,
            @ApiParam(value = "是否一次有效", required = true, defaultValue = "true") @RequestParam(value = "oneoff", defaultValue = "true") boolean oneoff,
            @ApiParam(value = "链接超时时间，单位：秒，取值范围[5min,24hr]", required = true) @RequestParam(value = "expireSeconds", defaultValue = "600") long expireSeconds) throws OpenapiException, IOException;

    @ApiOperation(value = "查询云账户主机成品评估明细（csv文件下载URL）")
    @RequestMapping(value = "/host/costCsvUrl/byCloud/{cloudId}", method = RequestMethod.GET)
    @ApiResponses(value = {
            @ApiResponse(code = 200, message = "流式数据")
    })
    public OAuthUrlResult getHostCostCsvUrlByCloud(@ApiParam(value = "所属云账户ID", required = true) @PathVariable("cloudId") Long cloudId,
                                                   @ApiParam(value = "表格语言，目前支持：zh,en") @RequestParam(value = "lang", defaultValue = "zh") String lang,
                                                   @ApiParam(value = "是否一次有效", required = true, defaultValue = "true") @RequestParam(value = "oneoff", defaultValue = "true") boolean oneoff,
                                                   @ApiParam(value = "链接超时时间，单位：秒，取值范围[5min,24hr]", required = true) @RequestParam(value = "expireSeconds", defaultValue = "600") long expireSeconds) throws OpenapiException, IOException;

    //主机成本优化建议：目前实现，仅获取升配、降配的建议，未考虑CDN等其他资源

    @ApiOperation(value = "获取云账户主机成本优化建议")
    @RequestMapping(value = "/host/advice/byCloud/{cloudId}", method = RequestMethod.GET)
    public OptimizedAdviceResult getHostOptimizedAdviceByCloud(@ApiParam(value = "所属云账户ID", required = true) @PathVariable("cloudId") Long cloudId) throws OpenapiException;

    @ApiOperation(value = "获取团队主机成本优化建议")
    @RequestMapping(value = "/host/advice/byTeam/{teamId}", method = RequestMethod.GET)
    public OptimizedAdviceResult getHostOptimizedAdviceByTeam(@ApiParam(value = "所属团队ID", required = true) @PathVariable("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取特定主机成本优化建议")
    @RequestMapping(value = "/host/advice/{hostId}", method = RequestMethod.GET)
    public OptimizedAdviceResult getHostOptimizedAdvice(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "启动虚拟机")
    @RequestMapping(value = "/host/{hostId}/start", method = RequestMethod.POST)
    void startHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "关闭虚拟机")
    @RequestMapping(value = "/host/{hostId}/stop", method = RequestMethod.POST)
    void stopHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "重启虚拟机")
    @RequestMapping(value = "/host/{hostId}/restart", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    void restartHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                     @ApiParam(value = "是否强制重启", required = false) @RequestParam(value = "force", required = false, defaultValue = "false") Boolean force) throws OpenapiException;

    @ApiOperation(value = "给指定用户生成指定主机的访问串", notes = "基于已有的登陆凭证")
    @RequestMapping(value = "/hostAccessAuth/{hostId}/{userId}/{credentialId}", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    AccessAuthResult createHostAccessAuth(
            @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
            @ApiParam(value = "用户ID", required = true) @PathVariable("userId") Long userId,
            @ApiParam(value = "登陆凭证ID", required = true) @PathVariable("credentialId") Long credentialId) throws OpenapiException;

    @ApiOperation(value = "获取主机一个用户的访问串信息", notes = "")
    @RequestMapping(value = "/hostAccessAuth/byHost/{hostId}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    HostAccessAuthResult getAccessAuthByHost(
            @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
            @ApiParam(value = "用户ID", required = true) @RequestParam("userId") Long userId) throws OpenapiException;

    @ApiOperation(value = "获取访问串信息", notes = "")
    @RequestMapping(value = "/hostAccessAuth/{id}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    AccessAuthResult getAccessAuth(@ApiParam(value = "访问串ID", required = true) @PathVariable("id") Long accessAuthId) throws OpenapiException;

    @ApiOperation(value = "删除访问串", response = BooleanResult.class, notes = "")
    @RequestMapping(value = "/hostAccessAuth/{id}", method = RequestMethod.DELETE)
    @Permissions(Permission.HostAccess)
    void deleteAccessAuthById(@ApiParam(value = "访问串ID", required = true) @PathVariable("id") Long accessAuthId) throws OpenapiException;

    @ApiOperation(value = "删除主机一个用户的访问串", response = BooleanResult.class, notes = "")
    @RequestMapping(value = "/hostAccessAuth/byHost/{hostId}", method = RequestMethod.DELETE)
    @Permissions(Permission.HostAccess)
    void deleteAccessAuthByHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                @ApiParam(value = "用户ID", required = true) @RequestParam("userId") Long userId) throws OpenapiException;


    @ApiOperation(value = "获取团队中所有公网主机列表", response = PublicIpHostsResult.class)
    @RequestMapping(value = "/host/publicHostByTeam/{teamId}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    PublicIpHostsResult findPublicHostsByTeam(
            @ApiParam(value = "团队id", required = true) @PathVariable(value = "teamId") Long teamId,
            @ApiParam(value = "创建时间起[yyyy-MM-dd'T'HH:mm:ss'Z]") @RequestParam(value = "createTimeFrom", required = false) String createTimeFrom,
            @ApiParam(value = "创建时间止[yyyy-MM-dd'T'HH:mm:ss'Z]") @RequestParam(value = "createTimeTo", required = false) String createTimeTo) throws OpenapiException;

    @ApiOperation(value = "添加主机的单用户授权", response = BooleanResult.class, notes = "")
    @RequestMapping(value = "/hostUserAuth/{hostId}", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    void hostUserAuth(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                      @ApiParam(value = "用户ID", required = true) @RequestParam("userId") Long userId) throws OpenapiException;

    @ApiOperation(value = "添加主机的角色授权", response = BooleanResult.class, notes = "")
    @RequestMapping(value = "/hostRoleAuth/{hostId}", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    void hostRoleAuth(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                      @ApiParam(value = "角色ID", required = true) @RequestParam("roleId") Long roleId) throws OpenapiException;


    @ApiOperation(value = "添加主机的部门授权", response = BooleanResult.class, notes = "")
    @RequestMapping(value = "/hostDepartmentAuth/{hostId}", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    void hostDepartmentAuth(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                            @ApiParam(value = "部门ID", required = true) @RequestParam("departmentId") Long departmentId) throws OpenapiException;

    @ApiOperation(value = "查询团队内安装了Agent的主机")
    @RequestMapping(value = "/host/findHostsWithAgent", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findHostsWithAgent(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId) throws OpenapiException;


    @ApiOperation(value = "查询团队内安装了Proxy的主机")
    @RequestMapping(value = "/host/findHostsWithProxy", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostResult findHostsWithProxy(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "根据主机模板导入云主机")
    @RequestMapping(value = "/host/importHostByTemplate", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    JobVo importHostByTemplate(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "用户ID", required = true) @RequestParam("userId") Long userId,
            @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "模板ID", required = true) @RequestParam("templateId") Long templateId,
            @ApiParam(value = "主机信息", required = true) @RequestBody CloudHostTemplateVo hostInfo) throws OpenapiException;

    @ApiOperation(value = "查询主机模板详情")
    @RequestMapping(value = "/host/templateDetail", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostTemplateInfoResult findTemplateDetail(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
            @ApiParam(value = "模板ID", required = true) @RequestParam("templateId") Long templateId) throws OpenapiException;

    @ApiOperation(value = "查询主机模板ID列表")
    @RequestMapping(value = "/host/templateIdList", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    HostTemplateResult findTemplateIdList(
            @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId) throws OpenapiException;
}
