package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.enums.zstack.*;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.Permissions;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import com.cloudbility.openapi.vo.monitor.AggregationData;
import com.cloudbility.openapi.vo.monitor.MetricSingletonMap;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.zstack.sdk.InstanceOfferingInventory;
import org.zstack.sdk.VmInstanceInventory;

import java.util.List;
import java.util.Set;

@Api(tags = "29 image host api", description = "镜像类主机操作API")
public interface ImageHostApi {
    @ApiOperation(value = "获取主机状态", response = GetHostsStatusResult.class, notes = "包括API导入和行云管家页面导入的主机ID")
    @RequestMapping(value = "/imageHost/findStatus", method = RequestMethod.GET)
    GetHostsStatusResult getHostStatus(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                                       @ApiParam(value = "主机ID列表", required = false) @RequestParam(value = "hostIds", required = false) String hostIds) throws OpenapiException;

    @ApiOperation(value = "获取主机详情", response = CloudHostVo.class)
    @RequestMapping(value = "/imageHost/{id}", method = RequestMethod.GET)
    @Permissions(Permission.HostAccess)
    CloudHostVo getHost(@ApiParam(value = "主机id", required = true) @PathVariable("id") long id) throws OpenapiException;

    @ApiOperation(value = "创建虚拟机")
    @RequestMapping(value = "/imageHost/create", method = RequestMethod.POST)
    CloudHostResult createHost(@ApiParam(value = "保证请求幂等性，最大值不超过64个ASCII字符", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                               @ApiParam(value = "团队ID", required = false) @RequestParam(value = "teamId", required = false) Long teamId,
                               @ApiParam(value = "云账户ID", required = true) @RequestParam(value = "cloudId", required = true) Long cloudId,
                               @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", required = true, defaultValue = "Portal") Scope scope,
                               @ApiParam(value = "项目ID", required = false) @RequestParam(value = "projectId", required = false) String projectId,
                               @ApiParam(value = "区域ID", required = true) @RequestParam(value = "regionId", required = true) String regionId,
                               @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
                               @ApiParam(value = "镜像ID", required = true) @RequestParam(value = "imageId", required = true) String imageId,
                               @ApiParam(value = "安全组ID", required = false) @RequestParam(value = "securityGroupId", required = false) String securityGroupId,
                               @ApiParam(value = "实例规格", required = true) @RequestParam(value = "instanceType", required = true) String instanceType,
                               @ApiParam(value = "VPC ID", required = true) @RequestParam(value = "vpcId", required = true) String vpcId,
                               @ApiParam(value = "子网ID", required = false) @RequestParam(value = "subnetId", required = false) String subnetId,
                               @ApiParam(value = "实例名称", required = false) @RequestParam(value = "instanceName", required = false) String instanceName,
                               @ApiParam(value = "实例描述", required = false) @RequestParam(value = "description", required = false) String description,
                               @ApiParam(value = "实例密码", required = false) @RequestParam(value = "password", required = false) String password,
                               @ApiParam(value = "购买数量", required = false) @RequestParam(value = "instanceCount", defaultValue = "1") Integer instanceCount,
                               @ApiParam(value = "实例收费类型", required = false) @RequestParam(value = "instanceChargeType", defaultValue = "PostPaid") CloudInstanceChargeType instanceChargeType,
                               @ApiParam(value = "网络类型", required = false) @RequestParam(value = "internetChargeType", defaultValue = "PayByBandwidth") CloudInternetChargeType internetChargeType,
                               @ApiParam(value = "最大公网出口带宽", required = false) @RequestParam(value = "internetMaxBandwidthOut", defaultValue = "0") Integer internetMaxBandwidthOut,
                               @ApiParam(value = "购买时长", required = false) @RequestParam(value = "period", defaultValue = "1") Integer period,
                               @ApiParam(value = "时长单位", required = false) @RequestParam(value = "periodUnit", defaultValue = "Month") String periodUnit,
                               @ApiParam(value = "系统盘，格式如{\"category\" : \"SSD\", \"size\" : 10}", required = false) @RequestParam(value = "systemDisk", required = false) String systemDisk,
                               @ApiParam(value = "数据盘，格式如[{\"category\" : \"SSD\", \"size\" : 10, \"deleteWithInstance\" : true}]", required = false) @RequestParam(value = "dataDisks", required = false) String dataDisks) throws OpenapiException;

    @ApiOperation(value = "查询购买虚拟机每个月的预估费用", response = DoubleResponse.class)
    @RequestMapping(value = "/imageHost/inquiryPriceMonthly", method = RequestMethod.POST)
    DoubleResponse inquiryPriceMonthly(@ApiParam(value = "供应商", required = false) @RequestParam(value = "vendor", required = false) String vendor,
                                                       @ApiParam(value = "云账户ID", required = true) @RequestParam(value = "cloudId", required = true) Long cloudId,
                                                       @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", required = true, defaultValue = "Portal") Scope scope,
                                                       @ApiParam(value = "项目ID", required = false) @RequestParam(value = "projectId", required = false) String projectId,
                                                       @ApiParam(value = "区域ID", required = true) @RequestParam(value = "regionId", required = true) String regionId,
                                                       @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
                                                       @ApiParam(value = "镜像ID", required = false) @RequestParam(value = "imageId", required = false) String imageId,
                                                       @ApiParam(value = "安全组ID", required = false) @RequestParam(value = "securityGroupId", required = false) String securityGroupId,
                                                       @ApiParam(value = "实例规格", required = true) @RequestParam(value = "instanceType", required = true) String instanceType,
                                                       @ApiParam(value = "VPC ID", required = false) @RequestParam(value = "vpcId", required = false) String vpcId,
                                                       @ApiParam(value = "子网ID", required = false) @RequestParam(value = "subnetId", required = false) String subnetId,
                                                       @ApiParam(value = "实例名称", required = false) @RequestParam(value = "instanceName", required = false) String instanceName,
                                                       @ApiParam(value = "实例描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                       @ApiParam(value = "实例密码", required = false) @RequestParam(value = "password", required = false) String password,
                                                       @ApiParam(value = "购买数量", required = false) @RequestParam(value = "instanceCount", defaultValue = "1") Integer instanceCount,
                                                       @ApiParam(value = "实例收费类型", required = false) @RequestParam(value = "instanceChargeType", defaultValue = "PostPaid") CloudInstanceChargeType instanceChargeType,
                                                       @ApiParam(value = "网络类型", required = false) @RequestParam(value = "internetChargeType", defaultValue = "PayByBandwidth") CloudInternetChargeType internetChargeType,
                                                       @ApiParam(value = "最大公网出口带宽", required = false) @RequestParam(value = "internetMaxBandwidthOut", defaultValue = "0") Integer internetMaxBandwidthOut,
                                                       @ApiParam(value = "购买时长", required = false) @RequestParam(value = "period", defaultValue = "1") Integer period,
                                                       @ApiParam(value = "时长单位", required = false) @RequestParam(value = "periodUnit", defaultValue = "Month") String periodUnit,
                                                       @ApiParam(value = "系统盘，格式如{\"category\" : \"SSD\", \"size\" : 10}", required = false) @RequestParam(value = "systemDisk", required = false) String systemDisk,
                                                       @ApiParam(value = "数据盘，格式如[{\"category\" : \"SSD\", \"size\" : 10, \"deleteWithInstance\" : true}]", required = false) @RequestParam(value = "dataDisks", required = false) String dataDisks) throws OpenapiException;

    @ApiOperation(value = "创建ZStack虚拟机", response = CloudHostResult.class)
    @RequestMapping(value = "/imageHost/cloud/{cloudId}/zstackCreate", method = RequestMethod.POST)
    @Permissions(Permission.HostAccess)
    CloudHostResult createZStackHost(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                     @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId", required = true) Long teamId,
                                                     @ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                     @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                     @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
                                                     @ApiParam(value = "集群ID", required = false) @RequestParam(value = "clusterId", required = false) String clusterId,
                                                     @ApiParam(value = "物理主机ID", required = false) @RequestParam(value = "hostId", required = false) String hostId,
                                                     @ApiParam(value = "镜像ID", required = true) @RequestParam(value = "imageId") String imageId,
                                                     @ApiParam(value = "实例规格ID", required = true) @RequestParam(value = "instanceOfferingId") String instanceOfferingId,
                                                     @ApiParam(value = "L3网络ID有序列表", required = true) @RequestParam(value = "networkIds") List<String> networkIds,
                                                     @ApiParam(value = "默认的L3网络", required = false) @RequestParam(value = "defaultNetworkId", required = false) String defaultNetworkId,
                                                     @ApiParam(value = "实例名称", required = true) @RequestParam(value = "instanceName") String instanceName,
                                                     @ApiParam(value = "实例描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                     @ApiParam(value = "主机名", required = false) @RequestParam(value = "hostName", required = false) String hostName,
                                                     @ApiParam(value = "根云盘(系统盘)规格Id", required = false) @RequestParam(value = "rootDiskOfferingId", required = false) String rootDiskOfferingId,
                                                     @ApiParam(value = "根云盘大小Gb", required = false) @RequestParam(value = "rootDiskSize", required = false) Long rootDiskSize,
                                                     @ApiParam(value = "主存储UUID", required = false) @RequestParam(value = "primaryStorageIdForRootVolume", required = false) String primaryStorageIdForRootVolume,
                                                     @ApiParam(value = "根磁盘置备方式", required = false) @RequestParam(value = "rootVolumeProvisioningStrategy", required = false) VolumeProvisioningStrategyType rootVolumeProvisioningStrategy,
                                                     @ApiParam(value = "云盘规格uuid列表", required = false) @RequestParam(value = "dataDiskOfferingIds", required = false) List<String> dataDiskOfferingIds,
                                                     @ApiParam(value = "数据盘置备方式", required = false) @RequestParam(value = "dataVolumeProvisioningStrategy", required = false) VolumeProvisioningStrategyType dataVolumeProvisioningStrategy,
                                                     @ApiParam(value = "系统设置,VMCreateSystemTags参数对象(json)", required = false) @RequestParam(value = "systemTags", required = false) String systemTags,
                                                     @ApiParam(value = "云主机创建策略", required = false) @RequestParam(value = "createStrategy", required = false) VMCreateStrategyType createStrategy,
                                                     @ApiParam(value = "购买数量", required = false) @RequestParam(value = "instanceCount", defaultValue = "1") Integer instanceCount) throws OpenapiException;

    @ApiOperation(value = "销毁虚拟机")
    @RequestMapping(value = "/imageHost/{hostId}/destroy", method = RequestMethod.DELETE)
    OpenapiResponse destroyHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "启动虚拟机")
    @RequestMapping(value = "/imageHost/{hostId}/start", method = RequestMethod.POST)
    OpenapiResponse startHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "关闭虚拟机")
    @RequestMapping(value = "/imageHost/{hostId}/stop", method = RequestMethod.POST)
    OpenapiResponse stopHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "重启虚拟机")
    @RequestMapping(value = "/imageHost/{hostId}/restart", method = RequestMethod.POST)
    OpenapiResponse restartHost(@ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                                @ApiParam(value = "是否强制重启", required = false) @RequestParam(value = "force", required = false, defaultValue = "false") Boolean force) throws OpenapiException;

    @ApiOperation(value = "续费虚拟机")
    @RequestMapping(value = "/imageHost/{hostId}/renew", method = RequestMethod.POST)
    OpenapiResponse renewHost(@ApiParam(value = "保证请求幂等性，最大值不超过64个ASCII字符", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                              @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                              @ApiParam(value = "包年包月续费时长", required = true) @RequestParam(value = "period", required = true, defaultValue = "1") Integer period,
                                              @ApiParam(value = "续费时长的时间单位", required = true) @RequestParam(value = "periodUnit", required = true, defaultValue = "Month") RenewPeriodUnit periodUnit) throws OpenapiException;

    @ApiOperation(value = "升级虚拟机的规格以及磁盘大小")
    @RequestMapping(value = "/imageHost/{hostId}/expandHostAndDisk", method = RequestMethod.POST)
    CloudHostVo expandHostAndDisk(@ApiParam(value = "保证请求幂等性，最大值不超过64个ASCII字符", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                  @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                                  @ApiParam(value = "实例规格，如ecs.g6.large", required = false) @RequestParam(value = "instanceType", required = false) String instanceType,
                                                  @ApiParam(value = "磁盘ID，如d-bp67acfmxazb4p", required = false) @RequestParam(value = "diskId", required = false) String diskId,
                                                  @ApiParam(value = "磁盘大小(GB)", required = false) @RequestParam(value = "newSize", required = false) Integer newSize) throws OpenapiException;

    @ApiOperation(value = "升级虚拟机配置")
    @RequestMapping(value = "/imageHost/{hostId}/modifySpec", method = RequestMethod.POST)
    CloudHostVo modifyHostSpec(@ApiParam(value = "保证请求幂等性，最大值不超过64个ASCII字符", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                               @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                               @ApiParam(value = "实例规格，如ecs.g6.large", required = true) @RequestParam(value = "instanceType", required = true) String instanceType) throws OpenapiException;

    @ApiOperation(value = "扩展数据盘大小")
    @RequestMapping(value = "/imageHost/{hostId}/resizeDisk", method = RequestMethod.POST)
    DataDiskVo resizeDisk(@ApiParam(value = "保证请求幂等性，最大值不超过64个ASCII字符", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                          @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                          @ApiParam(value = "磁盘ID，如d-bp67acfmxazb4p", required = true) @RequestParam(value = "diskId", required = true) String diskId,
                                          @ApiParam(value = "磁盘大小(GB)", required = true) @RequestParam(value = "newSize", required = true) Integer newSize) throws OpenapiException;

    @ApiOperation(value = "创建ZStack计算规格", response = ZStackHostSpecVo.class)
    @RequestMapping(value = "/imageHost/cloud/{cloudId}/zstackSpec", method = RequestMethod.POST)
    ZStackHostSpecVo createZStackHostSpec(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                          @ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                          @ApiParam(value = "计算规格名", required = true) @RequestParam(value = "name") String name,
                                                          @ApiParam(value = "cpu核数", required = true) @RequestParam(value = "cpuNum") int cpuNum,
                                                          @ApiParam(value = "内存大小(Mb)", required = true) @RequestParam(value = "memorySizeInMb") long memorySizeInMb,
                                                          @ApiParam(value = "物理机分配策略", required = true) @RequestParam(value = "allocatorStrategy") HostAllocatorStrategyType allocatorStrategy,
                                                          @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                          @ApiParam(value = "运行云主机最大数量", required = false) @RequestParam(value = "maxInstancePerHost", required = false) Integer maxInstancePerHost,
                                                          @ApiParam(value = "磁盘总速率(Mb/s)", required = false) @RequestParam(value = "volumeTotalBandwidthInMb", required = false) Long volumeTotalBandwidthInMb,
                                                          @ApiParam(value = "磁盘读速率(Mb/s)", required = false) @RequestParam(value = "volumeReadBandwidthInMb", required = false) Long volumeReadBandwidthInMb,
                                                          @ApiParam(value = "磁盘写速率(Mb/s)", required = false) @RequestParam(value = "volumeWriteBandwidthInMb", required = false) Long volumeWriteBandwidthInMb,
                                                          @ApiParam(value = "网络带宽上行速率(Kb/s)", required = false) @RequestParam(value = "networkInboundBandwidthInKb", required = false) Long networkInboundBandwidthInKb,
                                                          @ApiParam(value = "网络带宽下行速率(Kb/s)", required = false) @RequestParam(value = "networkOutboundBandwidthInKb", required = false) Long networkOutboundBandwidthInKb,
                                                          @ApiParam(value = "cpu最低策略模式", required = false) @RequestParam(value = "minimumCPUUsageHostAllocatorStrategy", required = false) HostAllocatorStrategyMode minimumCPUUsageHostAllocatorStrategy,
                                                          @ApiParam(value = "内存最低策略模式", required = false) @RequestParam(value = "minimumMemoryUsageHostAllocatorStrategy", required = false) HostAllocatorStrategyMode minimumMemoryUsageHostAllocatorStrategy) throws OpenapiException;

    @ApiOperation(value = "查询ZStack计算规格", response = ZStackHostSpecListResult.class)
    @RequestMapping(value = "/imageHost/cloud/{cloudId}/zstackSpecs", method = RequestMethod.GET)
    ZStackHostSpecListResult listZStackHostSpecs(@ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                                 @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                                 @ApiParam(value = "计算规格Id列表", required = false) @RequestParam(value = "instanceOfferingIds", required = false) Set<String> instanceOfferingIds,
                                                                 @ApiParam(value = "计算规格名", required = false) @RequestParam(value = "name", required = false) String name,
                                                                 @ApiParam(value = "cpu核数(大于等于)", required = false) @RequestParam(value = "cpuNum", required = false) Integer cpuNum,
                                                                 @ApiParam(value = "内存大小(Mb)(大于等于)", required = false) @RequestParam(value = "memorySizeInMb", required = false) Long memorySizeInMb,
                                                                 @ApiParam(value = "物理机分配策略", required = false) @RequestParam(value = "allocatorStrategy", required = false) HostAllocatorStrategyType allocatorStrategy,
                                                                 @ApiParam(value = "使用状态", required = false) @RequestParam(value = "usingState", required = false) UsingStateType state) throws OpenapiException;

    @ApiOperation(value = "查询ZStack物理机", response = ZStackPhysicalHostListResult.class)
    @RequestMapping(value = "/imageHost/cloud/{cloudId}/zstackPhysicalHosts", method = RequestMethod.GET)
    ZStackPhysicalHostListResult listZStackPhysicalHosts(@ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId", required = true) Long cloudId,
                                                                         @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope) throws OpenapiException;

    @ApiOperation(value = "查询ZStack服务资源信息", response = ZStackPhysicalResourceSummary.class)
    @RequestMapping(value = "/imageHost/cloud/{cloudId}/zstackPhysicalResourceSummary", method = RequestMethod.GET)
    ZStackPhysicalResourceSummary getZStackPhysicalResourceSummary(@ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId", required = true) Long cloudId,
                                                                                   @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope) throws OpenapiException;

    @ApiOperation(value = "修改ZStack默认网卡")
    @RequestMapping(value = "/imageHost/{hostId}/zstackDefaultNetwork", method = RequestMethod.PUT)
    OpenapiResponse modifyZStackDefaultNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                               @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                                               @ApiParam(value = "默认L3网络Id", required = true) @RequestParam(value = "defaultNetworkId") String defaultNetworkId,
                                                               @ApiParam(value = "是否立即重启", required = false) @RequestParam(value = "reboot", required = false) Boolean reboot) throws OpenapiException;

    @ApiOperation(value = "加载ZStackL3网络到云主机", response = VmInstanceInventory.class)
    @RequestMapping(value = "/imageHost/{hostId}/zstackAttachL3Network/{l3NetworkId}", method = RequestMethod.PUT)
    VmInstanceInventory attachZStackL3NetworkToHost(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                    @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
                                                                    @ApiParam(value = "L3网络Id", required = true) @RequestParam("l3NetworkId") String l3NetworkId,
                                                                    @ApiParam(value = "静态ip地址", required = false) @RequestParam(value = "staticIp", required = false) String staticIp) throws OpenapiException;

    @ApiOperation(value = "更改zstack实例规格状态", response = InstanceOfferingInventory.class)
    @RequestMapping(value = "/imageHost/zstackInstanceOffering/{instanceOfferingId}", method = RequestMethod.PUT)
    InstanceOfferingInventory changeZStackInstanceOfferingState(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                                @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                                                @ApiParam(value = "实例规格Id", required = true) @PathVariable("instanceOfferingId") String instanceOfferingId,
                                                                                @ApiParam(value = "状态(启用/停用)", required = true) @RequestParam("stateEvent") UsingStateEventType stateEvent) throws OpenapiException;

    @ApiOperation(value = "删除zstack实例规格", response = InstanceOfferingInventory.class)
    @RequestMapping(value = "/imageHost/zstackInstanceOffering/{instanceOfferingId}", method = RequestMethod.DELETE)
    OpenapiResponse deleteZStackInstanceOffering(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                 @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                 @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                                 @ApiParam(value = "实例规格Id", required = true) @PathVariable("instanceOfferingId") String instanceOfferingId,
                                                                 @ApiParam(value = "删除方式", required = false) @RequestParam(value = "deleteMode", required = false) DeleteModeType deleteMode) throws OpenapiException;

    @ApiOperation(value = "获取zstack实例网络信息", response = ZStackVmNicInfoResult.class)
    @RequestMapping(value = "/imageHost/zstackVmNic/{instanceId}", method = RequestMethod.GET)
    ZStackVmNicInfoResult getZStackVmNicInfo(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                             @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                             @ApiParam(value = "实例Id", required = true) @PathVariable("instanceId") String instanceId) throws OpenapiException;

    @ApiOperation(value = "获取主机特定监控项的最后一个数据点", response = AggregationData.class, notes = "如果监控项包括不同的dimension，返回的数据会包含所有dimension")
    @RequestMapping(value = "/imageHost/{hostId}/monitor", method = RequestMethod.GET)
    MetricSingletonMap getHostLastMetricData(
            @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId,
            @ApiParam(value = "主机监控项(MonitorMetric)，用逗号连接", required = true) @RequestParam("metrics") String metrics) throws OpenapiException;
}
