package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.KeyPairListResult;
import com.cloudbility.openapi.vo.KeyPairDetailVo;
import com.cloudbility.openapi.vo.KeyPairVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

@Api(tags = "Key pair api", description = "密钥对API")
public interface KeypairApi extends Openapi {

    @RequestMapping(value = "/keyPairs", method = RequestMethod.GET)
    @ApiOperation(value = "获取所有密钥对")
    KeyPairListResult listKeyPairs(@ApiParam(required = true, value = "团队ID") @RequestParam("teamId") Long teamId) throws OpenapiException;

    @RequestMapping(value = "/keyPair/{id}", method = RequestMethod.GET)
    @ApiOperation(value = "获取密钥对详情（授权信息）")
    KeyPairDetailVo getKeyPairInfo(
            @ApiParam(required = true, value = "密钥对ID") @PathVariable("id") Long id
    ) throws OpenapiException;

    @RequestMapping(value = "/keyPair", method = RequestMethod.POST)
    @ApiOperation(value = "创建密钥对", notes = "")
    KeyPairVo createKeyPair(
            @ApiParam(required = true, value = "团队ID") @RequestParam("teamId") Long teamId,
            @ApiParam(required = true, value = "密钥对名称（团队唯一）") @RequestParam("name") String name,
            @ApiParam(value = "密钥对描述") @RequestParam("description") String description,
            /*@ApiParam(value = "密钥对类型:ssh-rsa|ssh-ds") @RequestParam(value = "type", defaultValue = "ssh-rsa") String type,*/
            @ApiParam(value = "密钥文件（用字符串表示），如果未设置后台会自动生成") @RequestParam(required = false, value = "key") String key
            /*@ApiParam(value = "密钥文件的加密密码，可选") @RequestParam(required = false, value = "password") String password*/
    ) throws OpenapiException;

    @RequestMapping(value = "/keyPair/{id}", method = RequestMethod.PUT)
    @ApiOperation(value = "修改密钥对名称和描述", notes = "v4.18的实现：如果密钥对添加到登录凭证的主机上，删除密钥对后不会删除登录凭证中的密钥")
    void updateKeyPair(
            @ApiParam(required = true, value = "密钥对ID") @PathVariable("id") Long id,
            @ApiParam(required = true, value = "密钥对名称（团队唯一）") @RequestParam("name") String name,
            @ApiParam(value = "密钥对描述") @RequestParam("description") String description
    ) throws OpenapiException;

    @RequestMapping(value = "/keyPair/{id}/members", method = RequestMethod.PUT)
    @ApiOperation(value = "授权密钥对给团队成员", notes = "如果没有传递members，会取消所有成员授权")
    void updateKeyPairMembers(
            @ApiParam(required = true, value = "密钥对ID") @PathVariable("id") Long id,
            @ApiParam(value = "授权团队成员") @RequestParam(value = "members", required = false) String members
    ) throws OpenapiException;


    @RequestMapping(value = "/keyPair/{id}/roles", method = RequestMethod.PUT)
    @ApiOperation(value = "授权密钥对给角色", notes = "如果没有传递roles，会取消所有角色授权")
    void updateKeyPairRoles(
            @ApiParam(required = true, value = "密钥对ID") @PathVariable("id") Long id,
            @ApiParam(value = "授权角色") @RequestParam(value = "roles", required = false) String roles) throws OpenapiException;

    @RequestMapping(value = "/keyPair/{id}/departments", method = RequestMethod.PUT)
    @ApiOperation(value = "授权密钥对给组织单元", notes = "如果没有传递departments，会取消所有组织单元授权")
    void updateKeyPairDepartments(
            @ApiParam(required = true, value = "密钥对ID") @PathVariable("id") Long id,
            @ApiParam(value = "授权组织单元") @RequestParam(value = "departments", required = false) String departments) throws OpenapiException;

    @RequestMapping(value = "/keyPair/{id}", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除密钥对", notes = "v4.18的实现：如果密钥对添加到登录凭证的主机上，删除密钥对后不会删除登录凭证中的密钥")
    void deleteKeyPair(@ApiParam(required = true, value = "密钥对ID") @PathVariable("id") Long id) throws OpenapiException;
}
