package com.cloudbility.openapi.v2;


import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.AddLogResult;
import com.cloudbility.openapi.response.DbSqlLogResult;
import com.cloudbility.openapi.response.HostLogResult;
import com.cloudbility.openapi.response.LogResult;
import com.cloudbility.openapi.response.sessionLog.SessionCommandLogResult;
import com.cloudbility.openapi.vo.AddLogRequest;
import com.cloudbility.openapi.vo.HostActionType;
import com.cloudbility.openapi.vo.LogStatus;
import com.cloudbility.openapi.vo.Order;
import com.cloudbility.openapi.vo.log.CmdType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "5 log api", description = "日志API")
public interface LogApi extends Openapi {

    @ApiOperation(value = "获取主机日志")
    @RequestMapping(value = "/log/host", method = RequestMethod.GET)
    HostLogResult getHostLog(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                             @ApiParam(value = "操作类型，未指定时查询所有操作") @RequestParam(value = "actionType", required = false) HostActionType actionType,
                             @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                             @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size,
                             @ApiParam(value = "所属云账户，未指定时查询所有云账户") @RequestParam(value = "cloudId", required = false) Long cloudId,
                             @ApiParam(value = "状态：成功或失败") @RequestParam(value = "status", required = false) LogStatus status,
                             @ApiParam(value = "操作者ID") @RequestParam(value = "actorId", required = false) Long actorId,
                             @ApiParam(value = "主机ID") @RequestParam(value = "hostId", required = false) Long hostId,
                             @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
                             @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr,
                             @ApiParam(value = "过滤正在进行的会话,设置为true时必须指定hostId") @RequestParam(value = "filterRunningSession", required = false, defaultValue = "false") boolean filterRunningSession,
                             @ApiParam(value = "排序规则，按创建时间排序", required = true) @RequestParam(value = "order", defaultValue = "DESC") Order order) throws OpenapiException;


    @ApiOperation(value = "获取数据库SQL日志")
    @RequestMapping(value = "/log/db/sql", method = RequestMethod.GET)
    DbSqlLogResult getDbSqlLog(
            @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size,
            @ApiParam(value = "操作者ID") @RequestParam(value = "actorId", required = false) Long actorId,
            @ApiParam(value = "数据库ID") @RequestParam(value = "dbId", required = false) Long dbId,
            @ApiParam(value = "数据库会话ID") @RequestParam(value = "sessionId", required = false) Long sessionId,
            @ApiParam(value = "过滤成功或失败记录") @RequestParam(value = "success", required = false) Boolean success,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr,
            @ApiParam(value = "排序规则，按创建时间排序", required = true) @RequestParam(value = "order", defaultValue = "DESC") Order order) throws OpenapiException;


    @ApiOperation(value = "插入一条日志记录", notes = "AK需要有LogWrite权限才可以调用当前端口。日志的cluster必须以EXT_开头")
    @RequestMapping(value = "/log", method = RequestMethod.POST)
    AddLogResult addLog(@ApiParam(value = "日志信息", required = true) @RequestBody AddLogRequest request) throws OpenapiException;

    @ApiOperation(value = "查询日志", notes = "创建时间（如果有传入）查询范围是[startTime,endTime)。teamId和cluster两个参数是必填参数")
    @RequestMapping(value = "/log", method = RequestMethod.GET)
    LogResult getLog(
            @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "日志分类", required = true) @RequestParam("cluster") String cluster,
            @ApiParam(value = "操作") @RequestParam(value = "action", required = false) String action,
            @ApiParam(value = "操作者ID") @RequestParam(value = "actorId", required = false) Long actorId,
            @ApiParam(value = "状态") @RequestParam(value = "status", required = false) String status,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"") @RequestParam(value = "endTime", required = false) String endTimeStr,
            @ApiParam(value = "排序规则，按创建时间排序", required = true) @RequestParam(value = "order", defaultValue = "DESC") Order order,
            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数，不超过1000") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "查询主机会话指令日志", notes = "创建时间（如果有传入）查询范围是[startTime,endTime)")
    @RequestMapping(value = "/sessionCommandLog", method = RequestMethod.GET)
    SessionCommandLogResult getSessionCommandLog(
            @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "操作者ID") @RequestParam(value = "actorId", required = false) Long actorId,
            @ApiParam(value = "会话ID") @RequestParam(value = "sessionId", required = false) Long sessionId,
            @ApiParam(value = "主机ID，多个使用逗号分隔") @RequestParam(value = "hostIds", required = false) String hostIdsStr,
            @ApiParam(value = "指令类型") @RequestParam(value = "cmdType", required = false) CmdType cmdType,
            @ApiParam(value = "开始时间，UTC时间，格式\"yyyy-MM-dd\"") @RequestParam(value = "startTime", required = false) String startTimeStr,
            @ApiParam(value = "结束时间，UTC时间，格式\"yyyy-MM-dd\"") @RequestParam(value = "endTime", required = false) String endTimeStr,
            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数，不超过1000") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;
}
