package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.enums.zstack.*;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.zstack.sdk.*;

import java.util.Set;

@Api(tags = "26 network api", description = "网络API")
public interface NetworkApi {
    @ApiOperation(value = "创建安全组", response = SecurityGroupVo.class)
    @RequestMapping(value = "/network/createSecurityGroup", method = RequestMethod.POST)
    ResponseEntity<SecurityGroupVo> createSecurityGroup(
            @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
            @ApiParam(value = "VPC ID", required = false) @RequestParam(value = "vpcId", required = false) String vpcId,
            @ApiParam(value = "安全组名称", required = true) @RequestParam("securityGroupName") String securityGroupName,
            @ApiParam(value = "安全组描述", required = false) @RequestParam(value = "description", required = false) String description,
            @ApiParam(value = "安全组规则", required = false) @RequestParam(value = "securityGroupRules", required = false) String securityGroupRules) throws OpenapiException;

    @ApiOperation(value = "列出云账户的某个VPC下的所有安全组", response = SecurityGroupResult.class)
    @RequestMapping(value = "/network/securityGroups", method = RequestMethod.GET)
    ResponseEntity<SecurityGroupResult> listSecurityGroups(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                           @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
                                                           @ApiParam(value = "VPC ID，腾讯云没有该参数", required = false) @RequestParam(value = "vpcId", required = false) String vpcId) throws OpenapiException;

    @ApiOperation(value = "列出安全组的所有规则", response = SecurityGroupRuleResult.class)
    @RequestMapping(value = "/network/securityGroupRules", method = RequestMethod.GET)
    ResponseEntity<SecurityGroupRuleResult> listSecurityGroupRules(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                   @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
                                                                   @ApiParam(value = "安全组ID", required = true) @RequestParam(value = "securityGroupId", required = true) String securityGroupId) throws OpenapiException;

    @ApiOperation(value = "把主机加入到安全组中")
    @RequestMapping(value = "/network/joinHostToSecurityGroup", method = RequestMethod.POST)
    ResponseEntity<OpenapiResponse> joinHostToSecurityGroup(
            @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "主机ID", required = true) @RequestParam("hostId") Long hostId,
            @ApiParam(value = "安全组ID", required = true) @RequestParam("securityGroupId") String securityGroupId) throws OpenapiException;

    @ApiOperation(value = "从安全组中移除主机")
    @RequestMapping(value = "/network/removeHostFromSecurityGroup", method = RequestMethod.POST)
    ResponseEntity<OpenapiResponse> removeHostFromSecurityGroup(
            @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "主机ID", required = true) @RequestParam("hostId") Long hostId,
            @ApiParam(value = "安全组ID", required = true) @RequestParam("securityGroupId") String securityGroupId) throws OpenapiException;

    @ApiOperation(value = "创建子网", response = SubnetVo.class)
    @RequestMapping(value = "/network/createSubnet", method = RequestMethod.POST)
    ResponseEntity<SubnetVo> createSubnet(
            @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
            @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
            @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
            @ApiParam(value = "实例规格，如果zoneId参数为空，就找一个可以售卖该实例规格的可用区", required = false) @RequestParam(value = "instanceType", required = false) String instanceType,
            @ApiParam(value = "VPC ID", required = true) @RequestParam(value = "vpcId") String vpcId,
            @ApiParam(value = "CIDR", required = true) @RequestParam(value = "cidrBlock") String cidrBlock,
            @ApiParam(value = "子网名称", required = true) @RequestParam("subnetName") String subnetName,
            @ApiParam(value = "子网描述", required = false) @RequestParam(value = "description", required = false) String description) throws OpenapiException;

    @ApiOperation(value = "获取云账户的所有VPC", response = ListVpcsResult.class)
    @RequestMapping(value = "/network/vpcs", method = RequestMethod.GET)
    ResponseEntity<ListVpcsResult> getVpcsByCloud(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                  @ApiParam(value = "区域ID", required = true) @RequestParam("regionId") String regionId,
                                                  @ApiParam(value = "可用区ID", required = false) @RequestParam(value = "zoneId", required = false) String zoneId,
                                                  @ApiParam(value = "项目ID", required = false) @RequestParam(value = "projectId", required = false) String projectId) throws OpenapiException;

    @ApiOperation(value = "创建zstackVxlan池,并加入集群", response = CreateZStackL2VxLanPoolResult.class)
    @RequestMapping(value = "/network/zstackL2/vxLanPool", method = RequestMethod.POST)
    ResponseEntity<CreateZStackL2VxLanPoolResult> createZStackVxLanPool(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                        @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                        @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                        @ApiParam(value = "二层网络名称", required = true) @RequestParam("name") String name,
                                                                        @ApiParam(value = "二层网络的详细描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                        @ApiParam(value = "起始Vni", required = false) @RequestParam(value = "startVni", required = false) Integer startVni,
                                                                        @ApiParam(value = "结束Vni", required = false) @RequestParam(value = "endVni", required = false) Integer endVni,
                                                                        @ApiParam(value = "加入到的集群Id", required = false) @RequestParam(value = "clusterId", required = false) String clusterId,
                                                                        @ApiParam(value = "VTEP CIDR", required = false) @RequestParam(value = "cidr", required = false) String cidr) throws OpenapiException;

    @ApiOperation(value = "创建zstack普通二层网络,并加入集群", response = CreateZStackL2NoVLanNetworkResult.class)
    @RequestMapping(value = "/network/zstackL2/noVLan", method = RequestMethod.POST)
    ResponseEntity<CreateZStackL2NoVLanNetworkResult> createZStackL2NoVLanNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                                  @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                  @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                                  @ApiParam(value = "二层网络名称", required = true) @RequestParam("name") String name,
                                                                                  @ApiParam(value = "物理网卡(eth1, eno1)", required = true) @RequestParam("physicalInterface") String physicalInterface,
                                                                                  @ApiParam(value = "二层网络的详细描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                                  @ApiParam(value = "加入到的集群Id", required = false) @RequestParam(value = "clusterId", required = false) String clusterId) throws OpenapiException;

    @ApiOperation(value = "创建zstackVLan二层网络,并加入集群", response = CreateZStackL2VLanNetworkResult.class)
    @RequestMapping(value = "/network/zstackL2/vLan", method = RequestMethod.POST)
    ResponseEntity<CreateZStackL2VLanNetworkResult> createZStackL2VLanNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                              @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                              @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                              @ApiParam(value = "二层网络名称", required = true) @RequestParam("name") String name,
                                                                              @ApiParam(value = "VLanId", required = true) @RequestParam("vLanId") Integer vLanId,
                                                                              @ApiParam(value = "物理网卡(eth1, eno1)", required = true) @RequestParam("physicalInterface") String physicalInterface,
                                                                              @ApiParam(value = "二层网络的详细描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                              @ApiParam(value = "加入到的集群Id", required = false) @RequestParam(value = "clusterId", required = false) String clusterId) throws OpenapiException;

    @ApiOperation(value = "创建zstackVxLan二层网络", response = CreateZStackL2VxLanNetworkResult.class)
    @RequestMapping(value = "/network/zstackL2/vxLan", method = RequestMethod.POST)
    ResponseEntity<CreateZStackL2VxLanNetworkResult> createZStackL2VxLanNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                                @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                                @ApiParam(value = "二层网络名称", required = true) @RequestParam("name") String name,
                                                                                @ApiParam(value = "VxLan池Id", required = true) @RequestParam("vxLanPoolId") String vxLanPoolId,
                                                                                @ApiParam(value = "二层网络的详细描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                                @ApiParam(value = "vni", required = false) @RequestParam(value = "vni", required = false) Integer vni) throws OpenapiException;

    @ApiOperation(value = "将zstack二层网络加入集群", response = L2NetworkInventory.class)
    @RequestMapping(value = "/network/zstackL2/{networkId}/attachTo/{clusterId}", method = RequestMethod.POST)
    ResponseEntity<L2NetworkInventory> attachZStackL2NetworkToCluster(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                      @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                      @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                      @ApiParam(value = "二层网络Id", required = true) @PathVariable("networkId") String networkId,
                                                                      @ApiParam(value = "加入到的集群Id", required = true) @PathVariable(value = "clusterId") String clusterId,
                                                                      @ApiParam(value = "VTEP CIDR", required = false) @RequestParam(value = "cidr", required = false) String cidr) throws OpenapiException;

    @ApiOperation(value = "查询zstackVxlan池", response = ListZStackL2VxLanPoolsResult.class)
    @RequestMapping(value = "/network/zstackL2/vxLanPools", method = RequestMethod.GET)
    ResponseEntity<ListZStackL2VxLanPoolsResult> listZStackL2VxLanPools(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                        @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                        @ApiParam(value = "二层网络Id列表", required = false) @RequestParam(value = "networkIds", required = false) Set<String> networkIds) throws OpenapiException;

    @ApiOperation(value = "查询zstackVxlan", response = ListZStackL2VxLanNetworksResult.class)
    @RequestMapping(value = "/network/zstackL2/vxLans", method = RequestMethod.GET)
    ResponseEntity<ListZStackL2VxLanNetworksResult> listZStackL2VxLanNetworks(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                              @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                              @ApiParam(value = "二层网络Id列表", required = false) @RequestParam(value = "networkIds", required = false) Set<String> networkIds) throws OpenapiException;

    @ApiOperation(value = "查询zstackVlan", response = ListZStackL2VLanNetworksResult.class)
    @RequestMapping(value = "/network/zstackL2/vLans", method = RequestMethod.GET)
    ResponseEntity<ListZStackL2VLanNetworksResult> listZStackL2VLanNetworks(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                            @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                            @ApiParam(value = "二层网络Id列表", required = false) @RequestParam(value = "networkIds", required = false) Set<String> networkIds,
                                                                            @ApiParam(value = "物理网卡", required = false) @RequestParam(value = "physicalInterface", required = false) String physicalInterface) throws OpenapiException;

    @ApiOperation(value = "查询已用zstackVlanId列表", response = SetZStackL2VLanIdsResult.class)
    @RequestMapping(value = "/network/zstackL2/existsVLanIds", method = RequestMethod.GET)
    ResponseEntity<SetZStackL2VLanIdsResult> getZStackExistsL2VLanIdSet(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                        @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope) throws OpenapiException;

    @ApiOperation(value = "查询所有zstackL2网络", response = ListZStackL2NetworksResult.class)
    @RequestMapping(value = "/network/zstackL2/networks", method = RequestMethod.GET)
    ResponseEntity<ListZStackL2NetworksResult> listZStackL2Networks(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                    @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                    @ApiParam(value = "二层网络Id列表", required = false) @RequestParam(value = "networkIds", required = false) Set<String> networkIds,
                                                                    @ApiParam(value = "物理网卡", required = false) @RequestParam(value = "physicalInterface", required = false) String physicalInterface,
                                                                    @ApiParam(value = "起始ip", required = false) @RequestParam(value = "startIp", required = false) String startIp) throws OpenapiException;

    @ApiOperation(value = "删除zstack二层网络", response = OpenapiResponse.class)
    @RequestMapping(value = "/network/zstackL2/{networkId}", method = RequestMethod.DELETE)
    ResponseEntity<OpenapiResponse> deleteZStackL2Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                          @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                          @ApiParam(value = "L2网络Id", required = true) @PathVariable("networkId") String networkId,
                                                          @ApiParam(value = "删除方式", required = false) @RequestParam(value = "deleteMode", required = false) DeleteModeType deleteMode) throws OpenapiException;

    @ApiOperation(value = "查询所有zstackL3网络", response = ListZStackL3NetworksResult.class)
    @RequestMapping(value = "/network/zstackL3/networks", method = RequestMethod.GET)
    ResponseEntity<ListZStackL3NetworksResult> listZStackL3Networks(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                    @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                    @ApiParam(value = "三层网络Id列表", required = false) @RequestParam(value = "l3NetworkIds", required = false) Set<String> l3NetworkIds,
                                                                    @ApiParam(value = "三层网络名称", required = false) @RequestParam(value = "l3NetworkName", required = false) String l3NetworkName,
                                                                    @ApiParam(value = "二层网络Id", required = false) @RequestParam(value = "l2NetworkId", required = false) String l2NetworkId,
                                                                    @ApiParam(value = "使用状态", required = false) @RequestParam(value = "usingState", required = false) UsingStateType usingState,
                                                                    @ApiParam(value = "网络类型", required = false) @RequestParam(value = "networkType", required = false) L3NetworkType networkType) throws OpenapiException;

    @ApiOperation(value = "删除多个zstack三层网络", response = OpenapiResponse.class)
    @RequestMapping(value = "/network/zstackL3", method = RequestMethod.DELETE)
    ResponseEntity<OpenapiResponse> deleteZStackL3Networks(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                           @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                           @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                           @ApiParam(value = "L3网络Id", required = true) @RequestParam("networkIds") Set<String> networkIds,
                                                           @ApiParam(value = "删除方式", required = false) @RequestParam(value = "deleteMode", required = false) DeleteModeType deleteMode) throws OpenapiException;

    @ApiOperation(value = "删除zstack虚拟网络", response = OpenapiResponse.class)
    @RequestMapping(value = "/network/zstack/virtualNetwork/{l3NetworkId}", method = RequestMethod.DELETE)
    ResponseEntity<OpenapiResponse> deleteZStackVirtualNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                               @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                               @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                               @ApiParam(value = "L3网络Id", required = true) @PathVariable("l3NetworkId") String l3NetworkId,
                                                               @ApiParam(value = "是否同时删除二层网络", required = false) @RequestParam(value = "safeDeleteL2Network", required = false) Boolean safeDeleteL2Network) throws OpenapiException;

    @ApiOperation(value = "列出zstack网络服务模块", response = ListZStackNetworkServiceProvidersResult.class)
    @RequestMapping(value = "/network/zstackServiceProviders", method = RequestMethod.GET)
    ResponseEntity<ListZStackNetworkServiceProvidersResult> listZStackNetworkServiceProviders(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                              @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope) throws OpenapiException;

    @ApiOperation(value = "列出zstackIP地址范围", response = ListZStackIpRangesResult.class)
    @RequestMapping(value = "/network/zstackIpRanges", method = RequestMethod.GET)
    ResponseEntity<ListZStackIpRangesResult> listZStackIpRanges(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                @ApiParam(value = "ip范围Id列表", required = false) @RequestParam(value = "ipRangeIds", required = false) Set<String> ipRangeIds,
                                                                @ApiParam(value = "所属二层网络Id", required = false) @RequestParam(value = "l2networkId", required = false) String l2networkId,
                                                                @ApiParam(value = "所属三层网络Id", required = false) @RequestParam(value = "l3networkId", required = false) String l3networkId) throws OpenapiException;

    @ApiOperation(value = "列出zstackIP网络地址容量", response = GetIpAddressCapacityResult.class)
    @RequestMapping(value = "/network/zstackIpAddressCapacity", method = RequestMethod.GET)
    ResponseEntity<GetIpAddressCapacityResult> getZStackIpAddressCapacity(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                          @ApiParam(value = "所属三层网络Id", required = false) @RequestParam(value = "l3networkIds", required = false) Set<String> l3networkIds,
                                                                          @ApiParam(value = "ip范围Id列表", required = false) @RequestParam(value = "ipRangeIds", required = false) Set<String> ipRangeIds) throws OpenapiException;

    @ApiOperation(value = "ZStack三层网络添加DNS", response = L3NetworkInventory.class)
    @RequestMapping(value = "/network/zstackDns/{dns}/AddToL3Network/{l3networkId}", method = RequestMethod.POST)
    ResponseEntity<L3NetworkInventory> addZStackDnsToL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                               @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                               @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                               @ApiParam(value = "dns地址", required = true) @PathVariable("dns") String dns,
                                                               @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3networkId") String l3networkId) throws OpenapiException;

    @ApiOperation(value = "ZStack三层网络移除DNS", response = L3NetworkInventory.class)
    @RequestMapping(value = "/network/zstackDns/{dns}/RemoveFromL3Network/{l3networkId}", method = RequestMethod.POST)
    ResponseEntity<L3NetworkInventory> removeZStackDnsFromL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                    @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                    @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                    @ApiParam(value = "dns地址", required = true) @PathVariable("dns") String dns,
                                                                    @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3networkId") String l3networkId) throws OpenapiException;

    @ApiOperation(value = "向ZStack三层网络添加IP地址范围", response = IpRangeInventory.class)
    @RequestMapping(value = "/network/zstackAddIpRangeToL3Network/{l3networkId}", method = RequestMethod.POST)
    ResponseEntity<IpRangeInventory> addZStackIpRangeToL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                 @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                 @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                 @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3networkId") String l3networkId,
                                                                 @ApiParam(value = "ip地址范围名称", required = true) @RequestParam("name") String name,
                                                                 @ApiParam(value = "起始Ip地址", required = true) @RequestParam("startIp") String startIp,
                                                                 @ApiParam(value = "结束Ip地址", required = true) @RequestParam("endIp") String endIp,
                                                                 @ApiParam(value = "子网掩码", required = true) @RequestParam("netmask") String netmask,
                                                                 @ApiParam(value = "网关地址", required = true) @RequestParam("gateway") String gateway,
                                                                 @ApiParam(value = "DHCP服务IP", required = true) @RequestParam(value = "dhcpIp", required = false) String dhcpIp,
                                                                 @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description) throws OpenapiException;

    @ApiOperation(value = "向ZStack三层网络添加IPv6地址范围", response = IpRangeInventory.class)
    @RequestMapping(value = "/network/zstackAddIpv6RangeToL3Network/{l3networkId}", method = RequestMethod.POST)
    ResponseEntity<IpRangeInventory> addZStackIpv6RangeToL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                   @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                   @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                   @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3networkId") String l3networkId,
                                                                   @ApiParam(value = "ip地址范围名称", required = true) @RequestParam("name") String name,
                                                                   @ApiParam(value = "起始Ip地址", required = true) @RequestParam("startIp") String startIp,
                                                                   @ApiParam(value = "结束Ip地址", required = true) @RequestParam("endIp") String endIp,
                                                                   @ApiParam(value = "网关地址", required = true) @RequestParam("gateway") String gateway,
                                                                   @ApiParam(value = "前缀长度", required = true) @RequestParam("prefixLen") Integer prefixLen,
                                                                   @ApiParam(value = "DHCP服务IP", required = true) @RequestParam(value = "dhcpIp", required = false) String dhcpIp,
                                                                   @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description) throws OpenapiException;

    @ApiOperation(value = "向ZStack三层网络通过cidr方式添加IP地址范围", response = IpRangeInventory.class)
    @RequestMapping(value = "/network/zstackAddIpRangeByCidrToL3Network/{l3networkId}", method = RequestMethod.POST)
    ResponseEntity<IpRangeInventory> addZStackIpRangeByCidrForL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                        @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                        @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                        @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3networkId") String l3networkId,
                                                                        @ApiParam(value = "ip地址范围名称", required = true) @RequestParam("name") String name,
                                                                        @ApiParam(value = "网络CIDR", required = true) @RequestParam("cidr") String cidr,
                                                                        @ApiParam(value = "DHCP服务IP", required = true) @RequestParam(value = "dhcpIp", required = false) String dhcpIp,
                                                                        @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description) throws OpenapiException;

    @ApiOperation(value = "向ZStack三层网络通过cidr方式添加IPv6地址范围", response = IpRangeInventory.class)
    @RequestMapping(value = "/network/zstackAddIpv6RangeByCidrToL3Network/{l3networkId}", method = RequestMethod.POST)
    ResponseEntity<IpRangeInventory> addZStackIpv6RangeByCidrForL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                          @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                          @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3networkId") String l3networkId,
                                                                          @ApiParam(value = "ip地址范围名称", required = true) @RequestParam("name") String name,
                                                                          @ApiParam(value = "网络CIDR", required = true) @RequestParam("cidr") String cidr,
                                                                          @ApiParam(value = "IPv6地址分配模式", required = true) @RequestParam("ipv6AddressMode") Ipv6AddressModeType ipv6AddressMode,
                                                                          @ApiParam(value = "DHCP服务IP", required = true) @RequestParam(value = "dhcpIp", required = false) String dhcpIp,
                                                                          @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description) throws OpenapiException;

    @ApiOperation(value = "删除ZStackIP地址范围")
    @RequestMapping(value = "/network/zstackIpRange/{ipRangeId}", method = RequestMethod.DELETE)
    ResponseEntity<OpenapiResponse> deleteZStackIpRange(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                        @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                        @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                        @ApiParam(value = "ip范围Id", required = true) @PathVariable("ipRangeId") String ipRangeId,
                                                        @ApiParam(value = "删除方式", required = false) @RequestParam(value = "deleteMode", required = false) DeleteModeType deleteMode) throws OpenapiException;

    @ApiOperation(value = "创建zstack公有网络", response = L3NetworkInventory.class)
    @RequestMapping(value = "/network/zstackL3/publicNetwork", method = RequestMethod.POST)
    ResponseEntity<L3NetworkInventory> createZStackL3PublicNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                   @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                   @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                   @ApiParam(value = "二层网络Id", required = true) @RequestParam("l2NetworkId") String l2NetworkId,
                                                                   @ApiParam(value = "ip协议号[4,6]", required = true) @RequestParam("ipVersion") IpVersion ipVersion,
                                                                   @ApiParam(value = "三层网络名称", required = true) @RequestParam("name") String name,
                                                                   @ApiParam(value = "三层网络描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                   @ApiParam(value = "DNS域", required = false) @RequestParam(value = "dnsDomain", required = false) String dnsDomain,
                                                                   @ApiParam(value = "镜像网络Id", required = false) @RequestParam(value = "mirrorNetworkId", required = false) String mirrorNetworkId) throws OpenapiException;

    @ApiOperation(value = "创建zstack私有网络", response = L3NetworkInventory.class)
    @RequestMapping(value = "/network/zstackL3/privateNetwork", method = RequestMethod.POST)
    ResponseEntity<L3NetworkInventory> createZStackL3PrivateNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                    @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                    @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                    @ApiParam(value = "二层网络Id", required = true) @RequestParam("l2NetworkId") String l2NetworkId,
                                                                    @ApiParam(value = "三层网络名称", required = true) @RequestParam("name") String name,
                                                                    @ApiParam(value = "三层网络描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                    @ApiParam(value = "DNS域", required = false) @RequestParam(value = "dnsDomain", required = false) String dnsDomain,
                                                                    @ApiParam(value = "镜像网络Id", required = false) @RequestParam(value = "mirrorNetworkId", required = false) String mirrorNetworkId) throws OpenapiException;

    @ApiOperation(value = "创建zstack系统网络", response = L3NetworkInventory.class)
    @RequestMapping(value = "/network/zstackL3/systemNetwork", method = RequestMethod.POST)
    ResponseEntity<L3NetworkInventory> createZStackL3SystemNetwork(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                   @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                   @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                   @ApiParam(value = "二层网络Id", required = true) @RequestParam("l2NetworkId") String l2NetworkId,
                                                                   @ApiParam(value = "三层网络名称", required = true) @RequestParam("name") String name,
                                                                   @ApiParam(value = "三层网络描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                   @ApiParam(value = "DNS域", required = false) @RequestParam(value = "dnsDomain", required = false) String dnsDomain) throws OpenapiException;

    @ApiOperation(value = "创建zstack虚拟网络", response = ZStackVirtualNetworkCreateResult.class)
    @RequestMapping(value = "/network/zstack/virtualNetwork", method = RequestMethod.POST)
    ResponseEntity<ZStackVirtualNetworkCreateResult> createZStackL3Network(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                           @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                           @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                           @ApiParam(value = "二层网络名称", required = true) @RequestParam("l2NetworkName") String l2NetworkName,
                                                                           @ApiParam(value = "三层网络名称", required = true) @RequestParam("name") String name,
                                                                           @ApiParam(value = "物理网卡(eth1, eno1)", required = true) @RequestParam("physicalInterface") String physicalInterface,
                                                                           @ApiParam(value = "起始ip", required = true) @RequestParam("startIp") String startIp,
                                                                           @ApiParam(value = "结束ip", required = true) @RequestParam("endIp") String endIp,
                                                                           @ApiParam(value = "网关地址", required = true) @RequestParam("gateway") String gateway,
                                                                           @ApiParam(value = "VLanId", required = false) @RequestParam(value = "vLanId", required = false) Integer vLanId,
                                                                           @ApiParam(value = "子网掩码", required = false) @RequestParam(value = "netmask", required = false, defaultValue = "255.255.255.0") String netmask,
                                                                           @ApiParam(value = "前缀长度", required = false) @RequestParam(value = "prefixLen", required = false) Integer prefixLen,
                                                                           @ApiParam(value = "三层网络类型", required = false) @RequestParam(value = "l3NetworkType", required = false) L3NetworkType l3NetworkType,
                                                                           @ApiParam(value = "三层网络种类(Private,System,Public)", required = false) @RequestParam(value = "l3NetworkCategoryType", required = false) L3NetworkCategoryType l3NetworkCategory,
                                                                           @ApiParam(value = "IPV4/IPV6", required = false) @RequestParam(value = "ipVersion", required = false) IpVersion ipVersion,
                                                                           @ApiParam(value = "DHCP服务IP", required = false) @RequestParam(value = "dhcpIp", required = false) String dhcpIp,
                                                                           @ApiParam(value = "DNS域名", required = false) @RequestParam(value = "dnsDomain", required = false) String dnsDomain) throws OpenapiException;

    @ApiOperation(value = "创建zstack云路由规格", response = VirtualRouterOfferingInventory.class)
    @RequestMapping(value = "/network/zstackVirtualRouterOffering", method = RequestMethod.POST)
    ResponseEntity<VirtualRouterOfferingInventory> createZStackVirtualRouterOffering(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                                     @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                     @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                                     @ApiParam(value = "管理L3网络Id", required = true) @RequestParam("managementL3NetworkId") String managementL3NetworkId,
                                                                                     @ApiParam(value = "公有L3网络Id", required = true) @RequestParam("publicL3NetworkId") String publicL3NetworkId,
                                                                                     @ApiParam(value = "镜像Id", required = true) @RequestParam("imageId") String imageId,
                                                                                     @ApiParam(value = "名称", required = true) @RequestParam("name") String name,
                                                                                     @ApiParam(value = "CPU数量", required = true) @RequestParam("cpuNum") Integer cpuNum,
                                                                                     @ApiParam(value = "内存大小(Mb)", required = true) @RequestParam("memorySizeInMb") Long memorySizeInMb,
                                                                                     @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                                     @ApiParam(value = "是否默认", required = false) @RequestParam(value = "isDefault", required = false) Boolean isDefault) throws OpenapiException;

    @ApiOperation(value = "查询zstack云路由规格列表", response = ListZStackVirtualRouterOfferingsResult.class)
    @RequestMapping(value = "/network/zstackVirtualRouterOfferings", method = RequestMethod.GET)
    ResponseEntity<ListZStackVirtualRouterOfferingsResult> listZStackVirtualRouterOfferings(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                            @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                                            @ApiParam(value = "云规格路由id列表", required = false) @RequestParam(value = "virtualRouterTypeIds", required = false) Set<String> virtualRouterTypeIds,
                                                                                            @ApiParam(value = "镜像Id", required = false) @RequestParam(value = "imageId", required = false) String imageId,
                                                                                            @ApiParam(value = "管理L3网络Id", required = false) @RequestParam(value = "managementL3NetworkId", required = false) String managementL3NetworkId,
                                                                                            @ApiParam(value = "状态[启用/停用]", required = false) @RequestParam(value = "usingState", required = false) UsingStateType usingState,
                                                                                            @ApiParam(value = "cpu数量(大于等于)", required = false) @RequestParam(value = "cpuNum", required = false) Integer cpuNum,
                                                                                            @ApiParam(value = "内存大小Mb(大于等于)", required = false) @RequestParam(value = "memorySizeInMb", required = false) Long memorySizeInMb) throws OpenapiException;

    @ApiOperation(value = "创建zstackVpc路由器,并添加dns", response = VirtualRouterVmInventory.class)
    @RequestMapping(value = "/network/zstackVpcRouterWithDns", method = RequestMethod.POST)
    ResponseEntity<VirtualRouterVmInventory> createZStackVpcRouterWithDns(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                          @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                          @ApiParam(value = "云路由规格Id", required = true) @RequestParam("virtualRouterOfferingId") String virtualRouterOfferingId,
                                                                          @ApiParam(value = "名称", required = true) @RequestParam("name") String name,
                                                                          @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                          @ApiParam(value = "DNS", required = false) @RequestParam(value = "dns", required = false) String dns,
                                                                          @ApiParam(value = "默认静态ip", required = false) @RequestParam(value = "staticVip", required = false) String staticVip,
                                                                          @ApiParam(value = "高可用组id", required = false) @RequestParam(value = "haGroupId", required = false) String haGroupId,
                                                                          @ApiParam(value = "集群id", required = false) @RequestParam(value = "clusterId", required = false) String clusterId) throws OpenapiException;

    @ApiOperation(value = "向zstackVpc路由器添加/删除dns", response = VpcRouterVmInventory.class)
    @RequestMapping(value = "/network/zstackVpcRouter/{vpcRouterId}/Dns/{dns}", method = RequestMethod.POST)
    ResponseEntity<VpcRouterVmInventory> operateZStackDnsToVpcRouter(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                     @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                     @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                     @ApiParam(value = "vpc路由Id", required = true) @PathVariable("vpcRouterId") String vpcRouterId,
                                                                     @ApiParam(value = "DNS", required = true) @PathVariable("dns") String dns,
                                                                     @ApiParam(value = "操作(添加/删除)", required = true) @RequestParam("action") VpcRouterDnsActionType action) throws OpenapiException;

    @ApiOperation(value = "查询zstackVpc路由器列表", response = ListZStackVpcRoutersResult.class)
    @RequestMapping(value = "/network/zstackVpcRouters", method = RequestMethod.GET)
    ResponseEntity<ListZStackVpcRoutersResult> listZStackVpcRouters(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                    @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                    @ApiParam(value = "vpc路由器id列表", required = false) @RequestParam(value = "vpcRouterIds", required = false) Set<String> vpcRouterIds,
                                                                    @ApiParam(value = "名称", required = false) @RequestParam(value = "name", required = false) String name,
                                                                    @ApiParam(value = "集群Id", required = false) @RequestParam(value = "clusterId", required = false) String clusterId,
                                                                    @ApiParam(value = "默认三层网络Id", required = false) @RequestParam(value = "defaultL3NetworkId", required = false) String defaultL3NetworkId,
                                                                    @ApiParam(value = "默认路由（default routing）的三层网络Id", required = false) @RequestParam(value = "defaultRouteL3NetworkId", required = false) String defaultRouteL3NetworkId,
                                                                    @ApiParam(value = "物理主机Id", required = false) @RequestParam(value = "hostId", required = false) String hostId,
                                                                    @ApiParam(value = "镜像Id", required = false) @RequestParam(value = "imageId", required = false) String imageId,
                                                                    @ApiParam(value = "管理网络Id", required = false) @RequestParam(value = "managementNetworkId", required = false) String managementNetworkId,
                                                                    @ApiParam(value = "公有网络Id", required = false) @RequestParam(value = "publicNetworkId", required = false) String publicNetworkId,
                                                                    @ApiParam(value = "云路由规格Id", required = false) @RequestParam(value = "instanceOfferingId", required = false) String instanceOfferingId,
                                                                    @ApiParam(value = "状态(常用[Stopped/Running])", required = false) @RequestParam(value = "state", required = false) InstanceStateType state) throws OpenapiException;

    @ApiOperation(value = "删除zstackVpc路由器", response = VirtualRouterVmInventory.class)
    @RequestMapping(value = "/network/zstackVpcRouter/{vpcRouterId}", method = RequestMethod.DELETE)
    ResponseEntity<OpenapiResponse> deleteZStackVpcRouter(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                          @ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                          @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                          @ApiParam(value = "vpc路由Id", required = true) @RequestParam("vpcRouterId") String vpcRouterId) throws OpenapiException;

    @ApiOperation(value = "查询zstack三层网络Ip统计信息", response = ListZStackL3NetworkIpStatisticResult.class)
    @RequestMapping(value = "/network/zstackIpStatistic/{l3NetworkId}", method = RequestMethod.GET)
    ResponseEntity<ListZStackL3NetworkIpStatisticResult> listZStackL3NetworkIpStatistics(@ApiParam(value = "云账户ID", required = true) @RequestParam("cloudId") Long cloudId,
                                                                                         @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Console") Scope scope,
                                                                                         @ApiParam(value = "三层网络Id", required = true) @PathVariable("l3NetworkId") String l3NetworkId,
                                                                                         @ApiParam(value = "ip资源类型", required = false) @RequestParam(value = "ipResourceType", required = false) IpResourceType ipResourceType,
                                                                                         @ApiParam(value = "ip地址", required = false) @RequestParam(value = "ipAddress", required = false) String ipAddress) throws OpenapiException;
}
