package com.cloudbility.openapi.v2;


import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.NoticeInfo;
import com.cloudbility.openapi.response.NoticeListResult;
import com.cloudbility.openapi.response.NoticeTagResult;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.net.MalformedURLException;

@Api(tags = "14 notice api", description = "公告栏API")
public interface NoticeApi extends Openapi {


    @ApiOperation(value = "查询公告栏基本信息")
    @RequestMapping(value = "/notice/info", method = RequestMethod.GET)
    NoticeInfo getNoticeInfo() throws OpenapiException;

    @ApiOperation(value = "开启/关闭公告栏")
    @RequestMapping(value = "/notice/config", method = RequestMethod.PUT)
    void updateNoticeConfig(@ApiParam("是否开启") @RequestParam("enabled") boolean enabled) throws OpenapiException;

    @ApiOperation(value = "查询标签列表")
    @RequestMapping(value = "/notice/tags", method = RequestMethod.GET)
    NoticeTagResult getTags() throws OpenapiException;

    @ApiOperation(value = "创建标签")
    @RequestMapping(value = "/notice/tag", method = RequestMethod.POST)
    NoticeTagVo createTag(@ApiParam("名称") @RequestParam(value = "name", required = true) String name, @ApiParam(value = "颜色,形如#39c6ee", required = true) @RequestParam(value = "color", defaultValue = "#39c6ee") String color) throws OpenapiException;

    @ApiOperation(value = "删除标签")
    @RequestMapping(value = "/notice/tag/byName/{name}", method = RequestMethod.DELETE)
    void deleteTag(@ApiParam("名称") @PathVariable("name") String name) throws OpenapiException;

    @ApiOperation(value = "删除标签")
    @RequestMapping(value = "/notice/tag/{id}", method = RequestMethod.DELETE)
    void deleteTag(@ApiParam("标签id") @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "创建公告")
    @RequestMapping(value = "/notice", method = RequestMethod.POST)
    NoticeVo createNotice(
            @ApiParam("标题") @RequestParam(value = "title") String title,
            @ApiParam("内容，富文本格式") @RequestParam(value = "content") String content,
            @ApiParam("标签ID") @RequestParam(value = "tagId") Long tagId,
            @ApiParam("原文地址") @RequestParam(value = "sourceUrl", required = false) String sourceUrl
    ) throws OpenapiException;

    @ApiOperation(value = "修改公告")
    @RequestMapping(value = "/notice/{id}", method = RequestMethod.PUT)
    void updateNotice(
            @ApiParam("公告id") @PathVariable("id") Long id,
            @ApiParam("标题") @RequestParam(value = "title") String title,
            @ApiParam("内容，富文本格式") @RequestParam(value = "content") String content,
            @ApiParam("标签ID") @RequestParam(value = "tagId", required = false) Long tagId,
            @ApiParam("原文地址") @RequestParam(value = "sourceUrl", required = false) String sourceUrl
    ) throws OpenapiException;

    @ApiOperation(value = "发布公告")
    @RequestMapping(value = "/notice/{id}/publish", method = RequestMethod.PUT)
    void publishNotice(
            @ApiParam("公告id") @PathVariable("id") Long id,
            @ApiParam(value = "发布时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"，不设置时取当前时间") @RequestParam(value = "publishTime", required = false) String publishTimeStr
    ) throws OpenapiException;

    @ApiOperation(value = "取消发布")
    @RequestMapping(value = "/notice/{id}/unpublish", method = RequestMethod.PUT)
    void unpublishNotice(
            @ApiParam("公告id") @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "修改公告的封面（http表单）")
    @RequestMapping(value = "/notice/{id}/coverImg", method = RequestMethod.PUT)
    NoticeAttachmentVo updateNoticeCoverImg(
            @ApiParam("id") @PathVariable("id") Long id,
            @ApiParam(value = "图片，不超过5MB", required = true) @RequestParam("image") MultipartFile image) throws OpenapiException;

    @ApiOperation(value = "上传附件（http表单）")
    @RequestMapping(value = "/notice/{id}/attachment", method = RequestMethod.POST)
    NoticeAttachmentVo addNoticeAttachment(
            @ApiParam("id") @PathVariable("id") Long id,
            @ApiParam(value = "附件", required = true) @RequestParam("file") MultipartFile file) throws OpenapiException;

    @ApiOperation(value = "删除附件（或封面）")
    @RequestMapping(value = "/notice/attachment/{attachmentId}", method = RequestMethod.DELETE)
    void deleteNoticeAttachment(
            @ApiParam("attachmentId") @PathVariable("attachmentId") Long attachmentId) throws OpenapiException;

    @ApiOperation(value = "查询公告详情")
    @RequestMapping(value = "/notice/{id}", method = RequestMethod.GET)
    NoticeDetailVo getNoticeDetail(@ApiParam("公告ID") @PathVariable("id") Long id) throws OpenapiException, MalformedURLException;

    @ApiOperation(value = "查询公告列表")
    @RequestMapping(value = "/notice", method = RequestMethod.GET)
    NoticeListResult getNotices(
            @ApiParam("按标签过滤") @RequestParam(value = "tagId", required = false) Long tagId,
            @ApiParam("状态") @RequestParam(value = "status", required = false) PublishStatus status,
            @ApiParam("标题") @RequestParam(value = "title", required = false) String title,
            @ApiParam("排序字段：publishTime,createTime,title") @RequestParam(value = "sortField", required = false, defaultValue = "createTime") String sortField,
            @ApiParam("顺序顺序") @RequestParam(value = "sortOrder", defaultValue = "desc") SortOrder sortOrder,
            @ApiParam(value = "页码:从1开始") @RequestParam(value = "page", defaultValue = "1") int page,
            @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size,
            @ApiParam(value = "是否包含附件") @RequestParam(value = "includeAttachments", defaultValue = "false") boolean includeAttachments) throws OpenapiException, MalformedURLException;

}
