package com.cloudbility.openapi.v2;


import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.OpsPolicyGroupResult;
import com.cloudbility.openapi.response.OpsPolicyItemResult;
import com.cloudbility.openapi.response.OpsPolicyResult;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "19 ops policy api", description = "运维策略API")
public interface OpsPolicyApi extends Openapi {

    // 下面是运维策略
    @RequestMapping(value = "/opsPolicy/defaultItems", method = RequestMethod.GET)
    @ApiOperation(value = "获取默认运维策略条目", notes = "返回当前系统中的运维策略列表，更新版本后，新增和修改的运维策略会返回")
    OpsPolicyItemResult getDefaultOpsPolicyItems();

    @RequestMapping(value = "/opsPolicyGroups", method = RequestMethod.GET)
    @ApiOperation(value = "获取团队策略组列表")
    @Deprecated
    OpsPolicyGroupResult getOpsPolicyGroups(@ApiParam(required = true, value = "团队ID") @RequestParam("teamId") Long teamId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicyGroup/{id}", method = RequestMethod.GET)
    @ApiOperation(value = "获取策略组详情")
    @Deprecated
    OpsPolicyGroupDetailVo getOpsPolicyGroupDetail(@ApiParam(required = true, value = "策略组ID") @PathVariable("id") Long policyGroupId) throws OpenapiException;


    @RequestMapping(value = "/opsPolicy/{id}", method = RequestMethod.GET)
    @ApiOperation(value = "获取子策略详情")
    OpsPolicyDetailVo getOpsPolicyDetail(@ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long policyId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicyGroup/{id}", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除策略组")
    @Deprecated
    void deleteOpsPolicyGroup(@ApiParam(required = true, value = "策略组ID") @PathVariable("id") Long policyGroupId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除子策略")
    void deleteOpsPolicy(@ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long id) throws OpenapiException;

    @RequestMapping(value = "/opsPolicies/byGroup/{groupId}", method = RequestMethod.GET)
    @ApiOperation(value = "获取策略组子策略列表")
    @Deprecated
    OpsPolicyResult getOpsPolicies(@ApiParam(required = true, value = "策略组ID") @PathVariable("groupId") Long policyGroupId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/byCloud/{cloudId}", method = RequestMethod.GET)
    @ApiOperation(value = "获取云账户的子策略")
    @Deprecated
    OpsPolicyResult getOpsPoliciesByCloud(@ApiParam(required = true, value = "云账户ID") @PathVariable("cloudId") Long cloudId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}/items", method = RequestMethod.GET)
    @ApiOperation(value = "获取子策略的条目", notes = "v4.18：目前不会返回BatchType的详细数据")
    OpsPolicyItemResult getOpsPolicyItems(@ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long policyId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}/items", method = RequestMethod.PUT)
    @ApiOperation(value = "修改子策略的条目", notes = "注意：1，定义策略条目的identity:{category}.{key}；<br/>" +
            "2，更新请求的参数名是条目的identity，参数值是条目取值的字符串形式；<br/>" +
            "3，当前接口目前仅支持单一取值策略;<br/>" +
            "4，无法找到identity的参数，都会被忽略掉；<br/>" +
            "5，返回被修改过的策略条目")
    OpsPolicyItemResult updateOpsPolicyItems(@ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long policyId) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}/item/{identity:.+}", method = RequestMethod.GET)
    @ApiOperation(value = "获取子策略的条目", notes = "v4.18：目前不会返回BatchType的详细数据")
    OpsPolicyItemVo getOpsPolicyItem(@ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long policyId,
                                     @ApiParam(required = true, value = "策略条目的Id") @PathVariable("identity") String identity
    ) throws OpenapiException;

    @RequestMapping(value = "/opsPolicyGroup", method = RequestMethod.POST)
    @ApiOperation(value = "创建策略组")
    @Deprecated
    OpsPolicyGroupVo createOpsPolicyGroup(
            @ApiParam(required = true, value = "团队ID") @RequestParam("teamId") Long teamId,
            @ApiParam(required = true, value = "策略组名称") @RequestParam("name") String name,
            @ApiParam(value = "关联主机ID列表") @RequestParam(value = "hosts", required = false) String hosts) throws OpenapiException;

    @RequestMapping(value = "/opsPolicyGroup/{id}/hosts", method = RequestMethod.PUT)
    @ApiOperation(value = "修改策略组关联主机")
    @Deprecated
    void updateOpsPolicyGroupHosts(
            @ApiParam(required = true, value = "策略组ID") @PathVariable("id") Long id,
            @ApiParam(value = "主机ID列表，用逗号连接", required = false) @RequestParam(value = "hosts") String hosts) throws OpenapiException;

    @RequestMapping(value = "/opsPolicyGroup/{id}/hosts", method = RequestMethod.POST)
    @ApiOperation(value = "添加主机到策略组")
    @Deprecated
    void addOpsPolicyGroupHosts(
            @ApiParam(required = true, value = "策略组ID") @PathVariable("id") Long id,
            @ApiParam(value = "主机ID列表，用逗号连接", required = false) @RequestParam(value = "hosts") String hosts) throws OpenapiException;

    @RequestMapping(value = "/opsPolicyGroup/{id}/hosts", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除策略组关联的主机")
    @Deprecated
    void deleteOpsPolicyGroupHosts(
            @ApiParam(required = true, value = "策略组ID") @PathVariable("id") Long id,
            @ApiParam(value = "主机ID列表，用逗号连接", required = false) @RequestParam(value = "hosts") String hosts) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy", method = RequestMethod.POST)
    @ApiOperation(value = "创建子策略")
    @Deprecated
    OpsPolicyVo createOpsPolicy(
            @ApiParam(required = true, value = "策略组ID") @RequestParam("policyGroupId") Long policyGroupId,
            @ApiParam(required = true, value = "子策略名称") @RequestParam("name") String name,
            @ApiParam(required = true, value = "子策略描述") @RequestParam("description") String description,
            @ApiParam(required = true, value = "子策略优先级") @RequestParam("priority") Integer priority,
            @ApiParam(value = "授权角色") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "授权组织单元") @RequestParam(value = "departments", required = false) String departments,
            @ApiParam(value = "授权团队成员") @RequestParam(value = "members", required = false) String members) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}/members", method = RequestMethod.PUT)
    @ApiOperation(value = "授权子策略给团队成员")
    void updateOpsPolicyMembers(
            @ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "团队成员ID列表") @RequestParam(value = "members", required = false) String members) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}/departments", method = RequestMethod.PUT)
    @ApiOperation(value = "授权子策略给组织单元")
    void updateOpsPolicyDepartments(
            @ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "组织单元ID列表") @RequestParam(value = "departments", required = false) String departments) throws OpenapiException;

    @RequestMapping(value = "/opsPolicy/{id}/roles", method = RequestMethod.PUT)
    @ApiOperation(value = "授权子策略给角色")
    void updateOpsPolicyRoles(
            @ApiParam(required = true, value = "子策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "角色ID列表") @RequestParam(value = "roles", required = false) String members) throws OpenapiException;

    @Deprecated
    @RequestMapping(value = "/opsPolicyGroup/{id}/priorities", method = RequestMethod.PUT)
    @ApiOperation(value = "修改策略组子策略优先级", notes = "sortedPolicyIds：所有子策略的ID列表，用逗号连接，最前面的优先级越高")
    void updateOpsPolicyGroupPriorities(@ApiParam(required = true, value = "策略组ID") @PathVariable("id") Long policyGroupId,
                                        @ApiParam(required = true, value = "子策略ID排序列表") @RequestParam("sortedPolicyIds") String sortedPolicyIds
    ) throws OpenapiException;


}
