package com.cloudbility.openapi.v2;


import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.OpsPolicyItemResult;
import com.cloudbility.openapi.response.OpsPolicyResult;
import com.cloudbility.openapi.vo.OpsPolicyDetailVo;
import com.cloudbility.openapi.vo.OpsPolicyItemVo;
import com.cloudbility.openapi.vo.OpsPolicyVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "21 ops policy api v2", description = "运维策略API v2")
public interface OpsPolicyV2Api extends Openapi {

    @RequestMapping(value = "/v2/opsPolicy/defaultItems", method = RequestMethod.GET)
    @ApiOperation(value = "获取默认运维策略条目")
    OpsPolicyItemResult getDefaultOpsPolicyItems();

    @RequestMapping(value = "/v2/opsPolicyList", method = RequestMethod.GET)
    @ApiOperation(value = "获取团队策略列表")
    OpsPolicyResult opsPolicyList(@ApiParam(required = true, value = "团队ID") @RequestParam("teamId") Long teamId) throws OpenapiException;

    @RequestMapping(value = "/v2/opsPolicyDetail/{id}", method = RequestMethod.GET)
    @ApiOperation(value = "获取策略详情")
    OpsPolicyDetailVo getOpsPolicyDetail(@ApiParam(required = true, value = "策略ID") @PathVariable("id") Long policyId) throws OpenapiException;


    @RequestMapping(value = "/v2/opsPolicy/{id}", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除策略")
    void deleteOpsPolicy(@ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id) throws OpenapiException;


    @RequestMapping(value = "/v2/opsPolicy/byCloud/{cloudId}", method = RequestMethod.GET)
    @ApiOperation(value = "获取云账户的策略")
    OpsPolicyResult getOpsPoliciesByCloud(@ApiParam(required = true, value = "云账户ID") @PathVariable("cloudId") Long cloudId) throws OpenapiException;

    @RequestMapping(value = "/v2/opsPolicy/{id}/items", method = RequestMethod.GET)
    @ApiOperation(value = "获取策略的条目", notes = "v4.18：目前不会返回BatchType的详细数据")
    OpsPolicyItemResult getOpsPolicyItems(@ApiParam(required = true, value = "策略ID") @PathVariable("id") Long policyId) throws OpenapiException;

    @RequestMapping(value = "/v2/opsPolicy/{id}/items", method = RequestMethod.PUT)
    @ApiOperation(value = "修改策略的条目", notes = "注意：1，定义策略条目的identity:{category}.{key}；<br/>" +
            "2，更新请求的参数名是条目的identity，参数值是条目取值的字符串形式；<br/>" +
            "3，当前接口目前仅支持单一取值策略;<br/>" +
            "4，无法找到identity的参数，都会被忽略掉；<br/>" +
            "5，返回被修改过的策略条目")
    OpsPolicyItemResult updateOpsPolicyItems(@ApiParam(required = true, value = "策略ID") @PathVariable("id") Long policyId) throws OpenapiException;
    @RequestMapping(value = "/v2/opsPolicy/{id}/item/{identity:.+}", method = RequestMethod.GET)
    @ApiOperation(value = "获取策略条目", notes = "v4.18：目前不会返回BatchType的详细数据")
    OpsPolicyItemVo getOpsPolicyItem(@ApiParam(required = true, value = "策略ID") @PathVariable("id") Long policyId,
                                     @ApiParam(required = true, value = "策略条目的Id") @PathVariable("identity") String identity
    ) throws OpenapiException;


    @RequestMapping(value = "/v2/opsPolicy/{id}/hosts", method = RequestMethod.PUT)
    @ApiOperation(value = "修改策略关联主机")
    void updateOpsPolicyGroupHosts(
            @ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "主机ID列表，用逗号连接", required = false) @RequestParam(value = "hosts") String hosts) throws OpenapiException;

    @RequestMapping(value = "/v2/opsPolicy/{id}/hosts", method = RequestMethod.POST)
    @ApiOperation(value = "添加主机到策略")
    void addOpsPolicyGroupHosts(
            @ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "主机ID列表，用逗号连接", required = false) @RequestParam(value = "hosts") String hosts) throws OpenapiException;

    @RequestMapping(value = "/v2/opsPolicy/{id}/hosts", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除策略关联的主机")
    void deleteOpsPolicyGroupHosts(
            @ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "主机ID列表，用逗号连接", required = false) @RequestParam(value = "hosts") String hosts) throws OpenapiException;

    @RequestMapping(value = "v2/opsPolicy", method = RequestMethod.POST)
    @ApiOperation(value = "创建策略")
    OpsPolicyVo createOpsPolicy(
            @ApiParam(required = true, value = "团队ID") @RequestParam("teamId") Long teamId,
            @ApiParam(required = true, value = "策略名称") @RequestParam("name") String name,
            @ApiParam(required = true, value = "策略描述") @RequestParam("description") String description,
            @ApiParam(required = true, value = "策略优先级") @RequestParam("priority") Integer priority,
            @ApiParam(value = "授权角色") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "授权组织单元") @RequestParam(value = "departments", required = false) String departments,
            @ApiParam(value = "授权团队成员") @RequestParam(value = "members", required = false) String members) throws OpenapiException;

    @RequestMapping(value = "v2/opsPolicy/{id}/members", method = RequestMethod.PUT)
    @ApiOperation(value = "授权策略给团队成员")
    void updateOpsPolicyMembers(
            @ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "团队成员ID列表") @RequestParam(value = "members", required = false) String members) throws OpenapiException;

    @RequestMapping(value = "v2/opsPolicy/{id}/departments", method = RequestMethod.PUT)
    @ApiOperation(value = "授权策略给组织单元")
    void updateOpsPolicyDepartments(
            @ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "组织单元ID列表") @RequestParam(value = "departments", required = false) String departments) throws OpenapiException;

    @RequestMapping(value = "v2/opsPolicy/{id}/roles", method = RequestMethod.PUT)
    @ApiOperation(value = "授权策略给角色")
    void updateOpsPolicyRoles(
            @ApiParam(required = true, value = "策略ID") @PathVariable("id") Long id,
            @ApiParam(value = "角色ID列表") @RequestParam(value = "roles", required = false) String members) throws OpenapiException;

    @RequestMapping(value = "v2/opsPolicy/{teamId}/priorities", method = RequestMethod.PUT)
    @ApiOperation(value = "修改策略优先级", notes = "sortedPolicyIds：所有策略的ID列表，用逗号连接，最前面的优先级越高")
    void updateOpsPolicyGroupPriorities(@ApiParam(required = true, value = "团队ID") @PathVariable("teamId") Long teamId,
                                        @ApiParam(required = true, value = "子策略ID排序列表") @RequestParam("sortedPolicyIds") String sortedPolicyIds
    ) throws OpenapiException;


}
