package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.BooleanResult;
import com.cloudbility.openapi.response.OrderListResult;
import com.cloudbility.openapi.response.PagedOrderListResult;
import com.cloudbility.openapi.vo.OrderVo;
import com.cloudbility.openapi.vo.Scope;
import com.cloudbility.openapi.vo.ServeType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.math.BigDecimal;

/**
 * Created by lidonghao on 2020/8/6.
 */
@Api(tags = "24 order api", description = "订单API")
public interface OrderApi extends Openapi {

    @ApiOperation(value = "分页获取团队订单")
    @RequestMapping(value = "/order/byTeam", method = RequestMethod.GET)
    public PagedOrderListResult findOrdersByTeam(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                                 @ApiParam(value = "服务类型", required = false) @RequestParam(value = "serveType", required = false) ServeType serveType,
                                                 @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                 @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "分页获取团队的有效订单")
    @RequestMapping(value = "/order/valid", method = RequestMethod.GET)
    public PagedOrderListResult findValidOrdersByModule(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                                        @ApiParam(value = "模块Code", required = false) @RequestParam(value = "moduleCode", required = false) String moduleCode,
                                                        @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                        @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "根据订单号查询订单")
    @RequestMapping(value = "/order/byNumber", method = RequestMethod.GET)
    public OrderVo findOrderByNumber(@ApiParam(value = "订单号", required = true) @RequestParam(value = "orderNumber") String orderNumber) throws OpenapiException;

    @ApiOperation(value = "创建订单")
    @RequestMapping(value = "/order/create", method = RequestMethod.POST)
    public OrderVo createOrder(@ApiParam(value = "创建者ID", required = true) @RequestParam(value = "userId") Long userId,
                               @ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                               @ApiParam(value = "服务类型", required = true) @RequestParam(value = "serveType") ServeType serveType,
                               @ApiParam(value = "购买时长", required = true) @RequestParam(value = "productTerm") Integer productTerm,
                               @ApiParam(value = "订单总价", required = true) @RequestParam(value = "price") BigDecimal price,
                               @ApiParam(value = "模块Code", required = false) @RequestParam(value = "moduleCode", required = false) String moduleCode,
                               @ApiParam(value = "模块名称", required = false) @RequestParam(value = "moduleName", required = false) String moduleName,
                               @ApiParam(value = "套餐Code", required = false) @RequestParam(value = "editionCode", required = false) String editionCode,
                               @ApiParam(value = "套餐名称", required = false) @RequestParam(value = "editionName", required = false) String editionName,
                               @ApiParam(value = "配额，格式为[{\"quotaCode\":\"max_user\", \"quotaName\":\"最大用户数\", \"unit\":\"个\", \"quota\": 10}]", required = false) @RequestParam(value = "quotas", required = false) String quotas,
                               @ApiParam(value = "服务过期时间，UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = false) @RequestParam(value = "serveExpireTime", required = false) String serveExpireTimeStr,
                               @ApiParam(value = "产品ID，用于标识某个产品", required = false) @RequestParam(value = "productId", required = false) String productId,
                               @ApiParam(value = "订单来源", required = false) @RequestParam(value = "scope", required = false, defaultValue = "Portal") Scope scope) throws OpenapiException;

    @ApiOperation(value = "查询团队的试用订单")
    @RequestMapping(value = "/order/trial", method = RequestMethod.GET)
    public OrderListResult findTrailOrder(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                          @ApiParam(value = "模块Code", required = true) @RequestParam(value = "moduleCode") String moduleCode) throws OpenapiException;

    @ApiOperation(value = "查询团队某个模块的所有订单")
    @RequestMapping(value = "/order/byModule", method = RequestMethod.GET)
    public OrderListResult findOrdersByModule(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                              @ApiParam(value = "模块Code", required = true) @RequestParam(value = "moduleCode") String moduleCode) throws OpenapiException;

//    @ApiOperation(value = "结束订单")
//    @RequestMapping(value = "/order/finish", method = RequestMethod.POST)
//    public OrderVo findOrderByNumber(@ApiParam(value = "订单号", required = true) @RequestParam(value = "orderNumber") String orderNumber,
//                                     @ApiParam(value = "是否取消", required = true) @RequestParam(value = "cancel", required = true, defaultValue = "false") boolean cancel,
//                                     @ApiParam(value = "取消原因", required = false) @RequestParam(value = "reason", required = false) String reason) throws OpenapiException;

    @ApiOperation(value = "创建订单")
    @RequestMapping(value = "/order/createFromMarket", method = RequestMethod.POST)
    public OrderVo createFromMarket(@ApiParam(value = "团队code", required = true) @RequestParam(value = "teamCode") String teamCode,
                                         @ApiParam(value = "服务类型", required = true) @RequestParam(value = "serveType") ServeType serveType,
                                         @ApiParam(value = "模块code", required = true) @RequestParam(value = "moduleCode") String moduleCode,
                                         @ApiParam(value = "规格code", required = true) @RequestParam(value = "editionCode") String editionCode,
                                         @ApiParam(value = "购买时长", required = true) @RequestParam(value = "productTerm") Integer productTerm,
                                         @ApiParam(value = "配额，格式为[{\"quotaCode\":\"max_user\", \"quotaName\":\"最大用户数\", \"unit\":\"个\", \"quota\": 10}]", required = false) @RequestParam(value = "quotas", required = false) String quotas,
                                         @ApiParam(value = "订单来源", required = true) @RequestParam(value = "scope") Scope scope) throws OpenapiException;

    @ApiOperation(value = "结束订单")
    @RequestMapping(value = "/order/finish", method = RequestMethod.POST)
    public OrderVo finishMarketOrder(@ApiParam(value = "订单号", required = true) @RequestParam(value = "orderNumber") String orderNumber,
                                     @ApiParam(value = "是否取消", required = true) @RequestParam(value = "cancel", required = true, defaultValue = "false") boolean cancel,
                                     @ApiParam(value = "取消原因", required = false) @RequestParam(value = "reason", required = false) String reason) throws OpenapiException;

    @ApiOperation(value = "试用订单")
    @RequestMapping(value = "order/createTrialOrder", method = RequestMethod.POST)
    public OrderVo createTrialOrderMarket(@ApiParam(value = "团队id", required = true) @RequestParam(value = "teamId") Integer teamId,
                                          @ApiParam(value = "模块id", required = true) @RequestParam(value = "moduleCode") String moduleCode) throws OpenapiException;

    @ApiOperation(value = "更新订单来源")
    @RequestMapping(value = "order/updateScope", method = RequestMethod.POST)
    public ResponseEntity<BooleanResult> updateOrderScope(@ApiParam(value = "订单号", required = true) @RequestParam(value = "orderNumber") String orderNumber,
                                                          @ApiParam(value = "订单来源", required = true) @RequestParam(value = "scope") Scope scope) throws OpenapiException;

}
