package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.NoneSAAS;
import com.cloudbility.openapi.request.SmsRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.AkAccessToken;
import com.cloudbility.openapi.vo.CheckTokenResult;
import com.cloudbility.openapi.vo.RedirectPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@Api(tags = "other api", description = "其他API")
public interface OtherApi extends Openapi {


    @RequestMapping(value = "/oauth", method = RequestMethod.GET)
    @ApiOperation(
            value = "获取OAuth Token（客户端不需要认证）",
            notes = "客户端无需认证（即签名或者设置Authentication请求头）即可发起请求，建议在https环境下使用，以避免泄露accessKeySecret",
            produces = "application/json"
    )
    AkAccessToken getOauth(
            @ApiParam(required = true) @RequestParam("accessKeyId") String accessKeyId,
            @ApiParam(required = true) @RequestParam("accessKeySecret") String accessKeySecret,
            @ApiParam(required = true, value = "超时时间，单位秒，取值范围：[120,86400]", allowableValues = "range[120,7200]") @RequestParam(value = "expireSeconds", defaultValue = "300") int expireSeconds) throws OpenapiException;

    @RequestMapping(value = "/oauthV2", method = RequestMethod.GET)
    @ApiOperation(
            value = "获取OAuth Token（限已认证的客户端）",
            notes = "客户端需要认证（即签名或者设置Authentication请求头）才能发起请求",
            produces = "application/json"
    )
    AkAccessToken getOauth(@ApiParam(required = true, value = "超时时间，单位秒，取值范围：[120,86400]", allowableValues = "range[120,7200]") @RequestParam(value = "expireSeconds", defaultValue = "300") int expireSeconds) throws OpenapiException;

    @RequestMapping(value = "/checkOauth", method = RequestMethod.GET)
    @ApiOperation(
            value = "检测指定的Openapi Token是否合法",
            notes = "token可以设置在http头Authorization，或者请求参数token",
            produces = "application/json"
    )
    CheckTokenResult checkOauth(@ApiParam(required = false, value = "Authorization http头，与token参数二选一") @RequestHeader(value = "Authorization", required = false) String authorization,
                                @ApiParam(required = false, value = "token参数") @RequestParam(value = "token", required = false) String token);

    @RequestMapping(value = "/oauth/{token}", method = RequestMethod.DELETE)
    @ApiOperation(
            value = "删除OAuth Token（客户端无需认证）",
            notes = "客户端无需认证（即签名或者设置Authentication请求头）即可发起请求，建议在https环境下使用，以避免泄露accessKeySecret。这个接口可以删除API请求和登录链接的token",
            produces = "application/json"
    )
    void deleteOauth(
            @ApiParam(required = true) @RequestParam("accessKeyId") String accessKeyId, @ApiParam(required = true) @RequestParam("accessKeySecret") String accessKeySecret, @ApiParam(required = true) @PathVariable(value = "token") String token) throws OpenapiException;

    @RequestMapping(value = "/oauthV2/{token}", method = RequestMethod.DELETE)
    @ApiOperation(
            value = "删除OAuth Token（仅限已认证的客户端）",
            notes = "客户端需要认证（即签名或者设置Authentication请求头）才能发起请求。这个接口可以删除API请求和登录链接的token",
            produces = "application/json"
    )
    void deleteOauth(@ApiParam(required = true) @PathVariable(value = "token") String token) throws OpenapiException;


    @RequestMapping(value = "/ping", method = RequestMethod.GET)
    @ApiOperation(
            value = "检测openapi的连通性",
            response = BooleanResult.class,
            produces = "application/json",
            notes = "当前操作不需要认证也可以进行"
    )
    BooleanResult pingNew() throws OpenapiException;

    @RequestMapping(value = "/permissionQuota", method = RequestMethod.GET)
    @ApiOperation(
            value = "查询权限的配额使用情况",
            response = PermissionQuotaResult.class,
            produces = "application/json"
    )
    PermissionQuotaResult getPermissionQuota(
            @ApiParam(value = "权限名称列表，用','连接", required = true)
            @RequestParam("permissions") String permissions) throws OpenapiException;


    @RequestMapping(value = "/quota", method = RequestMethod.GET)
    @ApiOperation(
            value = "查询系统配额以及使用情况",
            response = GetQuotaResult.class,
            produces = "application/json",
            notes = "私有部署环境查询系统所有资源和配额；SAAS环境查询当前ISV所有资源和配额。v6之后升级了全新的配额系统"

    )
    @Deprecated
    GetQuotaResult getQuota() throws OpenapiException;


    @ApiOperation(value = "获取自动登的录授权URL", response = OAuthUrlResult.class)
    @RequestMapping(value = "/oauthLogin", method = RequestMethod.GET)
    @NoneSAAS
    OAuthUrlResult oauthLogin(
            @ApiParam(value = "登录用户的ID", required = true) @RequestParam("userId") Long userId,
            @ApiParam(value = "是否一次有效", required = true, defaultValue = "true") @RequestParam(value = "oneoff", defaultValue = "true") boolean oneoff,
            @ApiParam(value = "链接超时时间，单位：秒，取值范围[5min,24hr]", required = true) @RequestParam(value = "expireSeconds", defaultValue = "600") long expireSeconds,
            @ApiParam(value = "登录后重定向的团队ID,如果未设置，打开默认团队") @RequestParam(value = "teamId", required = false) Long teamId,
            @ApiParam(value = "如果设置了，登录后重定向到主机详情页") @RequestParam(value = "hostId", required = false) Long hostId,
            @ApiParam(value = "如果设置了，登录后打开特定菜单") @RequestParam(value = "page", required = false, defaultValue = "Home") RedirectPage page,
            @ApiParam(value = "如果设置了，登录后重定向到对应的工单流程") @RequestParam(value = "workflowInstanceId", required = false) Long workflowInstanceId,
            @ApiParam(value = "重定向URL。如果设置了，登录后打开特定URL，优先级比page和hostId高") @RequestParam(value = "redirectUrl", required = false) String redirectUrl
    ) throws OpenapiException;

    @RequestMapping(value = "/sms", method = RequestMethod.POST)
    @ApiOperation(
            value = "发送短信",
            response = SmsResult.class
    )
    SmsResult sendSmsNew(@ApiParam(value = "短信手机号模版等信息", required = true) @RequestBody SmsRequest request) throws OpenapiException;

    @RequestMapping(value = "/smsV2", method = RequestMethod.POST)
    @ApiOperation(
            value = "发送短信，支持国际手机号",
            response = SmsResultV2.class,
            notes = "返回每个手机号的发送结果"
    )
    SmsResultV2 sendSmsV2(@ApiParam(value = "短信手机号模版等信息", required = true) @RequestBody SmsRequest request) throws OpenapiException;


    @RequestMapping(value = "/checkJwt", method = RequestMethod.POST)
    @ApiOperation(
            value = "检查某个jwt是否有效",
            response = CheckJwtResult.class
    )
    CheckJwtResult checkJwt(@ApiParam(value = "jwt字符串", required = true) @RequestParam("jwt") String jwt);

    @RequestMapping(value = "/checkSaaS", method = RequestMethod.GET)
    @ApiOperation(
            value = "检查是否SaaS版",
            response = BooleanResponse.class
    )
    BooleanResponse checkSaaS();

    @ApiOperation(value = "校验手机验证码")
    @RequestMapping(value = "/verifyPhoneCode/{sessionId}/{code}", method = RequestMethod.GET)
    ResponseEntity<BooleanResult> verifyPhoneCode(@ApiParam(required = true) @PathVariable(value = "sessionId") String sessionId,
                                                  @ApiParam(required = true) @PathVariable(value = "code") String code) throws OpenapiException;

}

