package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.QuotaUsedItemVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import static org.springframework.util.MimeTypeUtils.APPLICATION_JSON_VALUE;

@Api(tags = "20 quota api", description = "配额API")
public interface QuotaApi extends Openapi {

    @ApiOperation(value = "获取配额条目")
    @RequestMapping(value = "/quotaItems", method = RequestMethod.GET)
    QuotaItemResult getQuotaItems() throws OpenapiException;

    @ApiOperation(value = "获取团队所有配额信息")
    @RequestMapping(value = "/team/{id}/quotas", method = RequestMethod.GET)
    QuotaUsedItemResult getTeamQuotas(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "获取团队某个配额信息")
    @RequestMapping(value = "/team/{id}/quota/{code:.+}", method = RequestMethod.GET)
    QuotaUsedItemVo getTeamQuota(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                 @ApiParam(value = "配额代码", required = true) @PathVariable("code") String code) throws OpenapiException;

    @ApiOperation(value = "更新团队外部配额的已用(v6）", notes = "仅支持外部配额")
    @RequestMapping(value = "/team/{id}/quota/{code:.+}/used", method = RequestMethod.PUT)
    QuotaUsedItemVo updateTeamQuotaUsed(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                        @ApiParam(value = "配额代码", required = true) @PathVariable("code") String code,
                                        @ApiParam(value = "已用", required = true) @RequestParam("used") long used) throws OpenapiException;

    @ApiOperation(value = "用追加的方式修改团队外部配额的已用(v6）", notes = "仅支持外部配额")
    @RequestMapping(value = "/team/{id}/quota/{code:.+}/increment", method = RequestMethod.POST)
    QuotaUsedItemVo increaseTeamQuotaUsed(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                          @ApiParam(value = "配额代码", required = true) @PathVariable("code") String code,
                                          @ApiParam(value = "已用配额的增量", required = true) @RequestParam("increment") long increment) throws OpenapiException;

    @ApiOperation(value = "获取团队某个模块信息")
    @RequestMapping(value = "/team/{id}/module/{code}", method = RequestMethod.GET)
    GetModuleResult getModuleVo(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                @ApiParam(value = "模块代码", required = true) @PathVariable("code") ModuleDefinition code) throws OpenapiException;

    @ApiOperation(value = "获取某个模块的所有购买时长")
    @RequestMapping(value = "/moduleTerms/{moduleCode}", method = RequestMethod.GET)
    ModuleTermListResult getModuleTerms(@ApiParam(value = "模块代码", required = true) @PathVariable("moduleCode") String moduleCode) throws OpenapiException;
}
