package com.cloudbility.openapi.v2;


import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.*;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;


@Api(tags = "11 script api", description = "脚本API")
public interface ScriptApi extends Openapi {

    @ApiOperation(value = "获取脚本库", response = ScriptLibraryListResult.class)
    @RequestMapping(value = "/script/library", method = RequestMethod.GET)
    public ScriptLibraryListResult getScriptLibraries(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取预设脚本库", response = ScriptLibraryListResult.class)
    @RequestMapping(value = "/preInstalledScriptLibraries", method = RequestMethod.GET)
    public ScriptLibraryListResult getPreInstalledScriptLibraries(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取预设脚本库的子库", response = ScriptLibraryListResult.class)
    @RequestMapping(value = "/preInstalledLibrary/{libraryId}/subLibrary", method = RequestMethod.GET)
    public ScriptLibraryListResult getPreInstalledSubLibrary(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                                             @ApiParam(value = "脚本库ID", required = true) @PathVariable("libraryId") Long libraryId) throws OpenapiException;


    @ApiOperation(value = "获取预设脚本库的脚本")
    @RequestMapping(value = "/script/byPreInstalledLibrary/{libraryId}", method = RequestMethod.GET)
    public ScriptListResult getScriptsByPreInstalledLibrary(@ApiParam(value = "脚本库ID", required = true) @PathVariable("libraryId") Long id,
                                                            @ApiParam(value = "是否包含脚本内容", required = false) @RequestParam(value = "includeContent", required = false, defaultValue = "false") boolean includeContent,
                                                            @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                            @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "根据名称获取脚本库", response = ScriptLibraryVo.class)
    @RequestMapping(value = "/script/library/byName", method = RequestMethod.GET)
    @ApiResponses(value = {
            @ApiResponse(code = 404, message = "未找到脚本库"),
            @ApiResponse(code = 200, message = "特定脚本库")
    })
    public ScriptLibraryVo getScriptLibraryByName(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                                  @ApiParam(value = "脚本库名称", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "修改脚本库名称")
    @RequestMapping(value = "/script/library/{id}", method = RequestMethod.PUT)
    public void updateScriptLibrary(@ApiParam(value = "脚本库ID", required = true) @PathVariable("id") Long id,
                                               @ApiParam(value = "脚本库名称", required = true) @RequestParam("name") String name
    ) throws OpenapiException;

    @ApiOperation(value = "获取脚本库")
    @RequestMapping(value = "/script/library/{id}", method = RequestMethod.GET)
    public ScriptLibraryVo getScriptLibrary(@ApiParam(value = "脚本库ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "创建脚本库", response = ScriptLibraryVo.class)
    @RequestMapping(value = "/script/library", method = RequestMethod.POST)
    public ScriptLibraryVo createScriptLibrary(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                                               @ApiParam(value = "脚本库名称", required = true) @RequestParam("name") String name,
                                               @ApiParam(value = "脚本库说明", required = false) @RequestParam(value = "description", required = false) String description,
                                               @ApiParam(value = "创建人ID，不设置时是团队拥有者", required = false) @RequestParam(value = "creator", required = false) Long creator
    ) throws OpenapiException;

    @ApiOperation(value = "删除脚本库",  notes = "脚本库如果还有脚本，则无法删除")
    @RequestMapping(value = "/script/library/{id}", method = RequestMethod.DELETE)
    public void deleteScriptLibrary(@ApiParam(value = "脚本库ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "删除脚本库中所有脚本", response = CountResult.class)
    @RequestMapping(value = "/script/byLibrary/{id}", method = RequestMethod.DELETE)
    public CountResult deleteScriptsByLibrary(@ApiParam(value = "脚本库ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "获取脚本库中的脚本", response = ScriptListResult.class, notes = "返回的结果按照创建时间正序排列")
    @RequestMapping(value = "/script/byLibrary/{id}", method = RequestMethod.GET)
    public ScriptListResult getScripts(@ApiParam(value = "脚本库ID", required = true) @PathVariable("id") Long id,
                                       @ApiParam(value = "是否包含脚本内容", required = false) @RequestParam(value = "includeContent", required = false, defaultValue = "false") boolean includeContent,
                                       @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                       @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "根据脚本名称查询脚本信息", response = ScriptVo.class)
    @RequestMapping(value = "/script/byName", method = RequestMethod.GET)
    @ApiResponses(value = {
            @ApiResponse(code = 404, message = "未找到脚本"),
            @ApiResponse(code = 200, message = "特定脚本")
    })
    public ScriptVo getScriptByName(@ApiParam(value = "脚本库ID", required = true) @RequestParam("libraryId") Long libraryId,
                                    @ApiParam(value = "脚本名称", required = true) @RequestParam("name") String name,
                                    @ApiParam(value = "是否包含脚本内容", required = false) @RequestParam(value = "includeContent", required = false, defaultValue = "false") boolean includeContent) throws OpenapiException;

    @ApiOperation(value = "添加脚本", response = ScriptVo.class)
    @RequestMapping(value = "/script", method = RequestMethod.POST)
    public ScriptVo createScript(@ApiParam(value = "脚本库ID", required = true) @RequestParam("libraryId") Long libraryId,
                                 @ApiParam(value = "脚本名称", required = true) @RequestParam("name") String name,
                                 @ApiParam(value = "脚本内容", required = true) @RequestParam("content") String content,
                                 @ApiParam(value = "脚本类型", required = true) @RequestParam("language") ScriptLanguage language,
                                 @ApiParam(value = "脚本参数", required = false) @RequestParam(value = "parameter", required = false) String parameter,
                                 @ApiParam(value = "脚本说明", required = false) @RequestParam(value = "description", required = false) String description,
                                 @ApiParam(value = "脚本超时", required = true) @RequestParam(value = "timeout", defaultValue = "300") Integer timeout,
                                 @ApiParam(value = "创建人ID，不设置时是团队拥有者", required = false) @RequestParam(value = "creator", required = false) Long creator) throws OpenapiException;


    @ApiOperation(value = "获取脚本信息", response = ScriptVo.class)
    @RequestMapping(value = "/script/{id}", method = RequestMethod.GET)
    public ScriptVo getScript(@ApiParam(value = "脚本ID", required = true) @PathVariable("id") Long id,
                              @ApiParam(value = "是否包含脚本内容", required = false) @RequestParam(value = "includeContent", required = true, defaultValue = "true") boolean includeContent) throws OpenapiException;

    @ApiOperation(value = "删除脚本")
    @RequestMapping(value = "/script/{id}", method = RequestMethod.DELETE)
    public void deleteScript(@ApiParam(value = "脚本ID", required = true) @PathVariable("id") Long id) throws OpenapiException;


    @ApiOperation(value = "修改脚本信息")
    @RequestMapping(value = "/script/{id}", method = RequestMethod.PUT)
    public void updateScriptInfo(
            @ApiParam(value = "脚本ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "脚本名称", required = true) @RequestParam("name") String name,
            @ApiParam(value = "脚本超时", required = true) @RequestParam("timeout") Integer timeout,
            @ApiParam(value = "脚本描述", required = true) @RequestParam("description") String description) throws OpenapiException;

    @ApiOperation(value = "修改脚本内容")
    @RequestMapping(value = "/script/{id}/content", method = RequestMethod.PUT)
    public void updateScriptContent(
            @ApiParam(value = "脚本ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "脚本内容", required = true) @RequestParam("content") String content) throws OpenapiException;

    @ApiOperation(value = "修改脚本内容",  notes = "脚本内容以payload方式提交", consumes = "application/octet-stream")
    @RequestMapping(value = "/script/{id}/contentWithPayload", method = RequestMethod.PUT)
    public void updateScriptContentWithPayload(
            @ApiParam(value = "脚本ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "执行脚本", response = ScriptVo.class/* notes = "注意，脚本参数和超时的优先级是：设置在主机上的>设置在ScriptExecutingHostsInfo上的>设置在脚本上的"*/)
    @RequestMapping(value = "/script/{id}/execute", method = RequestMethod.POST)
    public JobVo executeScript(@ApiParam(value = "脚本ID", required = true) @PathVariable("id") Long id,
                               @RequestBody ScriptExecutingHostsInfo executingHostsInfo
    ) throws OpenapiException;

    @ApiOperation(value = "获取脚本任务执行信息", response = ScriptRunResult.class, notes = "注意，读取的脚本输出不会超过10KB。超过10KB的输出流，请使用另外接口获取")
    @RequestMapping(value = "/script/job/{jobId}", method = RequestMethod.GET)
    public ScriptRunResult getScriptRunResult(@PathVariable("jobId") Long jobId,
                                              @ApiParam(value = "是否包含脚本输出", required = true) @RequestParam(value = "includeOutput", required = true, defaultValue = "false") boolean includeOutput) throws OpenapiException;

    @ApiOperation(value = "获取脚本在主机上的输出长度", notes = "-1表示不存在")
    @RequestMapping(value = "/script/jobOutput/{jobId}/{hostId}", method = {RequestMethod.HEAD})
    long headScriptOutputOnHost(@ApiParam(value = "任务ID", required = true) @PathVariable("jobId") Long jobId,
                                              @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "获取脚本在主机上的输出", notes = "直接返回流式数据。支持断点下载。")
    @RequestMapping(value = "/script/jobOutput/{jobId}/{hostId}", method = {RequestMethod.GET})
    @ApiResponses(value = {
            @ApiResponse(code = 404, message = "script-run has no output on current host"),
            @ApiResponse(code = 200, message = "流式数据")
    })
    InputStreamResource getScriptOutputOnHost(@ApiParam(value = "任务ID", required = true) @PathVariable("jobId") Long jobId,
                               @ApiParam(value = "主机ID", required = true) @PathVariable("hostId") Long hostId) throws OpenapiException;

    @ApiOperation(value = "通过Agent执行脚本命令", response = ScriptVo.class)
    @RequestMapping(value = "/script/executeCommandWithAgent", method = RequestMethod.POST)
    ResponseEntity<JobVo> executeCommandWithAgent(@ApiParam(value = "主机ID列表", required = true) @RequestParam(value = "hostIds", required = true) String hostIds,
                                                         @ApiParam(value = "平台,linux或者windows", required = true) @RequestParam(value = "platform", required = true, defaultValue = "linux") String platform,
                                                         @ApiParam(value = "Base64加密的脚本内容", required = true) @RequestParam(value = "command", required = true) String base64Command) throws OpenapiException;
}
