package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;


@Api(tags = "20 session api", description = "会话API")
public interface SessionApi extends Openapi {
    @RequestMapping(value = "/closeSession", method = RequestMethod.DELETE)
    @ApiOperation(
            value = "结束一个会话",
            response = BooleanResult.class,
            produces = "application/json"
    )
    BooleanResult closeSession(
            @ApiParam(value = "会话ID", required = true) @RequestParam("sessionId") Long sessionId,
            @ApiParam(value = "关闭理由", format = "reason") @RequestParam(value = "reason", required = false) String reason) throws OpenapiException;

    @ApiOperation(value = "创建并返回一个会话的websocketUrl", response = SessionOpenResult.class)
    @RequestMapping(value = "/openSessionGetWS", method = RequestMethod.GET)
    SessionOpenResult openSessionAndGetWebsocketUrl(
            @ApiParam(value = "登录用户的ID", required = true) @RequestParam("userId") Long userId,
            @ApiParam(value = "主机ID", required = true) @RequestParam("hostId") Long hostId,
            @ApiParam(value = "会话类型", required = true) @RequestParam("desktopType") DesktopType desktopType,
            @ApiParam(value = "目标端口") @RequestParam(value = "port", required = false) Integer port,
            @ApiParam(value = "接入方式", required = true) @RequestParam("accessType") AccessType accessType,
            //分辨率TODO
            @ApiParam(value = "会话账户,SSH/RDP会话是必须的") @RequestParam(value = "account", required = false) String account,
            @ApiParam(value = "会话账户密码", format = "password") @RequestParam(value = "password", required = false) String password,
            @ApiParam(value = "会话账户密钥") @RequestParam(value = "privateKey", required = false) String privateKey,
            @ApiParam(value = "密钥的加密密码") @RequestParam(value = "privateKeyPass", required = false) String privateKeyPass,
            @ApiParam(value = "用户有权限的登录凭证ID，与上面的密码、密钥不要同时使用") @RequestParam(value = "credentialId", required = false) Long credentialId) throws OpenapiException;

    @ApiOperation(value = "获取自动打开主机会话的授权URL", response = OAuthUrlResult.class)
    @RequestMapping(value = "/oauthSession", method = RequestMethod.GET)
    OAuthUrlResult getSessionOauthUrl(
            @ApiParam(value = "登录用户的ID", required = true) @RequestParam("userId") Long userId,
            @ApiParam(value = "是否一次有效", required = true, defaultValue = "true") @RequestParam(value = "oneoff", defaultValue = "true") boolean oneoff,
            @ApiParam(value = "主机ID", required = true) @RequestParam("hostId") Long hostId,
            @ApiParam(value = "会话类型", required = true) @RequestParam("desktopType") DesktopType desktopType,
            @ApiParam(value = "目标端口。不设置时根据协议选取默认端口") @RequestParam(value = "port", required = false) Integer port,
            @ApiParam(value = "接入方式", required = true) @RequestParam("accessType") AccessType accessType,
            @ApiParam(value = "超时时间,秒,[2min,24h]，默认300秒", required = true, defaultValue = "300") @RequestParam(value = "expireSeconds", defaultValue = "300") int expireSeconds,
            //分辨率TODO
            @ApiParam(value = "会话账户,SSH/RDP会话是必须的") @RequestParam(value = "account", required = false) String account,
            @ApiParam(value = "会话账户密码", format = "password") @RequestParam(value = "password", required = false) String password,
            @ApiParam(value = "会话账户密钥") @RequestParam(value = "privateKey", required = false) String privateKey,
            @ApiParam(value = "密钥的加密密码") @RequestParam(value = "privateKeyPass", required = false) String privateKeyPass,
            @ApiParam(value = "用户有权限的登录凭证ID，与上面的密码、密钥不要同时使用") @RequestParam(value = "credentialId", required = false) Long credentialId,
            @ApiParam(value = "设置RDP分辨率（默认值1024x768）不设置时取默认值") @RequestParam(value = "screenResolution", required = false) String screenResolution
    ) throws OpenapiException;

    @ApiOperation(value = "根据某个访问串获取唤醒本地工具的url", response = UrlResult.class, notes = "id和endpoint优先使用id，如果未指定中转服务，默认使用团队可用中转的其中一个")
    @RequestMapping(value = "/getOpenNativeUrl", method = RequestMethod.GET)
    UrlResult getOpenNativeUrl(@ApiParam(value = "访问串用户名", required = true) @RequestParam(value = "account") String account,
                               @ApiParam(value = "会话账户密码", format = "password", required = true) @RequestParam(value = "password") String password,
                               @ApiParam(value = "中转服务ID", required = false) @RequestParam(value = "transitId", required = false) Long transitId,
                               @ApiParam(value = "中转服务endpoint", required = false) @RequestParam(value = "endpoint", required = false) String endpoint
    ) throws OpenapiException;

    @ApiOperation(value = "根据分享code生成某个会话的分享链接url", response = OAuthUrlResult.class)
    @RequestMapping(value = "/oauthSessionShare", method = RequestMethod.GET)
    OAuthUrlResult getSessionShareOauthUrl(
            @ApiParam(value = "被分享的用户ID，如果为空则表明要创建临时用户", required = true) @RequestParam("userId") Long userId,
            @ApiParam(value = "会话分享Code", required = true) @RequestParam("shareCode") String shareCode) throws OpenapiException;

    @ApiOperation(value = "获取打开的会话数")
    @RequestMapping(value = "/sessions/countReady", method = RequestMethod.GET)
    CountResult countSessionReady();

    @ApiOperation(value = "获取打开的会话列表")
    @RequestMapping(value = "/sessions/listReady", method = RequestMethod.GET)
    SessionResult listSessionReady(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "会话应用类型", required = true) @RequestParam(value = "sessionAppType") SessionAppType sessionAppType) throws OpenapiException;

    @ApiOperation(value = "查询会话列表")
    @RequestMapping(value = "/sessions/listAll", method = RequestMethod.GET)
    SessionPageResult listSession(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "会话应用类型", required = true) @RequestParam(value = "sessionAppType") SessionAppType sessionAppType,
            @ApiParam(value = "会话状态") @RequestParam(value = "sessionStatus", required = false) SessionStatus sessionStatus,
            @ApiParam(value = "应用ID") @RequestParam(value = "appId", required = false) Long appId,
            @ApiParam(value = "分页序号") @RequestParam(value = "page", required = false, defaultValue = "1") Integer page,
            @ApiParam(value = "分页大小") @RequestParam(value = "size", required = false, defaultValue = "200") Integer pageSize
    ) throws OpenapiException;

    @ApiOperation(value = "下载会话录像")
    @RequestMapping(value = "/sessions/downloadRecord", method = RequestMethod.GET)
    void downloadRecord(
            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "用户ID", required = true) @RequestParam(value = "userId") Long userId,
            @ApiParam(value = "会话ID", required = true) @RequestParam(value = "sessionId") Long sessionId
            ) throws Exception;


    @ApiOperation(value = "获取团队在线成员数")
    @RequestMapping(value = "/sessions/countReadyByTeam", method = RequestMethod.GET)
    CountResult countReadyByTeam(@ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取团队可用中转列表", response = TransitServerResult.class)
    @RequestMapping(value = "/teamUsableTransitServer", method = RequestMethod.GET)
    TransitServerResult getTeamUsableTransitServer(@ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;


}
