package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.response.GetHostBriefResult;
import com.cloudbility.openapi.response.HostDetailListResult;
import com.cloudbility.openapi.response.ListTagResult;
import com.cloudbility.openapi.response.TagDetailResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "7 tag api", description = "标签操作API")
public interface TagApi extends Openapi {
    @ApiOperation(value = "查询团队标签列表", response = ListTagResult.class, notes = "获取团队的所有标签")
    @RequestMapping(value = "/tag/list", method = RequestMethod.GET)
    ListTagResult listTags(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "查询特定标签", response = TagDetailResult.class, notes = "获取特定标签")
    @RequestMapping(value = "/tag/{tagId}", method = RequestMethod.GET)
    TagDetailResult getTag(@ApiParam(value = "标签名称ID", required = true) @PathVariable("tagId") Long tagId) throws OpenapiException;

    @ApiOperation(value = "修改标签名称",  notes = "修改标签名称，不会修改标签的值")
    @RequestMapping(value = "/tag/{tagId}/tagName", method = RequestMethod.PUT)
    void updateTagName(@ApiParam(value = "标签ID", required = true) @PathVariable("tagId") Long tagId,
                                  @ApiParam(value = "新名称", required = true) @RequestParam("newName") String newName) throws OpenapiException;

    @ApiOperation(value = "修改标签值",  notes = "修改标签值，不会修改标签的名称")
    @RequestMapping(value = "/tag/{tagValueId}/tagValue", method = RequestMethod.PUT)
    void updateTagValue(@ApiParam(value = "标签值ID", required = true) @PathVariable("tagValueId") Long tagValueId,
                                   @ApiParam(value = "新值，如果不设置，则设置为null") @RequestParam(value = "newValue", required = false) String newValue) throws OpenapiException;

    @ApiOperation(value = "修改标签名称和标签值",  notes = "同时修改标签名称和值")
    @RequestMapping(value = "/tag/{tagValueId}", method = RequestMethod.PUT)
    void updateTag(@ApiParam(value = "标签值ID", required = true) @PathVariable("tagValueId") Long tagValueId,
                              @ApiParam(value = "新名称", required = true) @RequestParam("newName") String newName,
                              @ApiParam(value = "新值，如果不设置，则设置为null") @RequestParam(value = "newValue", required = false) String newValue) throws OpenapiException;

    @ApiOperation(value = "删除标签",  notes = "删除标签")
    @RequestMapping(value = "/tag/{tagId}", method = RequestMethod.DELETE)
    void deleteTag(@ApiParam(value = "标签名称ID", required = true) @PathVariable("tagId") Long tagId) throws OpenapiException;

    @ApiOperation(value = "新增标签", response = TagDetailResult.class, notes = "创建标签,返回创建的标签（如果已经存在，则返回已有标签）")
    @RequestMapping(value = "/tag", method = RequestMethod.POST)
    TagDetailResult createTag(@ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
                              @ApiParam(value = "标签名，如果包含':'字符，则会解析成'标签名称:标签值'的映射；如果没有包含':'字符，则会认为值为null", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "查询标签下的主机详情列表", response = HostDetailListResult.class, notes = "查询特定标签下的主机详情列表")
    @RequestMapping(value = "/tag/{tagId}/hostDetailList", method = RequestMethod.GET)
    HostDetailListResult getHostDetailListByTag(@ApiParam(value = "标签名称ID", required = true) @PathVariable("tagId") Long tagId) throws OpenapiException;

    @ApiOperation(value = "使用标签值查询标签下的主机详情列表", response = HostDetailListResult.class, notes = "查询特定标签值的主机详情列表")
    @RequestMapping(value = "/tagValue/{tagValueId}/hostDetailList", method = RequestMethod.GET)
    HostDetailListResult getHostDetailListByTagValue(@ApiParam(value = "标签值名称ID", required = true) @PathVariable("tagValueId") Long tagValueId) throws OpenapiException;

    @ApiOperation(value = "查询标签下的主机ID列表", response = GetHostBriefResult.class, notes = "查询特定标签下主机ID列表")
    @RequestMapping(value = "/tag/{tagId}/hostIdList", method = RequestMethod.GET)
    GetHostBriefResult getHostIdListByTag(@ApiParam(value = "标签名称ID", required = true) @PathVariable("tagId") Long tagId) throws OpenapiException;

    @ApiOperation(value = "使用标签值查询标签下的主机ID列表", response = GetHostBriefResult.class, notes = "查询特定标签值主机ID列表")
    @RequestMapping(value = "/tagValue/{tagValueId}/hostIdList", method = RequestMethod.GET)
    GetHostBriefResult getHostIdListByTagValue(@ApiParam(value = "标签值名称ID", required = true) @PathVariable("tagValueId") Long tagValueId) throws OpenapiException;

    @ApiOperation(value = "按照标签名给主机打标签", response = TagDetailResult.class,
            notes = "按照标签名给主机打标签，返回操作的标签详情。如果标签不存在会自动创建")
    @RequestMapping(value = "/tag/attachToHostByName/{hostIds}", method = RequestMethod.POST)
    TagDetailResult attachToHostByName(
            @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "主机id列表，用','连接", required = true) @PathVariable("hostIds") String idStr,
            @ApiParam(value = "标签名称", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "根据标签名称删除主机的某个标签",  notes = "根据标签名称删除主机的某个标签")
    @RequestMapping(value = "/tag/detachFromHostByName/{hostIds}", method = RequestMethod.DELETE)
    void detachFromHostByName(
            @ApiParam(value = "所属团队ID", required = true) @RequestParam("teamId") Long teamId,
            @ApiParam(value = "主机id列表，用','连接", required = true) @PathVariable("hostIds") String idStr,
            @ApiParam(value = "标签名称，如果包含值，则用':'连接", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "查询主机的标签列表", response = ListTagResult.class, notes = "查询主机的标签列表")
    @RequestMapping(value = "/tag/listByHost", method = RequestMethod.GET)
    ListTagResult listHostTags(@ApiParam(value = "主机ID", required = true) @RequestParam("hostId") Long hostId) throws OpenapiException;
}
