package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.NoneSAAS;
import com.cloudbility.openapi.bind.Permissions;
import com.cloudbility.openapi.request.ImportUserRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;

@Api(tags = "1 user api", description = "用户操作API", position = 1)
public interface UserApi extends Openapi {
    @ApiOperation(value = "导入单个用户", response = ImportUserResult.class)
    @RequestMapping(value = "/userV2", method = RequestMethod.POST)
    @ApiResponses(value = {
            @ApiResponse(code = 201, message = "User created successfully"),
            @ApiResponse(code = 409, message = "account or email conflict"),
    })
    @Permissions(Permission.UserAccess)
    ImportUserResult importUser(
            @ApiParam(value = "用户昵称，必填。", required = true) @RequestParam("name") String name,
            @ApiParam(value = "用户签名，可选。用户自定义描述") @RequestParam(value = "introduce", required = false) String introduce,
            @ApiParam(value = "性别，可选。0:未知；1,：男；2：女。不填时默认值为0。") @RequestParam(value = "gender", required = false, defaultValue = "0") int gender,
            @ApiParam(value = "地址，可选。") @RequestParam(value = "address", required = false) String address,
            @ApiParam(value = "首次登录强制修改密码，可选。默认为false。4.8新增属性。", defaultValue = "false") @RequestParam(value = "forceUpdate", required = false, defaultValue = "false") boolean forceUpdate,
            @ApiParam(value = "登录账户，可选。需保证全局唯一,未设置时，将由系统自动生成。4.8新增属性，只能在私有部署版中设置，SAAS版的Openapi不支持设置登录账户。账号不能包含逗号','。") @RequestParam(value = "account", required = false) String account,
            @ApiParam(value = "登录密码，可选。长度4~40个字符，不校验复杂性。4.8新增属性。") @RequestParam(value = "password", required = false) String password,
            @ApiParam(value = "默认加入的团队，可选。团队ID。4.8新增属性。") @RequestParam(value = "joinTeam", required = false) Long joinTeam,
            @ApiParam(value = "默认加入的团队，可选。团队code，4.10新加字段，与joinTeam不要同时设置，如果设置了joinTeamCode，会忽略joinTeam。") @RequestParam(value = "joinTeamCode", required = false) String joinTeamCode,
            @ApiParam(value = "在加入团队中默认授权的角色，可选。必须设置joinTeam或者joinTeamCode才可以设置roles。格式：用逗号','连接的角色ID或者code，code可以设定私有部署环境role_templates.json定义的系统角色。") @RequestParam(value = "roles", required = false) String roles,
            @ApiParam(value = "用户在团队中的有效期，可选。如果设置，会在到期后自动退出团队，如果不设置，则是永久成员。日期格式：yyyy-MM-dd。4.8新增属性。") @RequestParam(value = "expirationDate", required = false) String expirationDate,
            @ApiParam(value = "登录邮箱，可选。需保证全局唯一。4.9新增属性，只能在私有部署版中设置，SAAS用户的邮箱需要通过验证码绑定。") @RequestParam(value = "email", required = false) String email,
            @ApiParam(value = "是否发送通知邮件，可选。默认为false，只有设置了email才会生效。4.11新增属性。") @RequestParam(value = "sendEmail", required = false) boolean sendEmail
    ) throws OpenapiException;

    @ApiOperation(value = "导入用户", response = ImportUserResult.class, notes = "设置joinTeam或joinTeamCode可以自动加入新建用户到团队；参数roles用逗号连接多个角色id，如果是系统角色，可以用RoleCode。例如：roles='12,TEAM_ADMINER'表示id=12的角色和团队管理员角色。")
    @RequestMapping(value = "/user", method = RequestMethod.POST)
    @Permissions(Permission.UserAccess)
    @ApiResponses(value = {
            @ApiResponse(code = 201, message = "User created successfully"),
            @ApiResponse(code = 409, message = "account or email conflict"),
    })
    ImportUserResult importUsers(
            @ApiParam(value = "用户列表", required = true) @RequestBody ImportUserRequest request) throws Exception;

    @ApiOperation(value = "修改用户信息", notes = "修改用户的昵称、签名、性别、地址")
    @RequestMapping(value = "/user/{id}", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUser(
            @ApiParam(value = "用户ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "用户信息", required = true) @RequestBody UpdateUserVo userVo) throws Exception;

    @ApiOperation(value = "修改用户信息（根据账号）", notes = "修改用户的昵称、签名、性别、地址")
    @RequestMapping(value = "/user/byAccount/{account:.+}", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserByAccount(
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account,
            @ApiParam(value = "用户信息", required = true) @RequestBody UpdateUserVo userVo) throws OpenapiException;

    @ApiOperation(value = "修改用户登录密码", notes = "这个操作不校验密码复杂性，不保存改密记录")
    @RequestMapping(value = "/user/{id}/password", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserPassword(
            @ApiParam(value = "用户ID", required = true) @PathVariable("id") Long id, @ApiParam(value = "登录密码，长度4~40个字符，不校验复杂性", required = true) @RequestParam(value = "password") String password,
            @ApiParam(value = "是否发送通知邮件，可选。默认为false，只有用户设置了email才会生效。4.11新增属性。") @RequestParam(value = "sendEmail", required = false) boolean sendEmail
    ) throws Exception;

    @ApiOperation(value = "修改用户登录密码", notes = "这个操作不校验密码复杂性，不保存改密记录")
    @RequestMapping(value = "/user/byAccount/{account}/password", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserPasswordByAccount(
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account, @ApiParam(value = "登录密码，长度4~40个字符，不校验复杂性", required = true) @RequestParam(value = "password") String password,
            @ApiParam(value = "是否发送通知邮件，可选。默认为false，只有用户设置了email才会生效。4.11新增属性。") @RequestParam(value = "sendEmail", required = false) boolean sendEmail,
            @ApiParam(value = "密码是否是加密字串，可选。默认为false", required = false) @RequestParam(value = "encryptSensitive", required = false) boolean encryptSensitive
    ) throws Exception;

    @ApiOperation(value = "锁定用户", notes = "仅在私有部署版使用。锁定以后，无法登陆")
    @RequestMapping(value = "/user/{id}/blocked", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    @NoneSAAS
    void blockUser(
            @ApiParam(value = "用户ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "锁定时间，单位秒，缺省为0，为0时永久锁定", required = false) @RequestParam(value = "blockingSeconds", required = false, defaultValue = "0") Integer blockingSeconds,
            @ApiParam(value = "锁定原因", required = false) @RequestParam(value = "description", required = false) String description
    ) throws OpenapiException;

    @ApiOperation(value = "锁定用户（按账号）", notes = "仅在私有部署版使用。锁定以后，无法登陆")
    @RequestMapping(value = "/user/byAccount/{account}/blocked", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    @NoneSAAS
    void blockUserByAccount(
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account,
            @ApiParam(value = "锁定时间，单位秒，为0时永久锁定", required = true) @RequestParam(value = "blockingSeconds", defaultValue = "0") Integer blockingSeconds,
            @ApiParam(value = "锁定原因", required = false) @RequestParam(value = "description", required = false) String description
    ) throws OpenapiException;

    @ApiOperation(value = "解除用户锁定", notes = "仅在私有部署版使用")
    @RequestMapping(value = "/user/{id}/unblocked", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    @NoneSAAS
    void unblockUser(
            @ApiParam(value = "用户ID", required = true) @PathVariable("id") Long id
    ) throws OpenapiException;

    @ApiOperation(value = "解除用户锁定", notes = "仅在私有部署版使用")
    @RequestMapping(value = "/user/byAccount/{account}/unblocked", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    @NoneSAAS
    void unblockUserByAccount(
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account
    ) throws OpenapiException;

    @ApiOperation(value = "修改或删除用户邮箱", notes = "这个方法仅在私有部署环境可以使用。SAAS环境的邮箱必须通过验证码绑定。")
    @RequestMapping(value = "/user/{id}/email", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserEmail(
            @ApiParam(value = "用户ID或者账号", required = true) @PathVariable("id") Long id, @ApiParam(value = "邮箱，不能跟系统中的已有的邮箱重复。不设置或者为空串时，删除用户的邮箱", required = false) @RequestParam(value = "email", required = false) String email) throws OpenapiException;

    @ApiOperation(value = "修改或删除用户邮箱（根据账号)", notes = "这个方法仅在私有部署环境可以使用。SAAS环境的邮箱必须通过验证码绑定。")
    @RequestMapping(value = "/user/byAccount/{account}/email", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserEmailByAccount(
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account, @ApiParam(value = "邮箱，不能跟系统中的已有的邮箱重复。不设置或者为空串时，删除用户的邮箱", required = false) @RequestParam(value = "email", required = false) String email) throws OpenapiException;

    @ApiOperation(value = "修改或清理用户登录账号", notes = "这个方法仅在私有部署环境可以使用。设置成随机账号以后，不能再用原来的账号登陆。但是用户的手机和邮箱（如果设置了）还是可以登录的。")
    @RequestMapping(value = "/user/{id}/account", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserAccount(
            @ApiParam(value = "用户ID或者账号", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "账号，不能与现有系统中的账号重复。设置为空或者不设置时，系统生成随机账号。", required = false) @RequestParam(value = "account", required = false) String account) throws OpenapiException;

    @ApiOperation(value = "修改或清理用户登录账号（根据账号)", notes = "这个方法仅在私有部署环境可以使用。设置成随机账号以后，不能再用原来的账号登陆。但是用户的手机和邮箱（如果设置了）还是可以登录的。")
    @RequestMapping(value = "/user/byAccount/{account}/account", method = RequestMethod.PUT)
    @Permissions(Permission.UserAccess)
    void updateUserAccountByAccount(
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account,
            @ApiParam(value = "账号，不能与现有系统重复。设置为空或者不设置时，系统生成随机账号。", required = false) @RequestParam(value = "newAccount", required = false) String newAccount) throws OpenapiException;

    @ApiOperation(value = "批量删除用户", notes = "注意：无法删除团队创建者")
    @RequestMapping(value = "/user/{ids}", method = RequestMethod.DELETE)
    @Permissions(Permission.UserAccess)
    void deleteUsers(@ApiParam(value = "用户id列表，用','连接", required = true) @PathVariable("ids") String ids) throws OpenapiException;

    @ApiOperation(value = "删除一个用户（根据账号）", notes = "注意：无法删除团队创建者")
    @RequestMapping(value = "/user/byAccount/{account:.+}", method = RequestMethod.DELETE)
    @Permissions(Permission.UserAccess)
    void deleteUserByAccounts(@ApiParam(value = "用户账号", required = true) @PathVariable("account") String account) throws OpenapiException;

    @ApiOperation(value = "获取用户信息")
    @RequestMapping(value = "/user/{id}", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserVo getUser(@ApiParam(value = "用户id或者账号", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "获取用户信息（根据账号）")
    @RequestMapping(value = "/user/byAccount/{account:.+}", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserVo getUserByAccount(@ApiParam(value = "用户账号", required = true) @PathVariable("account") String account) throws OpenapiException;

    @ApiOperation(value = "获取用户信息（根据手机号）")
    @RequestMapping(value = "/user/byPhone/{phone:.+}", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserVo getUserByPhone(

            @ApiParam(value = "用户账号", required = true) @PathVariable("phone") String phone) throws OpenapiException;

    @ApiOperation(value = "获取用户信息（根据昵称）")
    @RequestMapping(value = "/user/byNickname", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserListResult getUserByNickname(@ApiParam(value = "用户昵称", required = true) @RequestParam("nickname") String nickname) throws OpenapiException;

    @ApiOperation(value = "获取用户信息（根据邮箱）")
    @RequestMapping(value = "/user/byEmail/{email:.+}", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserVo getUserByEmail(

            @ApiParam(value = "用户账号", required = true) @PathVariable("email") String email) throws OpenapiException;

    @ApiOperation(value = "获取所有用户ID", response = UserBriefResult.class)
    @RequestMapping(value = "/user/findIds", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserBriefResult findUserIds(@ApiParam(value = "团队id，teamId未设置时，将获取所有用户ID", required = false) @RequestParam(value = "teamId", required = false) Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取所有用户ID（根据团队code）", response = UserBriefResult.class)
    @RequestMapping(value = "/user/ids/byTeamCode/{teamCode:.+}", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserBriefResult findUserIdsByTeamCode(@ApiParam(value = "团队code", required = true) @PathVariable(value = "teamCode") String teamCode) throws OpenapiException;

    @ApiOperation(value = "分页获取团队成员")
    @RequestMapping(value = "/user/byTeam", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    PagedUserListResult findMembersByTeam(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId") Long teamId,
                                          @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                          @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "分页获取用户信息", response = PagedUserListResult.class)
    @RequestMapping(value = "/users", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    PagedUserListResult getUsers(@ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                 @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "获取用户在团队中的权限列表")
    @RequestMapping(value = "/functions/byUser", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    GetFunctionResult getFunctionsByUser(@ApiParam(value = "用户ID", required = true) @RequestParam("userId") long userId,
                                         @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") long teamId,
                                         @ApiParam(value = "模块", required = false) @RequestParam(value = "module", required = false) String module,
                                         @ApiParam(value = "校验类型", required = false) @RequestParam(value = "type", required = false) FunctionType type) throws OpenapiException;

    @ApiOperation(value = "获取用户在团队中的权限列表")
    @RequestMapping(value = "/functions/byJwt", method = RequestMethod.GET)
    GetFunctionResult getFunctionsByJwt(@ApiParam(value = "团队ID", required = true) @RequestParam("teamId") long teamId,
                                        @ApiParam(value = "用户jwt", required = true) @RequestParam("jwt") String jwt,
                                        @ApiParam(value = "模块", required = false) @RequestParam(value = "module", required = false) String module,
                                        @ApiParam(value = "校验类型", required = false) @RequestParam(value = "type", required = false) FunctionType type) throws OpenapiException;

    @ApiOperation(value = "根据昵称查询用户", notes = "根据用户昵称查询精准匹配用户")
    @RequestMapping(value = "/users/byName", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserListResult getUsersByNickname(@ApiParam(value = "用户昵称") @RequestParam(value = "name") String name) throws OpenapiException;

    @ApiOperation(value = "根据昵称查询用户", notes = "查询用户昵称包含name字符串的用户")
    @RequestMapping(value = "/users/byNameLike", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserListResult getUsersByNicknameLike(@ApiParam(value = "用户昵称包含的字符串") @RequestParam(value = "name") String name) throws OpenapiException;

    @ApiOperation(value = "根据创建时间查询用户", notes = "按照用户的创建时间为[fromTime,toTime)来查询")
    @RequestMapping(value = "/users/byCreateTime", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserListResult getUsersByCreateTime(
            @ApiParam(value = "用户创建时间开始。UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = true) @RequestParam(value = "fromTime") String fromTimeStr,
            @ApiParam(value = "用户创建时间截止。UTC时间，格式\"yyyy-MM-dd'T'HH:mm:ss'Z'\"", required = true) @RequestParam(value = "toTime") String toTimeStr
    ) throws OpenapiException;

    @ApiOperation(value = "查询用户列表（按游标：用户ID）", notes = "用户ID>cursorId的规则进行查询")
    @RequestMapping(value = "/users/byCursor", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserListResult getUsersByCursor(@ApiParam(value = "游标，即用户ID值的下限", required = true) @RequestParam(value = "cursorId", defaultValue = "0") long cursorId,
                                    @ApiParam("请求记录数，默认20") @RequestParam(value = "size", required = false, defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "获取用户加入的团队列表")
    @RequestMapping(value = "/user/{id}/joinedTeams", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    TeamListResult findJoinedTeam(@ApiParam(value = "用户ID", required = true) @PathVariable(value = "id") Long id) throws OpenapiException;

    @ApiOperation(value = "查询用户加入的团队列表")
    @RequestMapping(value = "/user/byAccount/{account}/joinedTeams", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    TeamListResult findJoinedTeamByAccount(@ApiParam(value = "用户账户", required = true) @PathVariable(value = "account") String account) throws OpenapiException;

    @ApiOperation(value = "查询用户授予的角色列表")
    @RequestMapping(value = "/user/{id}/joinedRoles", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    RoleListResult findJoinedRoles(@ApiParam(value = "用户ID", required = true) @PathVariable(value = "id") Long id,
                                   @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId
    ) throws OpenapiException;

    @ApiOperation(value = "查询用户授予的角色列表（按账户）")
    @RequestMapping(value = "/user/byAccount/{account}/joinedRoles", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    RoleListResult findJoinedRolesByAccount(@ApiParam(value = "用户账户", required = true) @PathVariable(value = "account") String account,
                                            @ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "获取系统在线用户数")
    @RequestMapping(value = "/users/countOnline", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    CountResult countUserOnline();

    @ApiOperation(value = "身份证实名认证")
    @RequestMapping(value = "/user/verifyIdCard/{realName}/{idCard}", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    void verifyIdCard(
            @ApiParam(value = "用户的真实姓名", required = true) @PathVariable("realName") String userName,
            @ApiParam(value = "身份证号码", required = true) @PathVariable("idCard") String idCard) throws OpenapiException;

    @ApiOperation(value = "获取团队在线成员数")
    @RequestMapping(value = "/users/countOnlineMembers", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    CountResult countTeamMemberOnline(@ApiParam(value = "团队ID", required = true) @RequestParam("teamId") Long teamId) throws OpenapiException;

    @ApiOperation(value = "创建或者更新用户扩展属性")
    @RequestMapping(value = "/user/saveOrUpdateUserExtendParam", method = RequestMethod.POST)
    @Permissions(Permission.UserAccess)
    WorkTaskVo.UserExtendParamVo saveOrUpdateUserExtendParam(@ApiParam(value = "用户ID", required = true) @RequestParam(value = "userId", required = true) Long userId,
                                                             @ApiParam(value = "属性名", required = true) @RequestParam(value = "paramName", required = true) String paramName,
                                                             @ApiParam(value = "属性值", required = true) @RequestParam(value = "paramValue", required = true) String paramValue,
                                                             @ApiParam(value = "描述信息", required = false) @RequestParam(value = "description", required = false) String description) throws OpenapiException;

    @ApiOperation(value = "创建或者更新用户扩展属性")
    @RequestMapping(value = "/user/deleteUserExtendParam", method = RequestMethod.POST)
    @Permissions(Permission.UserAccess)
    CountResult deleteUserExtendParam(@ApiParam(value = "用户ID", required = true) @RequestParam(value = "userId", required = true) Long userId,
                                      @ApiParam(value = "属性名", required = false) @RequestParam(value = "paramName", required = false) String paramName) throws OpenapiException;

    @ApiOperation(value = "查询用户扩展属性")
    @RequestMapping(value = "/user/getUserExtendParams", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    WorkTaskVo.UserExtendParamResult getUserExtendParams(@ApiParam(value = "用户ID", required = false) @RequestParam(value = "userId", required = false) Long userId,
                                                         @ApiParam(value = "属性名", required = false) @RequestParam(value = "paramName", required = false) String paramName,
                                                         @ApiParam(value = "属性值", required = false) @RequestParam(value = "paramValue", required = false) String paramValue) throws OpenapiException;

    @ApiOperation(value = "根据扩展属性查询用户")
    @RequestMapping(value = "/user/getUserListByExtendParam", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    UserListResult getUserListByExtendParam(@ApiParam(value = "属性名", required = true) @RequestParam(value = "paramName", required = true) String paramName,
                                            @ApiParam(value = "属性值", required = true) @RequestParam(value = "paramValue", required = true) String paramValue) throws OpenapiException;

    @ApiOperation(value = "判断用户是否加在某个团队中")
    @RequestMapping(value = "/user/isUserInTeam", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    BooleanResponse isUserInTeam(@ApiParam(value = "团队ID", required = true) @RequestParam(value = "teamId", required = true) Long teamId,
                                 @ApiParam(value = "用户ID", required = true) @RequestParam(value = "userId", required = true) Long userId) throws OpenapiException;


    @ApiOperation(value = "创建用户")
    @RequestMapping(value = "/user/create", method = RequestMethod.POST)
    @Permissions(Permission.UserAccess)
    SimpleUserVo createUser(@ApiParam(value = "登录账号", required = true) @RequestParam(value = "account", required = true) String account,
                            @ApiParam(value = "用户昵称", required = false) @RequestParam(value = "nickname", required = false) String nickname,
                            @ApiParam(value = "手机号", required = false) @RequestParam(value = "phone", required = false) String phone,
                            @ApiParam(value = "邮箱", required = false) @RequestParam(value = "email", required = false) String email,
                            @ApiParam(value = "密码", required = false) @RequestParam(value = "password", required = false) String password,
                            @ApiParam(value = "第三方用户唯一标识", required = false) @RequestParam(value = "thirdPartyUserId", required = false) String thirdPartyUserId,
                            @ApiParam(value = "加入的团队标识", required = false) @RequestParam(value = "joinTeamCode", required = false) String joinTeamCode,
                            @ApiParam(value = "角色ID列表,以半角逗号分隔", required = false) @RequestParam(value = "roleIds", required = false)String roleIds,
                            @ApiParam(value = "部门ID", required = false) @RequestParam(value = "departmentId", required = false)Long departmentId,
                            @ApiParam(value = "密码是否是加密字串", required = false) @RequestParam(value = "encryptSensitive", required = false) Boolean encryptSensitive) throws OpenapiException;

    @ApiOperation(value = "根据第三方用户标识查询用户")
    @RequestMapping(value = "/user/queryByThirdPartyUserId", method = RequestMethod.POST)
    @Permissions(Permission.UserAccess)
    SimpleUserVo queryByThirdPartyUserId(@ApiParam(value = "第三方用户唯一标识", required = false) @RequestParam(value = "thirdPartyUserId", required = false) String thirdPartyUserId) throws OpenapiException;

}
