package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.enums.zstack.DeleteModeType;
import com.cloudbility.openapi.enums.zstack.UsingStateEventType;
import com.cloudbility.openapi.enums.zstack.VolumeProvisioningStrategyType;
import com.cloudbility.openapi.enums.zstack.VolumeStatusType;
import com.cloudbility.openapi.response.DataDiskListResult;
import com.cloudbility.openapi.response.DiskOfferingListResult;
import com.cloudbility.openapi.response.OpenapiResponse;
import com.cloudbility.openapi.vo.DataDiskVo;
import com.cloudbility.openapi.vo.Scope;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.zstack.sdk.DiskOfferingInventory;

import java.util.Set;

@Api(tags = "28 volume api", description = "云盘操作API")
public interface VolumeApi {

    @ApiOperation(value = "创建ZStack云盘", notes = "如果云盘名称已经存在，则直接返回云盘信息，不做创建操作", response = DataDiskVo.class)
    @RequestMapping(value = "/volume/cloud/{cloudId}/zstackCreate", method = RequestMethod.POST)
    ResponseEntity<DataDiskVo> createZStackVolume(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                  @ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                  @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                  @ApiParam(value = "云盘名称", required = true) @RequestParam(value = "name") String name,
                                                  @ApiParam(value = "云盘规格Id", required = true) @RequestParam(value = "diskOfferingId") String diskOfferingId,
                                                  @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                  @ApiParam(value = "主存储Id", required = false) @RequestParam(value = "primaryStorageId", required = false) String primaryStorageId,
                                                  @ApiParam(value = "磁盘置备方式", required = false) @RequestParam(value = "provisioningStrategy", required = false) VolumeProvisioningStrategyType provisioningStrategy) throws OpenapiException;


    @ApiOperation(value = "查询ZStack云盘", response = DataDiskListResult.class)
    @RequestMapping(value = "/volume/cloud/{cloudId}/zstackList", method = RequestMethod.GET)
    ResponseEntity<DataDiskListResult> listZStackVolumes(@ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                         @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                         @ApiParam(value = "主机Id", required = false) @RequestParam(value = "instanceId", required = false) String instanceId,
                                                         @ApiParam(value = "云盘列表", required = false) @RequestParam(value = "diskIdSet", required = false) Set<String> diskIdSet,
                                                         @ApiParam(value = "云盘名称", required = false) @RequestParam(value = "name", required = false) String name,
                                                         @ApiParam(value = "磁盘状态", required = false) @RequestParam(value = "volumeStatus", required = false) VolumeStatusType volumeStatus) throws OpenapiException;

    @ApiOperation(value = "查询ZStack云盘规格列表", response = DiskOfferingListResult.class)
    @RequestMapping(value = "/volume/cloud/{cloudId}/zstackDiskOfferings", method = RequestMethod.GET)
    ResponseEntity<DiskOfferingListResult> listZStackDiskOfferings(@ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                                   @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                   @ApiParam(value = "云盘规格Id列表", required = false) @RequestParam(value = "diskOfferingIds", required = false) Set<String> diskOfferingIds,
                                                                   @ApiParam(value = "云盘规格名称", required = false) @RequestParam(value = "name", required = false) String name,
                                                                   @ApiParam(value = "云盘规格大小(Mb)", required = false) @RequestParam(value = "diskSizeInMb", required = false) Long diskSizeInMb) throws OpenapiException;

    @ApiOperation(value = "修改ZStack云盘规格状态", response = DiskOfferingListResult.class)
    @RequestMapping(value = "/volume/cloud/{cloudId}/zstackDiskOfferingsState", method = RequestMethod.PUT)
    ResponseEntity<DiskOfferingListResult> changeZStackDiskOfferings(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                     @ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                                     @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                     @ApiParam(value = "云盘规格Id列表", required = true) @RequestParam(value = "diskOfferingIds") Set<String> diskOfferingIds,
                                                                     @ApiParam(value = "云盘规格状态", required = true) @RequestParam(value = "stateEvent") UsingStateEventType stateEvent) throws OpenapiException;

    @ApiOperation(value = "删除ZStack云盘规格", response = OpenapiResponse.class)
    @RequestMapping(value = "/volume/cloud/{cloudId}/zstackDiskOfferings", method = RequestMethod.DELETE)
    ResponseEntity<OpenapiResponse> deleteZStackDiskOfferings(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                              @ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                              @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                              @ApiParam(value = "云盘规格Id列表", required = true) @RequestParam(value = "diskOfferingIds") Set<String> diskOfferingIds,
                                                              @ApiParam(value = "删除方式", required = true) @RequestParam(value = "deleteMode") DeleteModeType deleteMode) throws OpenapiException;


    @ApiOperation(value = "创建ZStack云盘规格", response = DiskOfferingInventory.class)
    @RequestMapping(value = "/volume/cloud/{cloudId}/zstackDiskOffering", method = RequestMethod.POST)
    ResponseEntity<DiskOfferingInventory> createZStackDiskOffering(@ApiParam(value = "保证请求幂等性", required = false) @RequestParam(value = "clientToken", required = false) String clientToken,
                                                                   @ApiParam(value = "云账户ID", required = true) @PathVariable(value = "cloudId") Long cloudId,
                                                                   @ApiParam(value = "云账户来源", required = true) @RequestParam(value = "scope", defaultValue = "Portal") Scope scope,
                                                                   @ApiParam(value = "云盘规格名称", required = true) @RequestParam(value = "name") String name,
                                                                   @ApiParam(value = "云盘规格大小(Mb)", required = true) @RequestParam(value = "diskSizeInMb") Long diskSizeInMb,
                                                                   @ApiParam(value = "描述", required = false) @RequestParam(value = "description", required = false) String description,
                                                                   @ApiParam(value = "云盘规格大小(Mb)", required = false) @RequestParam(value = "readBandwidthInMb", required = false) Long readBandwidthInMb,
                                                                   @ApiParam(value = "云盘规格大小(Mb)", required = false) @RequestParam(value = "writeBandwidthInMb", required = false) Long writeBandwidthInMb,
                                                                   @ApiParam(value = "云盘规格大小(Mb)", required = false) @RequestParam(value = "totalBandwidthInMb", required = false) Long totalBandwidthInMb) throws OpenapiException;
}
