package com.cloudbility.openapi.vo.monitor;

import java.util.Collection;

public class FuncUtil {

    public static Func fromString(String funcDesc) {
        int idx = funcDesc.indexOf("-");
        if (idx > 0) {
            String funcName = funcDesc.substring(0, idx);
            FuncName fn = FuncName.byName(funcName);
            try {
                Integer p1 = Integer.parseInt(funcDesc.substring(idx + 1));
                return new Func1(fn, p1);
            } catch (NumberFormatException e) {
                throw new IllegalArgumentException("Invalid metric function " + funcDesc);
            }
        } else {
            FuncName fn = FuncName.byName(funcDesc);
            return new Func(fn);
        }
    }

    public static String toString(Func func) {
        if (func instanceof Func1) {
            return func.getName() + "-" + ((Func1) func).getP1();
        }
        return func.getName().name();
    }

    public static Func1 create(FuncName funcName, int p) {
        if (funcName.getParamLen() < 1) {
            throw new IllegalArgumentException("func name length is 0");
        }
        return new Func1(funcName, p);
    }

    public static Func create(FuncName funcName) {
        return new Func(funcName);
    }


    /**
     * 计算平均值
     *
     * @param data
     * @return
     */
    public static double getAvg(Collection<Double> data) {
        double sum = 0;
        int count = 0;
        for (Double d : data) {
            if (d != null && !Double.isNaN(d)) {
                sum += d;
                count++;
            }
        }
        if (count > 0) {
            return sum / count;
        }
        return Double.NaN;
    }
}
